
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.*;
import teajay.bytecode.cpool.*;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class HalloWeltGenerator {
    public static void main(String[] args) throws ByteCodeException, IOException {
        ClassFile halloWelt = new ClassFile("HalloWelt", ClassAccessFlag.PUBLIC, ClassAccessFlag.SUPER);
        
        MethodInfo main = new MethodInfo("main",
                new MethodDescriptor(FieldDescriptor.getVoid(), FieldDescriptor.fromClass(String[].class)),
                halloWelt.getThis(), MethodAccessFlag.STATIC, MethodAccessFlag.PUBLIC);
        
        halloWelt.addMethod(main);
        
        JVMInstructionList lst = main.getCode().getInstructionList();
        
        lst.add(new GetStatic(FieldRefInfo.make(ClassInfo.fromClass(System.class),
                "out", FieldDescriptor.fromClass(PrintStream.class))));
        
        lst.add(new Ldc("Hallo Welt"));
        
        lst.add(new InvokeVirtual(MethodRefInfo.make(ClassInfo.fromClass(PrintStream.class),
                "println", FieldDescriptor.getVoid(), FieldDescriptor.getObject())));
        
        lst.pushInt(-1);
        
        lst.add(new InvokeStatic(MethodRefInfo.make("java/lang/System", "exit", "void", "int")));
        
        lst.add(new Return());       
        
        halloWelt.write(new File("HalloWelt.class"));
    }
}
