
import java.io.File;
import java.io.IOException;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.*;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class NestTest {
    public static void main(String... args) throws ByteCodeException, IOException, ClassNotFoundException {
        ClassFile outer = new ClassFile("OuterClass", ClassAccessFlag.SUPER, ClassAccessFlag.PUBLIC, ClassAccessFlag.FINAL);
        outer.addField(new FieldInfo("my", FieldDescriptor.getString(), FieldAccessFlag.PRIVATE, FieldAccessFlag.STATIC));
        
        MethodInfo init = new MethodInfo("<init>", new MethodDescriptor(FieldDescriptor.getVoid()), outer.getThis(), MethodAccessFlag.PUBLIC);
        outer.addMethod(init);
        CodeAttribute code = init.getCode();
        JVMInstructionList il = code.getInstructionList();
        il.add(new ALoad(0));
        il.add(new InvokeSpecial(MethodRefInfo.make("java/lang/Object", "<init>", "void")));
        il.add(new Return());
        
        MethodInfo main = new MethodInfo("main",
                new MethodDescriptor(FieldDescriptor.getVoid(), FieldDescriptor.fromClass(String[].class)), outer.getThis(),
                MethodAccessFlag.PUBLIC, MethodAccessFlag.STATIC);
        outer.addMethod(main);
        
        il = main.getCode().getInstructionList();
        
        FieldDescriptor printStream = FieldDescriptor.fromString("java/io/PrintStream");

        FieldRefInfo systemOut = new FieldRefInfo(new ClassInfo("java/lang/System"), new NameAndTypeInfo<>("out", printStream));

        FieldDescriptor voidDescr = FieldDescriptor.fromClass(void.class);
        
        MethodRefInfo printlnString = new MethodRefInfo(new ClassInfo(printStream), new NameAndTypeInfo<>("println", 
                new MethodDescriptor(voidDescr, FieldDescriptor.fromClass(String.class))));
        
        il.add(new GetStatic(systemOut));
        
        il.add(new GetStatic(FieldRefInfo.make(outer.getThis(), "my", FieldDescriptor.getString())));
        il.add(new InvokeVirtual(printlnString));
        il.add(new Return());
        
        outer.write(new File("OuterClass.class"));
    }
}
