
import java.io.DataInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class ReadPath {

    private static void read(InputStream in) throws IOException, ByteCodeException {
        try (DataInputStream din = new DataInputStream(in)) {
            ClassFile cf = ClassFile.read(din);
            System.err.println(cf.getThis().getClassName());
            System.err.println("\t" + cf.getMajorVersion().getValue() + "." + cf.getMinorVersion().getValue());
        }
    }

    public static void main(String[] args) throws IOException, ByteCodeException {
        String bootpath;
        if (args.length > 0) {
            bootpath = String.join(System.getProperty("path.separator"), args);
        } else {
            bootpath = System.getProperty("sun.boot.class.path");
        }
        StringTokenizer tok = new StringTokenizer(bootpath, System.getProperty("path.separator"));
        while (tok.hasMoreElements()) {
            File path = new File(tok.nextToken());
            if (path.isFile() && path.getName().toLowerCase().endsWith(".jar")) {
                try (ZipFile z = new ZipFile(path)) {
                    Enumeration<? extends ZipEntry> entries = z.entries();
                    while (entries.hasMoreElements()) {
                        ZipEntry entry = entries.nextElement();
                        if (entry.getName().toLowerCase().endsWith(".class")) {
                            read(z.getInputStream(entry));
                        }
                    }
                }
            } else if (path.isFile() && path.getName().toLowerCase().endsWith(".class")) {
                read(new FileInputStream(path));
            } else if (path.isDirectory()) {
                File[] files = path.listFiles(new FileFilter() {
                    @Override
                    public boolean accept(File file) {
                        return file.isFile() && file.getName().toLowerCase().endsWith(".class");
                    }
                });
                for (File file : files) {
                    read(new FileInputStream(file));
                }
                files = path.listFiles(new FileFilter() {
                    @Override
                    public boolean accept(File file) {
                        return file.isDirectory();
                    }
                });
                if (files.length > 0) {
                    String[] arg = new String[files.length];
                    for (int i = 0; i < files.length; i++) {
                        arg[i] = files[i].getAbsolutePath();
                    }
                    main(arg);
                }
            }
        }

    }
}
