package teajay.bytecode;

import teajay.bytecode.exceptions.ByteCodeException;

/**
 * The JVM base types.
 * The string value of these enum constants can be used in field descriptors to
 * represent their respective value.
 * @author Michael Kußmann <michi@outx.de>
 */
public enum BaseType {
    Void("V"),
    Boolean("Z"),
    Byte("B"),
    Short("S"),
    Char("C"),
    Int("I"),
    Float("F"),
    Double("D"),
    Long("J"),
    /**
     * Syntax: L&lt;fully qualified name&gt;;
     */
    Reference("L"),
    /**
     * Syntax: [...[ &lt; field descriptor &gt;
     */
    Array("[");
    
    private final String value;

    private BaseType(String value) {
        this.value = value;
    }
    
    public String getValue() {
        return value;
    }
    
    public char getChar() {
        return value.charAt(0);
    }
    
    public static BaseType parse(char c) throws ByteCodeException {
        switch (c) {
            case 'V':
                return Void;
            case 'B':
                return Byte;
            case 'C':
                return Char;
            case 'D':
                return Double;
            case 'F':
                return Float;
            case 'I':
                return Int;
            case 'J':
                return Long;
            case 'L':
                return Reference;
            case 'S':
                return Short;
            case 'Z':
                return Boolean;
            case '[':
                return Array;
            default:
                throw new ByteCodeException("unknown base type " + c);
        }
    }
    
    public static BaseType parseNoThrow(char c) {
        try {
            return parse(c);
        } catch (ByteCodeException ex) {
            return null;
        }
    }
}
