package teajay.bytecode;

import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * All operands to an instruction must implement this interface.
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public interface Operand extends IndexWritable {

    /**
     * The number of bytes this entity will take as an operand.
     *
     * @return length as operand
     */
    public abstract OpSize getDefaultLengthAsOperand();

    /**
     * The value of this entity as an operand.
     *
     * @param cpool the value of cpool
     * @return value of this entity as operand
     * @throws ByteCodeException if the required data to calculate the result is
     * not yet available
     */
    public abstract long getAddress(ConstantPool cpool) throws ByteCodeException;
}
