package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * signed byte
 * @author Michael Kußmann <michi@outx.de>
 */
public final class S1 implements Operand {

    public static final S1 ZERO = new S1();
    private final byte value;
    
    private S1() {
        this.value = 0;
    }

    public S1(int value) throws ByteCodeException {
        if (value > Byte.MAX_VALUE || value < Byte.MIN_VALUE) {
            throw new ByteCodeException("signed short (S1) out of bounds: " + value);
        }
        this.value = (byte) value;
    }

    public S1(long value) throws ByteCodeException {
        if (value > Byte.MAX_VALUE || value < Byte.MIN_VALUE) {
            throw new ByteCodeException("signed short (S1) out of bounds: " + value);
        }
        this.value = (byte) value;
    }

    public final byte getValue() {
        return value;
    }
    
    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeByte(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpCode.OpSize getDefaultLengthAsOperand() {
        return OpCode.OpSize.U1;
    }

    @Override
    public String toString() {
        return "S1: " + value;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof S1) {
            S1 other = (S1) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + this.value;
        return hash;
    }
}
