package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * signed short
 * @author Michael Kußmann <michi@outx.de>
 */
public final class S2 implements Operand {

    public static final S2 ZERO = new S2();
    private final short value;
    
    private S2() {
        this.value = 0;
    }

    public S2(int value) throws ByteCodeException {
        if (value > Short.MAX_VALUE || value < Short.MIN_VALUE) {
            throw new ByteCodeException("signed short (S2) out of bounds: " + value);
        }
        this.value = (short) value;
    }

    public S2(long value) throws ByteCodeException {
        if (value > Short.MAX_VALUE || value < Short.MIN_VALUE) {
            throw new ByteCodeException("signed short (S2) out of bounds: " + value);
        }
        this.value = (short) value;
    }

    public final short getValue() {
        return value;
    }
    
    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeShort(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U2;
    }

    @Override
    public String toString() {
        return "S2: " + value;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof S2) {
            S2 other = (S2) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + this.value;
        return hash;
    }
}
