package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * signed int
 * @author Michael Kußmann <michi@outx.de>
 */
public final class S4 implements Operand {

    public static final S4 ZERO = new S4();
    private final int value;

    private S4() {
        this.value = 0;
    }

    public S4(int value) {
        this.value = value;
    }

    public S4(long value) throws ByteCodeException {
        if (value < Integer.MIN_VALUE || value > Integer.MAX_VALUE) {
            throw new ByteCodeException("signed int (S4) out of bounds: " + value);
        }
        this.value = (int) value;
    }

    public final int getValue() {
        return value;
    }

    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeInt(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpCode.OpSize getDefaultLengthAsOperand() {
        return OpCode.OpSize.U4;
    }

    @Override
    public String toString() {
        return "S4: " + value;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof S4) {
            S4 other = (S4) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 17 * hash + this.value;
        return hash;
    }

}
