package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * unsigned short
 * @author Michael Kußmann <michi@outx.de>
 */
public final class U2 implements Operand {

    public static final U2 ZERO = new U2();
    private final int value;

    private U2() {
        this.value = 0;
    }

    U2(short value) {
        if (value < 0) {
            throw new IllegalArgumentException("usigned short (u2) out of bounds: " + value);
        }
        this.value = value;
    }

    public U2(int value) throws ByteCodeException {
        if (value > 0xFFFF || value < 0) {
            throw new ByteCodeException("usigned short (u2) out of bounds: " + value);
        }
        this.value = value;
    }

    public U2(long value) throws ByteCodeException {
        if (value > 0xFFFF || value < 0) {
            throw new ByteCodeException("usigned short (u2) out of bounds: " + value);
        }
        this.value = (int) value;
    }

    public final int getValue() {
        return value;
    }

    public U1 toU1() throws ByteCodeException {
        return new U1(value);
    }

    public U4 toU4() throws ByteCodeException {
        return new U4(value);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeShort(value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U2;
    }

    @Override
    public String toString() {
        return "U2: " + value;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof U2) {
            U2 other = (U2) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + this.value;
        return hash;
    }
}
