package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * unsigned int
 * @author Michael Kußmann <michi@outx.de>
 */
public final class U4 implements Operand {

    public static final U4 ZERO = new U4();
    private final long value;

    private U4() {
        this.value = 0L;
    }

    U4(Long value) {
        if (value < 0) {
            throw new IllegalArgumentException("usigned int (u4) out of bounds: " + value);
        }
        this.value = value;
    }

    public U4(int value) throws ByteCodeException {
        if (value < 0) {
            throw new ByteCodeException("usigned int (u4) out of bounds: " + value);
        }
        this.value = value;
    }

    public U4(long value) throws ByteCodeException {
        if ((value & 0xFFFFFFFF00000000L) != 0) {
            
            throw new ByteCodeException("usigned int (u4) out of bounds: " + value);
        }
        this.value = value;
    }

    public U2 toU2() throws ByteCodeException {
        return new U2(value);
    }

    public U1 toU1() throws ByteCodeException {
        return new U1(value);
    }

    public final long getValue() {
        return value;
    }

    @Override
    public long getAddress(ConstantPool cpool) {
        return value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeInt((int) value);
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        this.write(out, cpool);
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U4;
    }

    @Override
    public String toString() {
        return "U4: " + value;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof U4) {
            U4 other = (U4) obj;
            return other.value == value;
        }
        return false;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 79 * hash + (int) (this.value ^ (this.value >>> 32));
        return hash;
    }
}
