package teajay.bytecode;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class VersionConstants {
    public static enum JavaMajorVersion implements ByteCodeWritable {
        Java11_0(55),
        Java10_0(54),
        Java9_0(53),
        Java8_0(52),
        Java7_0(51),
        Java6_0(50),
        Java5_0(49),
        Java1_4(48),
        Java1_3(47),
        Java1_2(46),
        Java1_1(45),
        Unknown(0);
        
        public final U2 value;
        
        private JavaMajorVersion(int major) {
            this.value = new U2( (short) major);
        }

        @Override
        public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
            value.write(out, cpool);
        }
        
        public static JavaMajorVersion getByValue(int val) {
            switch (val) {
                case 55:
                    return Java11_0;
                case 54:
                    return Java10_0;
                case 53:
                    return Java9_0;
                case 52:
                    return Java8_0;
                case 51:
                    return Java7_0;
                case 50:
                    return Java6_0;
                case 49:
                    return Java5_0;
                case 48:
                    return Java1_4;
                case 47:
                    return Java1_3;
                case 46:
                    return Java1_2;
                case 45:
                    return Java1_1;
                default:
                    return Unknown;
            }
        }
    }
    public static final U4 JAVA_MAGIC = new U4(new Long(0xCAFEBABEL));
    public static final U2 J7_MINOR = new U2((short) 0);
    public static final U2 J7_MAJOR = new U2((short) 51);
    /**
     * This is the safe code length for jvm version 7.
     */
    public static final int MAX_CODE_LENGTH = 0xFFFE;
}
