package teajay.bytecode.attributes;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.ConstantPoolAddable;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public abstract class AttributeInfo implements ByteCodeWritable, ConstantPoolAddable {
    
    private final Utf8Info attributeName;
    
    protected AttributeInfo(Utf8Info attributeName) {
        this.attributeName = attributeName;
    }
    
    public Utf8Info getName() {
        return attributeName;
    }
    /**
     * Returns the length in bytes, that this attribute will consume.
     * @param cpool the value of cpool
     * @return
     * @throws ByteCodeException 
     */
    public abstract U4 getLength(ConstantPool cpool) throws ByteCodeException;
    
    @Override
    /**
     * When overriding this method call make a super-call before writing own data.
     */
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        attributeName.writeIndex(out, cpool);
        this.getLength(cpool).write(out, cpool);
    }
    
    @Override
    /**
     * When overriding this method keep in mind to make a super-call eventually.
     */
    public void add(ConstantPool cpool) throws ByteCodeException {
        attributeName.add(cpool);
    }
    
    @Override
    public String toString() {
        return "Attribute: " + attributeName.getValue();
    }
    
    protected static void checkName(String target, Utf8Info check) {
        if (check.getValue() == null || !check.getValue().equals(target)) {
            throw new Error(target + " (internal): invalid name argument " + check);
        }
    }
}
