package teajay.bytecode.attributes;

import java.io.DataInputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * Implementation of the Synthetic attribute as stated in "The JavaTM Virtual
 * Machine Specification Version 7". Revision: 27.07.2012
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class Synthetic extends AttributeInfo {
    
    public static final String NAME = "Synthetic";
    
    public Synthetic() {
        this(new Utf8Info(NAME));
    }
    
    private Synthetic(Utf8Info name) {
        super(name);
        checkName(NAME, name);
    }
    
    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return U4.ZERO;
    }
    
    public static Synthetic read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        return new Synthetic(name);
    }
}
