package teajay.bytecode.attributes;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * All unknown attributes that are encountered by the <tt>AttributeReader</tt>
 * become an instance of this class.
 * @author Michael Kußmann <michi@outx.de>
 */
public final class UnknownAttribute extends AttributeInfo {
    private final byte[] data;
    
    UnknownAttribute(Utf8Info name, byte[] data) throws ByteCodeException {
        super(name);
        this.data = data;
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(data.length);
    }
    
    public byte[] getData() {
        return data;
    }
    
    public DataInputStream getStream() {
        return new DataInputStream(new ByteArrayInputStream(data));
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        out.write(data);
    }    
}
