package teajay.bytecode.attributes.annotations;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.attributes.annotations.ElementValue.EnumDescr;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.DoubleInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FloatInfo;
import teajay.bytecode.cpool.IntegerInfo;
import teajay.bytecode.cpool.LongInfo;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum AnnotationElementType implements ByteCodeWritable {

    Byte('B', Byte.class, IntegerInfo.class),
    Char('C', Character.class, IntegerInfo.class),
    Double('D', Double.class, DoubleInfo.class),
    Float('F', Float.class, FloatInfo.class),
    Int('I', Integer.class, IntegerInfo.class),
    Long('J', Long.class, LongInfo.class),
    Short('S', Short.class, IntegerInfo.class),
    Boolean('Z', Boolean.class, IntegerInfo.class),
    String('s', String.class, Utf8Info.class),
    Enum('e', EnumDescr.class, Enum.class),
    Class('c', FieldDescriptor.class),
    Annotation('@', Annotation.class),
    Array('[', List.class, Collection.class);
    private final char value;
    private final Collection<Class<?>> type;

    private AnnotationElementType(char value, Class<?>... type) {
        this.value = value;
        this.type = Arrays.asList(type);
    }

    public char getValue() {
        return value;
    }
    
    public Collection<Class<?>> getTypes() {
        return type;
    }
    
    public boolean isCorrectDataType(Class<?> type) {
        return this.type.contains(type);
    }

    public static AnnotationElementType parse(char c) throws ByteCodeException {
        switch (c) {
            case 'B':
                return Byte;
            case 'C':
                return Char;
            case 'D':
                return Double;
            case 'F':
                return Float;
            case 'I':
                return Int;
            case 'J':
                return Long;
            case 'S':
                return Short;
            case 'Z':
                return Boolean;
            case 's':
                return String;
            case 'e':
                return Enum;
            case 'c':
                return Class;
            case '@':
                return Annotation;
            case '[':
                return Array;
            default:
                throw new ByteCodeException("unknown annotation element type " + c);
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        out.writeByte(value);
    }
}
