package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class AppendFrame extends LocalVerifierStackMapFrame {

    AppendFrame(JVMInstruction start) {
        super(start, new VerificationTypeList(3));
    }

    @Override
    U4 getLength() throws ByteCodeException {
        return new U4(1 + 2 + getLocalTypeList().getLength().getValue());
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        if (getLocalTypeList().isEmpty()) {
            throw new ByteCodeException("Cannot have an empty verification type list.");
        }
        super.write(out, cpool);
        offsetDelta().write(out, cpool);
        getLocalTypeList().write(out, cpool);
    }

    @Override
    U1 getFrameType() throws ByteCodeException {
        int size = getLocalTypeList().getSize().getValue();
        return new U1(251 + size);
    }

    @Override
    List<VerificationType> getStack() {
        return Collections.emptyList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        List<VerificationType> oldLocals = this.getPredecessor().getLocals();
        List<VerificationType> appendList = getLocalTypeList().getList();
        List<VerificationType> newLocals = new ArrayList<>(oldLocals.size() + appendList.size());
        newLocals.addAll(oldLocals);
        newLocals.addAll(appendList);
        return newLocals;
    }
}
