package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * Implements the array type codes of the newarray instruction.
 * @author Michael Kußmann <michi@outx.de>
 */
public enum ArrayType implements Operand {
    BOOLEAN(4),
    CHAR(5),
    FLOAT(6),
    DOUBLE(7),
    BYTE(8),
    SHORT(9),
    INT(10),
    LONG(11);

    public final int intValue;

    private ArrayType(int value) {
        this.intValue = value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeByte(this.intValue);
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U1;
    }

    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        this.write(out, cpool);
    }

    @Override
    public long getAddress(ConstantPool cpool) throws ByteCodeException {
        return intValue;
    }
}
