package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class ChopFrame extends StackMapFrame {
    private final U1 frameType;
    
    ChopFrame(JVMInstruction start, U1 frameType) throws ByteCodeException {
        super(start);
        int val = frameType.getValue();
        if (val < 248 || val > 250) {
            throw new ByteCodeException("Invalid frame type argument for same frame: " + val);
        }
        this.frameType = frameType;
    }

    @Override
    U4 getLength() throws ByteCodeException {
        return new U4(1 + 2);
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        offsetDelta().write(out, cpool);
    }

    @Override
    U1 getFrameType() {
        return frameType;
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
    }
    
    @Override
    List<VerificationType> getStack() {
        return Collections.emptyList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        List<VerificationType> oldLocals = this.getPredecessor().getLocals();
        int numChop = 251 - frameType.getValue();
        int diff = oldLocals.size() - numChop;
        if (diff < 0) {
            throw new ByteCodeException("Invalid use of the chop frame: There are no locals to chop!");
        }
        return oldLocals.subList(0, diff);
    }
}
