package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeAddressOverflow;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class CompareInstruction extends JumpInstruction {
    private final Operand branchTarget;

    protected CompareInstruction(OpCode opcode, Operand branchTarget) {
        this.setOpcode(opcode);
        this.setOperands(branchTarget);
        this.branchTarget = branchTarget;
    }

    @Override
    boolean validate(ConstantPool cpool) throws ByteCodeException {
        long diff = branchTarget.getAddress(cpool) - getAddress(cpool);
        if (diff > Short.MAX_VALUE || diff < Short.MIN_VALUE) {
            throw new ByteCodeAddressOverflow(opcode() + ": Target address exceeds signed short range.");
        } else {
            return true;
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        opcode().write(out, cpool);
        this.diffS2(branchTarget, cpool).write(out, cpool);
    }
}
