package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.Operand;
import teajay.bytecode.U2;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.ConstantPoolAddable;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class ExceptionTableEntry implements ByteCodeWritable, ConstantPoolAddable {

    private final Operand startPC;
    private final Operand endPC;
    private final Operand handlerPC;
    private final ClassInfo catchType;

    ExceptionTableEntry(Operand startPC, Operand endPC, Operand handlerPC, ClassInfo catchType) {
        this.startPC = startPC;
        this.endPC = endPC;
        this.handlerPC = handlerPC;
        this.catchType = catchType;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        startPC.writeIndex(out, cpool);
        endPC.writeIndex(out, cpool);
        handlerPC.writeIndex(out, cpool);
        if (catchType != null) {
            catchType.writeIndex(out, cpool);
        } else {
            U2.ZERO.write(out, cpool);
        }
    }

    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        if (catchType != null) {
            catchType.add(cpool);
        }
    }
    
    public ClassInfo getCatchType() {
        return catchType;
    }

    public Operand getEndPC() {
        return endPC;
    }

    public Operand getHandlerPC() {
        return handlerPC;
    }

    public Operand getStartPC() {
        return startPC;
    }
}
