package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class FLoad extends SimpleInstruction {
    private final int n;
    /**
     * Creates a new instruction.
     * @param n The index of the local variable.
     * @throws ByteCodeException if the index is out of the bounds of the jvm7 spec.
     */
    public FLoad(int n) throws ByteCodeException {
        this.n = n;
        switch (n) {
            case 0:
                this.setOpcode(OpCode.fload_0);
                break;
            case 1:
                this.setOpcode(OpCode.fload_1);
                break;
            case 2:
                this.setOpcode(OpCode.fload_2);
                break;
            case 3:
                this.setOpcode(OpCode.fload_3);
                break;
            default:
                if (n <= 0xFF && n >= 0) {
                    this.setOpcode(OpCode.fload);
                    this.setOperands(new U1(n));
                } else if (n <= 0xFFFF && n >= 0) {
                    this.setOpcode(OpCode.wide1);
                    this.setOperands(OpCode.fload, new U2(n));
                } else {
                    throw new ByteCodeException("fload: local variable index out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return n + 1;
    }
}
