package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class Goto extends JumpInstruction {

    private final Operand branchTarget;

    public Goto(JVMInstruction instr) {
        this.branchTarget = instr;
        this.setOpcode(OpCode.vm_goto);
        this.setOperands(instr);
    }
    
    public Goto(Label label) {
        this.branchTarget = label;
        this.setOpcode(OpCode.vm_goto);
        this.setOperands(label);
    }

    @Override
     int getLength() throws ByteCodeException {
        switch (opcode()) {
            case vm_goto:
                return 3;
            case goto_w:
                return 5;
            default:
                throw new RuntimeException("Unreachable case reached.");
        }
    }

    @Override
    boolean validate(teajay.bytecode.cpool.ConstantPool cpool) throws ByteCodeException {
        long diff = branchTarget.getAddress(cpool) - this.getAddress(cpool);
        
        if (opcode() == OpCode.vm_goto && (diff > Short.MAX_VALUE || diff < Short.MIN_VALUE)) {
            this.setOpcode(OpCode.goto_w);
            return false;
        } else {
            return true;
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        OpCode opcode = opcode();
        opcode.write(out, cpool);
        switch (opcode) {
            case goto_w:
                this.diffS4(branchTarget, cpool).write(out, cpool);
                break;
            default:
                this.diffS2(branchTarget, cpool).write(out, cpool);
        }
    }
}
