package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class IConst extends SimpleInstruction {

    public IConst(int n) throws ByteCodeException {
        switch (n) {
            case -1:
                this.setOpcode(OpCode.iconst_m1);
                break;
            case 0:
                this.setOpcode(OpCode.iconst_0);
                break;
            case 1:
                this.setOpcode(OpCode.iconst_1);
                break;
            case 2:
                this.setOpcode(OpCode.iconst_2);
                break;
            case 3:
                this.setOpcode(OpCode.iconst_3);
                break;
            case 4:
                this.setOpcode(OpCode.iconst_4);
                break;
            case 5:
                this.setOpcode(OpCode.iconst_5);
                break;
            default:
                if (n >= Byte.MIN_VALUE && n <= Byte.MAX_VALUE) {
                    this.setOpcode(OpCode.bipush);
                    this.setOperands(new U1(n));
                } else if (n >= Short.MIN_VALUE && n <= Short.MAX_VALUE) {
                    this.setOpcode(OpCode.sipush);
                    this.setOperands(new U2(n));
                } else {
                    throw new ByteCodeException("iconst: value out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return 0;
    }
}
