package teajay.bytecode.attributes.code;

import teajay.bytecode.*;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 * This class implements the iinc instruction.
 * @author Michael Kußmann <michi@outx.de>
 */
public final class IInc extends SimpleInstruction {
    private final int index;
    
    /**
     * Creates a new instruction.
     * @param index The index of the local variable.
     * @param incr The amount to be added. The amount is signed and may be inside the range of a short.
     * @throws ByteCodeException if index or incr are out of bounds.
     */
    public IInc(int index, int incr) throws ByteCodeException {
        this.index = index;
        if (index <= 0xFF && incr <= Byte.MAX_VALUE && incr >= Byte.MIN_VALUE && index >= 0) {
            this.setOpcode(OpCode.iinc);
            this.setOperands(new U1(index), new S1(incr));
        } else if (index <= 0xFFFF && incr <= Short.MAX_VALUE && incr >= Short.MIN_VALUE && index >= 0) {
            this.setOpcode(OpCode.wide2);
            this.setOperands(OpCode.iinc, new U2(index), new S2(incr));
        } else {
            throw new ByteCodeException("iinc: index or increment out of bounds: (" + index + ", " + incr + ")");
        }
    }

    @Override
    int getLocalModifier() {
        return index + 1;
    }
}
