package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class IStore extends SimpleInstruction {
    private final int n;

    public IStore(int n) throws ByteCodeException {
        this.n = n;
        switch (n) {
            case 0:
                this.setOpcode(OpCode.istore_0);
                break;
            case 1:
                this.setOpcode(OpCode.istore_1);
                break;
            case 2:
                this.setOpcode(OpCode.istore_2);
                break;
            case 3:
                this.setOpcode(OpCode.istore_3);
                break;
            default:
                if (n < 0xFF) {
                    this.setOpcode(OpCode.istore);
                    this.setOperands(new U1(n));
                } else if (n < 0xFFFF) {
                    this.setOpcode(OpCode.wide1);
                    this.setOperands(OpCode.istore, new U2(n));
                } else {
                    throw new ByteCodeException("istore: local variable index out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return n + 1;
    }
}
