package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.OpCode;
import teajay.bytecode.Operand;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.InvokeDynamicInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class InvokeDynamic extends JVMInstruction {
    private final InvokeDynamicInfo info;
    private final int push;
    private final int pop;
    
    public InvokeDynamic(InvokeDynamicInfo info) {
        this.info = info;
        MethodDescriptor md = info.getNameAndType().getDescriptor();
        FieldDescriptor returnType = md.getReturnType();
        switch (returnType.getBaseType()) {
            case Long:
            case Double:
                push = 2;
                break;
            case Void:
                push = 0;
                break;
            default:
                push = 1;
        }
        List<FieldDescriptor> params = md.getParameters();
        int popCount = params.size();
        for (FieldDescriptor param : params) {
            switch (param.getBaseType()) {
                case Long:
                case Double:
                    popCount++;
            }
        }
        this.pop = popCount;
    }

    @Override
    int getLength() throws ByteCodeException {
        return 1 + 2 + 2;
    }

    @Override
    public OpCode opcode() throws ByteCodeException {
        return OpCode.invokedynamic;
    }

    @Override
    protected Collection<? extends Operand> operands() {
        return Arrays.asList(info);
    }

    @Override
    int getLocalModifier() {
        return 0;
    }

    @Override
    int getPush() throws ByteCodeException {
        return push;
    }

    @Override
    int getPop() throws ByteCodeException {
        return pop;
    }

    @Override
    boolean validate(ConstantPool cpool) throws ByteCodeException {
        return true;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        OpCode.invokedynamic.write(out, cpool);
        info.writeIndex(out, cpool);
        out.writeShort(0);
    }
    
    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        info.add(cpool);
    }
}
