package teajay.bytecode.attributes.code;

import java.util.List;
import teajay.bytecode.OpCode;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class InvokeInstruction extends SimpleInstruction {
    private final List<FieldDescriptor> params;
    private final FieldDescriptor returnType;
    private final boolean popsRef;
    
    protected InvokeInstruction(MethodRefInfo method, OpCode opcode) {
        MethodDescriptor descr = method.getNameAndType().getDescriptor();
        this.params = descr.getParameters();
        this.returnType = descr.getReturnType();
        switch (opcode) {
            case invokestatic:
                this.popsRef = false;
                break;
            default:
                this.popsRef = true;
        }
        this.setOpcode(opcode);
    }

    @Override
    int getPush() {
        return calcPush(returnType);
    }

    @Override
    int getPop() {
        return popsRef ? calcPop(params) + 1 : calcPop(params);
    }
    
    @Override
    int getLocalModifier() {
        return 0;
    }

    private static int calcPop(List<FieldDescriptor> fields) {
        int count = fields.size();
        for (FieldDescriptor fd : fields) {
            if (fd.isLongOrDouble()) {
                count++;
            }
        }
        return count;
    }
    
    private static int calcPush(FieldDescriptor type) {
        if (type.isVoid()) {
            return 0;
        } else if (type.isLongOrDouble()) {
            return 2;
        } else {
            return 1;
        }
    }
}
