package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class LLoad extends SimpleInstruction {

    private final int n;

    public LLoad(int n) throws ByteCodeException {
        this.n = n;
        switch (n) {
            case 0:
                this.setOpcode(OpCode.lload_0);
                break;
            case 1:
                this.setOpcode(OpCode.lload_1);
                break;
            case 2:
                this.setOpcode(OpCode.lload_2);
                break;
            case 3:
                this.setOpcode(OpCode.lload_3);
                break;
            default:
                if (n <= 0xFF && n >= 0) {
                    this.setOpcode(OpCode.lload);
                    this.setOperands(new U1(n));
                } else if (n <= 0xFFFF && n >= 0) {
                    this.setOpcode(OpCode.wide1);
                    this.setOperands(OpCode.lload, new U2(n));
                } else {
                    throw new ByteCodeException("lload: index out of bounds: " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return n + 2;
    }
}
