package teajay.bytecode.attributes.code;

import teajay.bytecode.OpCode;
import teajay.bytecode.U1;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class LStore extends SimpleInstruction {

    private final int n;

    public LStore(int n) throws ByteCodeException {
        this.n = n;
        switch (n) {
            case 0:
                this.setOpcode(OpCode.lstore_0);
                break;
            case 1:
                this.setOpcode(OpCode.lstore_1);
                break;
            case 2:
                this.setOpcode(OpCode.lstore_2);
                break;
            case 3:
                this.setOpcode(OpCode.lstore_3);
                break;
            default:
                if (n <= 0xFF && n >= 0) {
                    this.setOpcode(OpCode.lstore);
                    this.setOperands(new U1(n));
                } else if (n <= 0xFFFF && n >= 0) {
                    this.setOpcode(OpCode.wide1);
                    this.setOperands(OpCode.lstore, new U2(n));
                } else {
                    throw new ByteCodeException("lstore: local variable index out of bounds " + n);
                }
        }
    }

    @Override
    int getLocalModifier() {
        return n + 2;
    }
}
