package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
final class SameLocalsOneStackItemExtendedFrame extends StackVerifierStackMapFrame {
    
    static final int FRAME_TYPE = 247;
    
    SameLocalsOneStackItemExtendedFrame(JVMInstruction start) {
        super(start, new VerificationTypeList(1));
    }
    
    SameLocalsOneStackItemExtendedFrame(JVMInstruction start, VerificationType type) throws ByteCodeException {
        super(start, new VerificationTypeList(1));
        this.getStackTypeList().add(type);
    }    

    @Override
    U1 getFrameType() throws ByteCodeException {
        return new U1(247);
    }

    @Override
    U4 getLength() throws ByteCodeException {
        return new U4(1 + 2 + this.getStackTypeList().getLength().getValue());
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        if (getStackTypeList().isEmpty()) {
            throw new ByteCodeException("Cannot have an empty verification type list.");
        }
        super.write(out, cpool);
        offsetDelta().write(out, cpool);
        getStackTypeList().write(out, cpool);
    }

    @Override
    List<VerificationType> getStack() {
        return getStackTypeList().getList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        return this.getPredecessor().getLocals();
    }
}
