package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.exceptions.FrameOffsetDeltaOutOfBounds;

/**
 * This frame is limited to a maximum offset delta of 63.
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class SameLocalsOneStackItemFrame extends StackVerifierStackMapFrame {

    private SameLocalsOneStackItemExtendedFrame extended = null;

    SameLocalsOneStackItemFrame(JVMInstruction start) {
        super(start, new VerificationTypeList(1));
    }

    @Override
    U1 getFrameType() throws ByteCodeException, FrameOffsetDeltaOutOfBounds {
        int offsetDelta = getOffsetDelta();
        if (offsetDelta < 0) {
            throw new FrameOffsetDeltaOutOfBounds("SameLocalsOneStackItemFrame: offset delta out of bounds: " + offsetDelta);
        } else if (offsetDelta > 63) {
            return new U1(SameLocalsOneStackItemExtendedFrame.FRAME_TYPE);
        } else {
            return new U1(offsetDelta + 64);
        }
    }

    @Override
    U4 getLength() throws ByteCodeException {
        if (extended == null) {
            return new U4(1 + this.getStackTypeList().getLength().getValue());
        } else {
            return extended.getLength();
        }
    }

    @Override
    void validate() throws ByteCodeException {
        int offsetDelta = getOffsetDelta();
        VerificationTypeList lst = getStackTypeList();
        if (offsetDelta > 63) {
            if (!lst.isEmpty()) {
                if (extended == null) {
                    extended = new SameLocalsOneStackItemExtendedFrame(getStart(), lst.getList().get(0));
                }
                extended.setOffsetDelta(offsetDelta);
            } else {
                throw new ByteCodeException("Cannot have an empty verification type list.");
            }
        } else if (lst.isEmpty()) {
            throw new ByteCodeException("Cannot have an empty verification type list.");
        } else {
            extended = null;
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        if (getStackTypeList().isEmpty()) {
            throw new ByteCodeException("Cannot have an empty verification type list.");
        }
        if (extended == null) {
            super.write(out, cpool);
            getStackTypeList().write(out, cpool);
        } else {
            extended.write(out, cpool);
        }
    }

    @Override
    List<VerificationType> getStack() {
        return getStackTypeList().getList();
    }

    @Override
    List<VerificationType> getLocals() throws ByteCodeException {
        return this.getPredecessor().getLocals();
    }
}
