package teajay.bytecode.attributes.code;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.U1;
import teajay.bytecode.U4;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum VariableInfo implements ByteCodeWritable {
    Top(0, 1),
    Integer(1, 1),
    Float(2, 1),
    Long(4, 1),
    Double(3, 1),
    Null(5, 1),
    UninitializedThis(6, 1),
    Object(7, 3),
    Uninitialized(8, 3);
    
    private final U1 value;
    private final U4 bytesize;
    
    private VariableInfo(int k, int bytesize) {
        try {
            this.value = new U1(k);
            this.bytesize = new U4(bytesize);
        } catch (ByteCodeException ex) {
            throw new Error("Something really went wrong here: " + ex.getMessage());
        }
    }
    
    public U4 getLength() {
        return bytesize;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        value.write(out, cpool);
    }
}
