package teajay.bytecode.attributes.nest;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;
import static teajay.bytecode.util.Tools.ccpt;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class NestHost extends AttributeInfo {

    public static final String NAME = "NestHost";
    
    private final ClassInfo hostClass;
    
    public NestHost(ClassInfo hostClass) {
        this(hostClass, new Utf8Info(NAME));
    }
    
    private NestHost(ClassInfo hostClass, Utf8Info name) {
        super(name);
        this.hostClass = hostClass;   
    }
    
    public ClassInfo getHostClass() {
        return hostClass;
    }

    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(2);
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        hostClass.writeIndex(out, cpool);
    }
    
    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        super.add(cpool);
        hostClass.add(cpool);
    }
    
    public static NestHost read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
         ClassInfo cl = ccpt(cpool.get(in.readUnsignedShort()), ClassInfo.class, cpool);
         NestHost ex = new NestHost(cl, name);
         return ex;
    }
}
