package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.U2;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class AbstractDynamicInfo extends CPoolEntry {
    private final U2 bootstrapIndex;
    private final NameAndTypeInfo<MethodDescriptor> nameAndType;
    
    protected AbstractDynamicInfo(CPoolTag tag, U2 bootstrapIndex, NameAndTypeInfo<MethodDescriptor> nameAndType) throws ByteCodeException {
        super(tag);
        this.bootstrapIndex = bootstrapIndex;
        this.nameAndType = nameAndType;
    }
    
    public final U2 getBootstrapIndex() {
        return this.bootstrapIndex;
    }
    
    public final NameAndTypeInfo<MethodDescriptor> getNameAndType() {
        return this.nameAndType;
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        bootstrapIndex.write(out, cpool);
        nameAndType.getCIndex(cpool).write(out, cpool);
    }
    
    @Override
    public String toString() {
        return this.getClass().getSimpleName() + " (bootstrap_index = " + bootstrapIndex + ", name_and_type = " + nameAndType + ")";
    }
    
    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof AbstractDynamicInfo) && super.equals(obj)) {
            AbstractDynamicInfo other = (AbstractDynamicInfo) obj;
            return other.bootstrapIndex.equals(bootstrapIndex) && other.getNameAndType().equals(getNameAndType());
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 51 * super.hashCode();
        hash = 89 * hash + Objects.hashCode(this.bootstrapIndex);
        hash = 89 * hash + Objects.hashCode(this.nameAndType);
        return hash;
    }
    
    @Override
    public void add(ConstantPool pool) throws ByteCodeException {
        super.add(pool);
        nameAndType.add(pool);
    }

}
