package teajay.bytecode.cpool;

import teajay.bytecode.Operand;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class CPoolIndex implements Operand {

    private final int index;

    protected CPoolIndex(int index) {
        if (index > 0xFFFF || index < 0) {
            throw new IllegalArgumentException("constant pool index out of bounds: " + index);
        }
        this.index = index;
    }

    public final int getValue() {
        return index;
    }
    
    @Override
    public final long getAddress(ConstantPool cpool) {
        return index;
    }
    
    public abstract U1Index toU1Index();
    
    public abstract U2Index toU2Index();
    
    @Override
    public String toString() {
        return this.getClass().getSimpleName() + ": " + index;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 53 * hash + this.index;
        return hash;
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof CPoolIndex) && ((CPoolIndex) obj).index == index;
    }
}
