package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 * @param <T>
 */
public abstract class CPoolRef<T extends Utf8Info> extends CPoolEntry {
    private final ClassInfo className;
    private final NameAndTypeInfo<T> nameAndType;
    
    protected CPoolRef(CPoolTag tag, ClassInfo className, NameAndTypeInfo<T> nameAndType) {
        super(tag);
        switch (tag) {
            case FieldRef:
            case MethodRef:
            case InterfaceMethodRef:
                break;
            default:
                throw new IllegalArgumentException("invalid ref type: " + tag);
        }
        this.className = className;
        this.nameAndType = nameAndType;
    }
    
    public final ClassInfo getRefClass() {
        return this.className;
    }
    
    public final NameAndTypeInfo<T> getNameAndType() {
        return this.nameAndType;
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        className.getCIndex(cpool).write(out, cpool);
        nameAndType.getCIndex(cpool).write(out, cpool);
    }
    
    @Override
    public String toString() {
        return this.getClass().getSimpleName() + " (className = " + className + ", nameAndType = " + nameAndType + ")";
    }
    
    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof CPoolRef) && super.equals(obj)) {
            CPoolRef<?> other = (CPoolRef<?>) obj;
            return other.className.equals(className) && other.nameAndType.equals(nameAndType);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 23 * super.hashCode();
        hash = 17 * hash + Objects.hashCode(this.className);
        hash = 17 * hash + Objects.hashCode(this.nameAndType);
        return hash;
    }
    
    @Override
    public void add(ConstantPool pool) throws ByteCodeException {
        super.add(pool);
        className.add(pool);
        nameAndType.add(pool);
    }
}
