package teajay.bytecode.cpool;

import java.io.DataInputStream;
import java.io.IOException;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum CPoolTag {
    Class(7),
    FieldRef(9),
    MethodRef(10),
    InterfaceMethodRef(11),
    String(8),
    Integer(3),
    Float(4),
    Long(5),
    Double(6),
    NameAndType(12),
    Utf8(1),
    MethodHandle(15),
    MethodType(16),
    Dynamic(17),
    InvokeDynamic(18),
    Module(19),
    Package(20),
    Dummy(-1);
    
    public final int intValue;
    
    private static final CPoolTag[] tags;
    
    static {
        int maxVal = -1;
        for (CPoolTag tag : CPoolTag.values()) {
            if (maxVal < tag.intValue) {
                maxVal = tag.intValue;
            }
        }
        tags = new CPoolTag[maxVal + 1];
        for (CPoolTag tag : CPoolTag.values()) {
            if (tag != Dummy) {
                tags[tag.intValue] = tag;
            }
        }
    }

    private CPoolTag(int value) {
        this.intValue = value;
    }

    static CPoolTag forValue(int value) {
        return value > 0 && value < tags.length ? tags[value] : null;
        /*if (value > 0 && value < tags.length) {
            for (CPoolTag tag : CPoolTag.values()) {
                if (value == tag.intValue) {
                    return tag;
                }
            }
            return tags[value];
        }
        return null;*/
    }

    static CPoolTag read(DataInputStream in) throws IOException, ByteCodeException {
        CPoolTag tag = forValue(in.readUnsignedByte());
        if (tag != null) {
            return tag;
        } else {
            throw new ByteCodeException("unknown constant pool tag");
        }
    }
}
