package teajay.bytecode.cpool;

import teajay.bytecode.exceptions.MalformedClassNameException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public final class InterfaceMethodRefInfo extends MethodRefInfo {
    
    public static InterfaceMethodRefInfo make(ClassInfo className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) {
        FieldDescriptor[] copy = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++) {
            copy[i] = parameterTypes[i];
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, copy)));
    }
    
    public static InterfaceMethodRefInfo make(String className, String name, FieldDescriptor returnType, FieldDescriptor... parameterTypes) throws MalformedClassNameException {
        FieldDescriptor[] copy = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++) {
            copy[i] = parameterTypes[i];
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(returnType, copy)));
    }

    public static InterfaceMethodRefInfo make(String className, String name, String returnType, String... parameterTypes) throws MalformedClassNameException {
        FieldDescriptor[] parameters = new FieldDescriptor[parameterTypes.length];
        for (int i = 0; i < parameters.length; i++) {
            parameters[i] = FieldDescriptor.fromString(parameterTypes[i]);
        }
        return new InterfaceMethodRefInfo(className, new NameAndTypeInfo<>(name, new MethodDescriptor(FieldDescriptor.fromString(returnType), parameters)));
    }

    public InterfaceMethodRefInfo(ClassInfo className, NameAndTypeInfo<MethodDescriptor> nameAndType) {
        super(CPoolTag.InterfaceMethodRef, className, nameAndType);
    }
    
    public InterfaceMethodRefInfo(String className, NameAndTypeInfo<MethodDescriptor> nameAndType) throws MalformedClassNameException {
        super(CPoolTag.InterfaceMethodRef, new ClassInfo(className), nameAndType);
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof InterfaceMethodRefInfo) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        return 59 * super.hashCode();
    }
}
