package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.OpCode.OpSize;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class U1Index extends CPoolIndex {

    public U1Index(int index) {
        super(index);
        if (index > 0xFF) {
            throw new IllegalArgumentException("u1 constant pool index out of bounds: " + index);
        }
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException {
        out.writeByte(this.getValue());
    }
    
    @Override
    public void writeIndex(DataOutputStream out, ConstantPool cpool) throws IOException {
        write(out, cpool);
    }    

    @Override
    public U1Index toU1Index() {
        return this;
    }

    @Override
    public U2Index toU2Index() {
        return new U2Index(getValue());
    }

    @Override
    public OpSize getDefaultLengthAsOperand() {
        return OpSize.U1;
    }
    
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof U1Index) && super.equals(obj);
    }

    @Override
    public int hashCode() {
        int hash = 71 * super.hashCode();
        return hash;
    }
}
