package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
abstract class U1U2Entry<T extends CPoolEntry> extends CPoolEntry {

    private final T reference;

    protected U1U2Entry(CPoolTag tag, T reference) {
        super(tag);
        this.reference = reference;
    }

    public final T getValue() {
        return reference;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        reference.getCIndex(cpool).write(out, cpool);
    }

    @Override
    public String toString() {
        return this.getClass().getSimpleName() + ": references=" + getValue();
    }

    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof U1U2Entry) && super.equals(obj)) {
            return ((U1U2Entry) obj).reference.equals(reference);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = super.hashCode();
        hash = 23 * hash + Objects.hashCode(this.reference);
        return hash;
    }

    @Override
    public void add(ConstantPool pool) throws ByteCodeException {
        super.add(pool);
        reference.add(pool);
    }
}
