package teajay.bytecode.cpool;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Objects;
import teajay.bytecode.cpool.MethodDescriptor.MParser;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.exceptions.UTFOutOfBoundsException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class Utf8Info extends CPoolEntry {

    private final String value;

    public Utf8Info(String value) throws UTFOutOfBoundsException {
        super(CPoolTag.Utf8);
        if (value.length() > 0xFFFF) {
            throw new UTFOutOfBoundsException("constant pool: utf8 string out of bounds");
        }
        this.value = value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        out.writeUTF(value);
    }

    public final String getValue() {
        return value;
    }

    @Override
    public String toString() {
        return this.getClass().getSimpleName() + ": " + value;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj != null && (obj instanceof Utf8Info) && super.equals(obj)) {
            return ((Utf8Info) obj).value.equals(value);
        } else if (obj instanceof String) {
            return ((String) obj).equals(value);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        int hash = 13 * super.hashCode();
        hash = 43 * hash + Objects.hashCode(this.value);
        return hash;
    }
    
    static Utf8Info parse(String utf8) throws UTFOutOfBoundsException {
        return parse(utf8, new MParser());
    }

    static Utf8Info parse(String utf8, MParser parser) throws UTFOutOfBoundsException {
        if (utf8.length() > 0) {
            if (utf8.charAt(0) == '(') {
                MethodDescriptor md = MethodDescriptor.parseMethodDescriptor(utf8, parser);
                if (md != null) {
                    return md;
                }
            } else {
                FieldDescriptor fd = FieldDescriptor.parseFieldDescriptor(utf8);
                if (fd != null) {
                    return fd;
                }
            }
        }
        return new Utf8Info(utf8);
    }
}
