package teajay.bytecode.methods;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.ByteCodeWritable;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public enum MethodKind implements ByteCodeWritable {
    REF_getField(1),
    REF_getStatic(2),
    REF_putField(3),
    REF_putStatic(4),
    REF_invokeVirtual(5),
    REF_invokeStatic(6),
    REF_invokeSpecial(7),
    REF_newInvokeSpecial(8),
    REF_invokeInterface(9);
    
    private final int intValue;
    
    private MethodKind(int value) {
        this.intValue = value;
    }

    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        out.writeByte(this.intValue);
    }
    
    public static MethodKind forValue(int value) {
        for (MethodKind tag : MethodKind.values()) {
            if (value == tag.intValue) {
                return tag;
            }
        }
        return null;
    }
    
    public static MethodKind read(DataInputStream in) throws IOException, ByteCodeException {
        MethodKind tag = forValue(in.readUnsignedByte());
        if (tag != null) {
            return tag;
        } else {
            throw new ByteCodeException("Invalid format for MethodKind.");
        }
    }
}
