/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package abstractsyntaxtree;

import codegeneration.Visitor;
import lexer.Symbol;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 */
public class MemberDeclNode extends SyntaxNode {

    private MethodeOrFieldDeclNode methodeOrFieldDecl = null;
    private VoidMethodeDeclaratorRestHelperNode voidMethodeRestDecl = null;
    private ConstructorDeclaratorRestHelperNode constructorRestDecl = null;
    private GenericMethodeOrConstructorDeclNode genericMethodeOrConstructorDecl = null;
    private ClassDeclarationNode classDecl = null;
    private InterfaceDeclarationNode interfaceDecl = null;

    public MemberDeclNode(Symbol s, MethodeOrFieldDeclNode methodeOrField) {
        super(s);
        this.methodeOrFieldDecl = methodeOrField;
    }

    public MemberDeclNode(Symbol s, VoidMethodeDeclaratorRestHelperNode voidMethodeDecl) {
        super(s);
        this.voidMethodeRestDecl = voidMethodeDecl;
    }

    public MemberDeclNode(Symbol s, ConstructorDeclaratorRestHelperNode constructorRestDecl) {
        super(s);
        this.constructorRestDecl = constructorRestDecl;
    }

    public MemberDeclNode(Symbol s, GenericMethodeOrConstructorDeclNode genericMethodeOrConstructor) {
        super(s);
        this.genericMethodeOrConstructorDecl = genericMethodeOrConstructor;
    }

    public MemberDeclNode(Symbol s, ClassDeclarationNode classDecl) {
        super(s);
        this.classDecl = classDecl;
    }

    public MemberDeclNode(Symbol s, InterfaceDeclarationNode interfaceDecl) {
        super(s);
        this.interfaceDecl = interfaceDecl;
    }

    public boolean hasConstructor() {
        if (this.constructorRestDecl == null) {
            return false;
        }
        return true;
    }

    @Override
    public void accept(Visitor visitor) throws CompilerException {
        visitor.begin(this);
        Visitor v = visitor.visit(this);
        checkNull(this.methodeOrFieldDecl, v);
        checkNull(this.voidMethodeRestDecl, v);
        checkNull(this.constructorRestDecl, v);
        checkNull(this.genericMethodeOrConstructorDecl, v);
        checkNull(this.classDecl, v);
        checkNull(this.interfaceDecl, v);
        visitor.end(this);
    }
}
