/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.environment;

import abstractsyntaxtree.AssignmentOperatorNode;
import abstractsyntaxtree.PrefixOpNode;
import abstractsyntaxtree.SyntaxNode;
import codegeneration.instruction.LValueVariableInstructionCreator;
import codegeneration.instruction.RValueVariableInstructionCreator;
import codegeneration.instruction.VariableInstructionCreator;
import datastructs.Pair;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Deque;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import stage3.states.BlockState;
import stage3.states.MethodState;
import stage3.utils.InformationCenter;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.CheckCast;
import teajay.bytecode.attributes.code.GetField;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.IConst;
import teajay.bytecode.attributes.code.IMode;
import teajay.bytecode.attributes.code.IXOr;
import teajay.bytecode.attributes.code.If0;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.PlaceholderInstruction;
import teajay.bytecode.attributes.code.Pop;
import teajay.bytecode.attributes.code.Pop2;
import teajay.bytecode.attributes.code.VerificationType;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;
import teajay.methods.MethodFinder;
import teajay.methods.MethodFinder.QueryType;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Burzan
 */
public class ExpressionEnvironment {

    private static class PrefixOp {

        final PrefixOpNode.PrefixOps kind;
        final Type castType;

        PrefixOp(PrefixOpNode.PrefixOps kind, Type castType) {
            this.kind = kind;
            this.castType = castType;
        }

        PrefixOp(PrefixOpNode.PrefixOps kind) {
            this.kind = kind;
            this.castType = null;
        }
    }
    private ClassFile file;
    private MethodInfo info;
    //Deprectated PART
    public String lValueResultExpected;
    private String resultType;
    public List<String> parametersType = new ArrayList<>();
    public Object expressionType;
    private Deque<String> instructionTypes = new LinkedList<>();
    // END DEPRECATED PART 
    private Deque<Type> resultTypeList = new LinkedList<>();
    private Type typeOfResult;
    private List<FieldDescriptor> callParameters = new LinkedList<>();
    private List<Type> callTypeParameters = new LinkedList<>();
    private LinkedList<JVMInstruction> lValueInstruction = new LinkedList<>();
    private LinkedList<Type> lValueResultType = new LinkedList<>();
    private boolean isLValueExpression;
    private Stage3Environment stage3Env;
    private Deque<PrefixOp> prefixOpsList = new LinkedList<>();
    private AssignmentOperatorNode.Operators lValueAssignOp;
    private boolean lValueCall;
    private Type lValueCallType;
    private Type lValueCallTypeParam;
    private VariableInstructionCreator variableInstructionCreator;
    private PlaceholderInstruction placeholder;
    private ShortCircuitEnvironment shortCircuitEnv = null;

    public ExpressionEnvironment() {
    }

    public ExpressionEnvironment(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.file = stage3Env.getClassFile();
        this.info = stage3Env.getMethodInfo();
        if (this.stage3Env.getExpressionEnvironment() != null) {
            this.shortCircuitEnv = this.stage3Env.getExpressionEnvironment().shortCircuitEnv;
            for (Type type : stage3Env.getExpressionEnvironment().getOperandStack()) {
                setResultType(type);
            }
        }
    }

    ExpressionEnvironment(Stage3Environment stage3Env, boolean withPrefix) {
        this.stage3Env = stage3Env;
        this.file = stage3Env.getClassFile();
        this.info = stage3Env.getMethodInfo();
        if (stage3Env.getExpressionEnvironment() != null) {
            this.prefixOpsList = stage3Env.getExpressionEnvironment().prefixOpsList;
            this.shortCircuitEnv = this.stage3Env.getExpressionEnvironment().shortCircuitEnv;

            for (Type type : stage3Env.getExpressionEnvironment().getOperandStack()) {
                setResultType(type);
            }
        }
    }
    /**
     * 
     * @return true if the current method static 
     */
    public boolean staticContext() {
        return this.info.isStatic();
    }
    /**
     * 
     * @return true if the current part of the expression LVALUE  
     */
    public boolean isRvalue() {
        return !isLValueExpression;
    }
    /**
     * 
     * @return true if the current part of the expression RVALUE 
     */
    public boolean isLvalue() {
        return isLValueExpression;
    }
    /**
     * Add a instruction to instruction list of the current method
     * @param ins instruction 
     */
    public void insertCode(JVMInstruction ins) {
        try {
            BlockState state = stage3Env.hasState(BlockState.class);

            if (state != null && !state.getLabels().isEmpty()) {
                List<Label> labels = state.getLabels();
                for (Label l : labels) {
                    System.out.print("[ " + l.getName() + " ");
                }
                System.out.print(" ] \n");

                stage3Env.getLocalTable().addStackMapFrame(
                        this.info.getCode().getInstructionList().add(
                        state.getLabels(), ins));
                state.resetLabelList();

            } else {
                System.out.println("\t" + ins);
                this.info.getCode().getInstructionList().add(ins);
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }

    /**
     * add type to type stack
     *
     * @param type
     */
    public final void setResultType(Type type) {
        setResultType(type, true);
    }

    /**
     * Debug method, print the type stack
     */
    public void printResultTypeList() {
        System.err.println(resultTypeList);
    }

    /**
     * add type to type stack is withPush true, otherwise set field typeOfResult
     * = resultType
     *
     * @param resultType
     * @param withPush
     * @exception NullPointerException is resultType null
     */
    public final void setResultType(Type resultType, boolean withPush) {
        if (resultType == null) {
            throw new NullPointerException("resultType");
        }
        this.typeOfResult = resultType;
        if (withPush) {
            this.resultTypeList.addLast(resultType);
        }
    }

    /**
     * Stores the arguments types
     *
     * @param typeOfResult
     */
    public void pushCallParameter(Type typeOfResult) {
        this.callParameters.add(stage3Env.getTypeUtils().makeFieldDescriptor(typeOfResult));
        this.callTypeParameters.add(typeOfResult);

    }

    /**
     * save the first call parameter
     * @param t
     */
    public void addFirstTypeParameter(Type t) {
        this.callParameters.add(0, stage3Env.getTypeUtils().makeFieldDescriptor(t));
        this.callTypeParameters.add(0, typeOfResult);
    }

    /**
     * Get the arguments types form call converted in FieldDescriptos
     *
     * @return FieldDescriptor[]
     */
    public FieldDescriptor[] getCallParameters() {
        return this.callParameters.toArray(new FieldDescriptor[0]);
    }

    /**
     * Get the arguments types
     *
     * @return List<Type>
     */
    public List<Type> getCallTypeParameters() {
        return this.callTypeParameters;
    }

    @Deprecated
    public String popInstructionStack() {
        return this.instructionTypes.pop();
    }

    /**
     *
     * @return last inserted result type
     */
    public Type typeOfResult() {
        return this.typeOfResult;
    }

    /**
     *
     * @return the current MethodInfo
     */
    public MethodInfo getMethod() {
        return this.info;
    }

    /**
     *
     * @return the current ClassFile
     */
    public ClassFile getFile() {
        return this.file;
    }

    /**
     *
     * @return the current class name
     */
    public String getClassName() {
        return this.file.getThis().getClassName();
    }

    @Deprecated
    public void setResultType(String resultType) {
        setResultType(resultType, true);
    }

    @Deprecated
    public void setResultType(String resultType, boolean withPush) {
        this.resultType = resultType;
        if (withPush == true) {
            this.instructionTypes.push(resultType);
        }
    }

    @Deprecated
    public String getResultType() {
        return this.resultType;
    }

    /**
     * pop type from type stack
     *
     * @return Type
     */
    public Type popType() {
        if (!this.resultTypeList.isEmpty()) {
            Type pop = this.resultTypeList.removeLast();
            this.typeOfResult = this.resultTypeList.peekLast();
            return pop;
        }
        return this.typeOfResult;
    }

    /**
     * the expression is a RValue expression
     */
    public void setRValue() {
        this.isLValueExpression = false;
        this.variableInstructionCreator = new RValueVariableInstructionCreator(stage3Env);
    }

    /**
     * This method manage the LValue-RValue Instruction: exp += exp2
     *
     * @param instruction
     * @param type ResultType of the last instruction
     */
    public void addLValueInstruction(JVMInstruction instruction, Type type) {
        this.isLValueExpression = true;
        this.lValueInstruction.add(instruction);
        this.lValueResultType.add(type);
    }
    /**
     * Helper function to clone Environment
     * @param env current environment
     * @return copy of the current environment
     */
    public static ExpressionEnvironment cloneLValueEnvironment(ExpressionEnvironment env) {
        if (env.isLvalue()) {
            ExpressionEnvironment environment = new ExpressionEnvironment(env.stage3Env);
            environment.lValueInstruction = new LinkedList<>();

            for (JVMInstruction ins : env.lValueInstruction) {
                environment.lValueInstruction.add(ins);
            }
            environment.typeOfResult = env.lValueResultType.peekLast();
            environment.setLValue(env.getLValueOperator());
            return environment;
        }
        return null;

    }

    /**
     * Writes the stored LValue-Instructions
     *
     * @throws ByteCodeException
     */
    public void writeLValueInstruction() throws ByteCodeException {
        if (!lValueInstruction.isEmpty()) {
            for (JVMInstruction instr : lValueInstruction) {
                insertCode(instr);
            }
            lValueInstruction.clear();
            if (!lValueResultType.isEmpty()) {
                this.typeOfResult = lValueResultType.peekLast();
            } else {
                this.typeOfResult = new Type.SimpleType("void");
            }
//            if (typeOfResult != null) {
//                this.resultTypeList.push(typeOfResult);
//            }
        }
    }

    /**
     *
     * @return result type of LValue instruction
     */
    public Type getLValueResultType() {
        return this.lValueResultType.getLast();
    }

    /**
     * set instruction without type
     *
     * @param instr instruction
     */
    public void setLValueInstruction(JVMInstruction instr) {
        this.lValueInstruction.add(instr);
    }

    /**
     *
     * @return true if exist type on type stack
     */
    public boolean existItemOnStack() {
        if (resultTypeList.isEmpty()) {
            return false;
        }
        Type peek = resultTypeList.peekLast();
        Type voidType = new Type.SimpleType("void");
        try {
            stage3Env.getTypeUtils().checkAssignability(peek, voidType);
            return false;
        } catch (TypeException ex) {
            return true;
        }
    }

    /**
     *
     * @return the type stack
     */
    public Collection<Type> getOperandStack() {
        return Collections.unmodifiableCollection(resultTypeList);
    }

    /**
     * Helper method to wirte if a == false instruction
     *
     * @return Label for the else position
     */
    public Label writeIf() {
        try {
            //convert Boolean to boolean
            MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
            info.getCode().getInstructionList().add(new InvokeVirtual(ref));
            Label label = new Label("ifSprung : " + info.getName().getValue() + " -> " + file.getThis().getSimpleName());
            info.getCode().getInstructionList().add(new If0(IMode.eq, label));
            //debug message
            System.out.println(new InvokeVirtual(ref));
            System.out.println(new If0(IMode.eq, label));
            return label;
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
        return null;
    }

    /**
     * Helper method to wirte while statements
     *
     * @return the label to loop
     */
    public Label writeWhile() {
        Label label = new Label("whileSprung");

        writeWhile(label, false);
        return label;
    }

    /**
     * Helper method to write while or do-while statements
     *
     * @param label to loop
     * @param doWhile true if doWhile
     */
    public void writeWhile(Label label, boolean doWhile) {
        try {
            MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
            info.getCode().getInstructionList().add(new InvokeVirtual(ref));
            if (doWhile) {
                info.getCode().getInstructionList().add(new If0(IMode.ne, label));
            } else {
                info.getCode().getInstructionList().add(new If0(IMode.eq, label));
            }
            System.out.println(new InvokeVirtual(ref));
            System.out.println(new If0(IMode.eq, label));
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * Helper method to write if a != false instruction
     *
     * @return the label for the else position
     */
    public Label writeNeqIf() {
        try {
            //conver Boolean to boolean
            MethodRefInfo ref = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
            info.getCode().getInstructionList().add(new InvokeVirtual(ref));
            Label label = new Label("ifSprung");
            info.getCode().getInstructionList().add(new If0(IMode.ne, label));
            System.out.println(new InvokeVirtual(ref));
            System.out.println(new If0(IMode.eq, label));
            return label;
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
        return null;
    }

    /**
     *
     * @return the stage3Environment
     */
    public Stage3Environment getStage3Env() {
        return this.stage3Env;
    }

    /**
     * Helper method to wirte goto instruction,
     * in context of try-catch
     *
     * @param jmpOverCatches
     */
    public void writeGoto(Label jmpOverCatches) {
        insertCode(new Goto(jmpOverCatches));
    }

    /**
     * Exist item on type stack at the end of a expression, this method
     * write pop or pop2 instruction pop2 if the last item is double or long 
     * otherwise write pop
     */
    public void writePop() {
        Type peek = resultTypeList.peekLast();
        if (peek instanceof Type.SimpleType) {
            Type.SimpleType simpleType = (Type.SimpleType) peek;
            if (simpleType.name.equals("double") || simpleType.name.equals("long")) {
                insertCode(new Pop2());
                System.out.println(new Pop2());
                return;
            }
        }
        insertCode(new Pop());
        System.out.println(new Pop());
    }

    /**
     * Store the type of a cast expression
     *
     * @param castType 
     */
    public void pushCastPrefix(Type castType) {
        if (castType == null) {
            throw new NullPointerException("castType");
        }
        this.prefixOpsList.push(new PrefixOp(PrefixOpNode.PrefixOps.CAST, castType));
    }

    /**
     *
     * @param prefix oeprator
     */
    public void pushPrefix(PrefixOpNode.PrefixOps prefix) {
        this.prefixOpsList.push(new PrefixOp(prefix));
    }
    private static Set<TeaJayAttribute.Type> forbiddenCastTypes = new HashSet<>(
            Arrays.asList(TeaJayAttribute.Type.Closure, TeaJayAttribute.Type.Hidden, TeaJayAttribute.Type.Impl));

    /**
     * Write all prefix of a expression: - -exp
     *
     * @param node to create correct CompilerException
     * @throws CompilerException if prefix not supported
     */
    public void writePrefix(SyntaxNode node) throws CompilerException {

        try {
            if (!this.prefixOpsList.isEmpty()) {
                PrefixOp prefixOp = this.prefixOpsList.pop();
                PrefixOpNode.PrefixOps prefix = prefixOp.kind;
                FieldDescriptor fieldDesc = this.stage3Env.getTypeUtils().makeFieldDescriptor(popType());
                switch (prefix) {
                    case INVERT:
                    case ADD_ADD:
                    case SUB_SUB:
                        throw new Error("unsupported operators in " + node.getLine());
                    case NOT: {
                        String typeName = fieldDesc.getClassInfo().getClassName();
                        if (typeName.equals("java/lang/Boolean")) {
                            MethodRefInfo booleanValue = MethodRefInfo.make("java/lang/Boolean", "booleanValue", FieldDescriptor.fromString("boolean"));
                            MethodRefInfo valueOf = MethodRefInfo.make("java/lang/Boolean", "valueOf", "java/lang/Boolean", "boolean");
                            insertCode(new InvokeVirtual(booleanValue));
                            insertCode(new IConst(1));
                            insertCode(new IXOr());
                            insertCode(new InvokeStatic(valueOf));
                            setResultType(new Type.ClassType(null, "java/lang/Boolean", null));
                        } else {
                            throw new CompilerException("Bad operand type for operator \'!\': " + typeName, node.getSymbol());
                        }
                    }
                    break;
                    case ADD:
                        //Nothing to do
                        break;
                    case SUB: {
                        String name = fieldDesc.getClassInfo().getClassName();
                        Pair<MethodInfo, MethodFinder.DistanceTailPair> sub = MethodFinder.findMethod("operator_sub", name,
                                stage3Env.getClassFile().getThis().getClassName(), node, QueryType.STATIC_ONLY, fieldDesc);
                        if (sub == null) {
                            throw new CompilerException("no such method: ? operator_sub(" + name + ")", node.getSymbol());
                        } else if (sub.value1.isVarArgs()) {
                            throw new CompilerException("operator methods may not be varargs", node.getSymbol());
                        }
                        insertCode(new InvokeStatic(sub.value1.makeMethodRef()));
                        setResultType(new Type.ClassType(null, sub.value1.getDescriptor().getReturnType().getClassInfo().getClassName(), null));
                    }
                    break;
                    case CAST:
                        Type castType = prefixOp.castType;
                        ClassInfo cast = stage3Env.getTypeUtils().makeFieldDescriptor(castType).getClassInfo();
                        ClassFileRef ref = ClassFinder.instance().find(cast.getClassName());
                        if (ref == null) {
                            throw new CompilerException("Type not found: " + cast.getClassName(), node.getSymbol());
                        }
                        if (fieldDesc.isPrimitive()
                                || (castType instanceof Type.ArrayType && !fieldDesc.isArray() && !fieldDesc.isObject()
                                && !fieldDesc.getClassInfo().getClassName().equals("null"))) {
                            throw new CompilerException("impossible cast: " + castType + " -> " + fieldDesc.getClassInfo().getClassName(), node.getSymbol());
                        }
                        TeaJayAttribute.Type teaJayAttributeType = InformationCenter.getTeaJayAttributeType(ref.get());
                        if (forbiddenCastTypes.contains(teaJayAttributeType)) {
                            throw new CompilerException("Cannot cast to TeaJay kind: " + teaJayAttributeType, node.getSymbol());
                        }
                        insertCode(new CheckCast(cast));
                        setResultType(castType);

                        break;
                    default:
                        throw new AssertionError(prefix.name());

                }
            }
        } catch (ByteCodeException | ResolveException | IOException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * Reset the call parameter list
     */
    public void resetCallParameters() {
        this.callParameters.clear();
        this.callTypeParameters.clear();
    }

    /**
     * The expression is a LValue expression with assignment operator
     *
     * @param assignOp
     */
    public void setLValue(AssignmentOperatorNode.Operators assignOp) {
        this.lValueAssignOp = assignOp;
        this.isLValueExpression = true;
        this.variableInstructionCreator = new LValueVariableInstructionCreator(stage3Env);
    }

    /**
     *
     * @return the assignment operator
     */
    public AssignmentOperatorNode.Operators getLValueOperator() {
        return this.lValueAssignOp;
    }

    /**
     *
     * @param true if expression: exp[10] = 12
     */
    public void setLValueCall(boolean is) {
        this.lValueCall = is;
    }

    /**
     *
     * @return true if expression like exp[10] = 12
     */
    public boolean isLValueCall() {
        return this.lValueCall;
    }

    /**
     * if expression: exp[10][12]... = 12
     * then save the [ 12 ] expression
     *
     * @param typeOfResult from the last [exp]
     * @param t result type from exp[10][12]...
     */
    public void setLValueCall(Type typeOfResult, Type t) {
        setLValueCall(true);
        this.lValueCallType = typeOfResult;
        this.lValueCallTypeParam = t;
    }
    /**
     * 
     * @return the type for the LValue calling 
     */
    public Type getLValueCallType() {
        return this.lValueCallType;
    }
    /**
     * 
     * @return the parameter type for the LValue calling
     */
    public Type getLValueCallTypeParam() {
        return this.lValueCallTypeParam;
    }

    /**
     *
     * @return RValueInstructionCreator if expression RValue otherwise
     * LValueInstructionCreator
     */
    public VariableInstructionCreator getVariableInstructionCreator() {
        return this.variableInstructionCreator;
    }

    public void insertPlaceholder(PlaceholderInstruction placeholder) {
        insertCode(placeholder);
        if (this.placeholder != null) {
            throw new Error("Fatal error: placeholder is already defined");
        }
        this.placeholder = placeholder;
    }

    public PlaceholderInstruction getPlaceholder() {
        PlaceholderInstruction p = this.placeholder;
        this.placeholder = null;
        return p;

    }

    /**
     *
     * @param aThis true if access to this field, false if access to super field
     */
    public void writeThisOrSuper(boolean aThis) {
        try {
            String thisOrSuper = aThis ? "this" : "super";
            insertCode(new ALoad(0));
            //get field info from class file
            FieldInfo field = this.stage3Env.getClassFile().getField(thisOrSuper);
            if (field == null) {
                MethodState mstate = stage3Env.hasState(MethodState.class);
                if (mstate.isClosure()) {
                    setResultType(stage3Env.getThis());
                } else {
                    throw new Error("no this or super field present!");
                }
            } else {
                //write getfield Instruction
                insertCode(new GetField(field.getFieldRefInfo(this.stage3Env.getClassFile().getThis())));
                Type fieldType = stage3Env.getTypeUtils().getFieldType(field);
                //set type to type stack
                setResultType(fieldType);
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * 
     * @param shortCircuitEnv
     */
    public void setShortCircuit(ShortCircuitEnvironment shortCircuitEnv) {
        this.shortCircuitEnv = shortCircuitEnv;

    }

    /**
     * bind all placeholder instruction
     *
     * @param gotoLabel to nop jump
     */
    public void writeShortCircuit(Label gotoLabel) {
        this.shortCircuitEnv.write(stage3Env, gotoLabel, new Type[0]);
        resetShortCircuit();
    }

    /**
     * delete short cicuit environment
     */
    public void resetShortCircuit() {
        this.shortCircuitEnv = null;
    }

    /**
     * check has expression short circuit semantic
     *
     * @return true if expression has short circuit
     */
    public boolean isShortCircuit() {
        if (shortCircuitEnv == null) {
            return false;
        }
        return true;
    }

    /**
     * @deprecated @param stackType
     * @param instruction
     */
    private void insertCodeWithVerificationStack(Type.ClassType stackType, JVMInstruction instruction) {
        try {
            BlockState state = stage3Env.hasState(BlockState.class);

            if (state != null && !state.getLabels().isEmpty()) {
                List<Label> labels = state.getLabels();
                for (Label l : labels) {
                    System.out.print("[S " + l.getName() + " ");
                }
                stage3Env.getLocalTable().addStackMapFrame(this.stage3Env.getMethodInfo().
                        getCode().getInstructionList().
                        add(labels, instruction),
                        VerificationType.valueOf(ClassInfo.fromClass(Boolean.class)));
                System.out.println(instruction);
                state.resetLabelList();
                //            stage3Env.getExpressionEnvironment().setResultType(stackType);
                // stage3Env.getExpressionEnvironment().resetShortCircuit();

            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * insert instruction to instruction list and force stack map frame at
     * instruction
     *
     * @param instruction
     * @param stackTypes
     */
    public void forceInsertCodeWithVerificationStack(JVMInstruction instruction, Collection<Type> stackTypes) {
        for (Type type : stackTypes) {
            if (type == null) {
                throw new Error("Fatal error: stack type may not be null");
            }
        }
        try {
            BlockState state = stage3Env.hasState(BlockState.class);
            List<Label> labels = Collections.EMPTY_LIST;

            if (state != null && !state.getLabels().isEmpty()) {
                labels = state.getLabels();
                for (Label l : labels) {
                    System.out.print("[S " + l.getName() + " ");
                }
            }
            List<VerificationType> verficationTypes = new ArrayList<>();

            for (Type type : stackTypes) {
                FieldDescriptor descr = stage3Env.getTypeUtils().makeFieldDescriptor(type);
                verficationTypes.add(VerificationType.valueOf(descr));
            }
            stage3Env.getLocalTable().addStackMapFrame(this.stage3Env.getMethodInfo().
                    getCode().getInstructionList().
                    add(labels, instruction),
                    verficationTypes);
            System.out.println(instruction);
            if (state != null) {
                state.resetLabelList();
            }



        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * insert instruction to instruction list and set stack map frame at this
     * instruction if the last block state has labels
     *
     * @param instruction
     * @param stackTypes
     */
    public void insertCodeWithVerificationStack(JVMInstruction instruction, Collection<Type> stackTypes) {
        for (Type type : stackTypes) {
            if (type == null) {
                throw new Error("Fatal error: stack type may not be null");
            }
        }
        try {
            BlockState state = stage3Env.hasState(BlockState.class);

            if (state != null && !state.getLabels().isEmpty()) {
                List<Label> labels = state.getLabels();
                for (Label l : labels) {
                    System.out.print("[S " + l.getName() + " ");
                }
                List<VerificationType> verficationTypes = new ArrayList<>();

                for (Type type : stackTypes) {
                    if (type instanceof Type.Uninitialized) {
                        Type.Uninitialized uninitialized = (Type.Uninitialized) type;
                        if (uninitialized.getInstruction() != null) {
                            verficationTypes.add(VerificationType.makeUninitialized(uninitialized.getInstruction()));
                        } else {
                            verficationTypes.add(VerificationType.makeUninitializedThis());
                        }
                    } else {
                        FieldDescriptor descr = stage3Env.getTypeUtils().makeFieldDescriptor(type);
                        verficationTypes.add(VerificationType.valueOf(descr));
                    }
                }
                stage3Env.getLocalTable().addStackMapFrame(this.stage3Env.getMethodInfo().
                        getCode().getInstructionList().
                        add(labels, instruction),
                        verficationTypes);
                System.out.println(instruction);
                state.resetLabelList();
                //stage3Env.getExpressionEnvironment().setResultType(stackType[stackType.length - 1]);
                //stage3Env.getExpressionEnvironment().resetShortCircuit();

            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    /**
     * helper method convert stack type to list
     *
     * @param instruction
     * @param stackType
     */
    public void insertCodeWithVerificationStack(JVMInstruction instruction, Type... stackType) {
        insertCodeWithVerificationStack(instruction, Arrays.asList(stackType));
    }

    public ShortCircuitEnvironment getShortCircuit() {
        return this.shortCircuitEnv;
    }

    public void setStage3Environment(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.file = stage3Env.getClassFile();
        this.info = stage3Env.getMethodInfo();

    }
    /**
     * remove all items from operand stack
     */
    public void cleanOperandStack() {
        this.typeOfResult = null;
        this.resultTypeList.clear();
    }
}
