/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.environment;

import abstractsyntaxtree.InfixOpNode;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import stage3.states.BlockState;
import teajay.bytecode.BaseType;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.IConst;
import teajay.bytecode.attributes.code.IMode;
import teajay.bytecode.attributes.code.If0;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.attributes.code.PlaceholderInstruction;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class ShortCircuitEnvironment {

    private Label falseJmp;
    private Label trueJmp;
    private boolean released = false;
    private InfixOpNode.InfixOps operator;
    private List<ShortCircuitSemantic> shortCircutSemanticList = new LinkedList<>();
    private boolean withDup = false;
    /**
     * 
     * @param trueLabel position to wirte true in instruction list
     * @param falseLabel position to wirte true in instruction false
     * @param operator 
     */
    public ShortCircuitEnvironment(Label trueLabel, Label falseLabel, InfixOpNode.InfixOps operator) {
        this.trueJmp = trueLabel;
        this.falseJmp = falseLabel;
        if (trueJmp.equals(falseJmp)) {
            throw new Error("Fatal error: true and false label shows to the same address");
        }
        this.operator = operator;
    }

    public void setFalseLabel(Label l) {
        this.falseJmp = l;
    }

    public void setTrueLabel(Label l) {
        this.trueJmp = l;
    }

    public Label getFalseLabel() {
        return this.falseJmp;
    }

    public Label getTrueLabel() {
        return this.trueJmp;
    }
    /**
     * write ture and false instructions in the instuction list and
     * binds all PlaceHolderInstruction
     * @param stage3Env
     * @param gotoLabel jump to instruction after short circuit  
     */
    private void write(Stage3Environment stage3Env, Label gotoLabel) {


        if (this.trueJmp.equals(this.falseJmp)) {
            throw new Error("Fatal error:true and false label shows to the same address");
        }
        try {
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            BlockState hasState = stage3Env.hasState(BlockState.class);
            hasState.addLabel(trueJmp);
            //Write true
            env.insertCode(new IConst(1));
            env.insertCode(new InvokeStatic(MethodRefInfo.make("java/lang/Boolean",
                    "valueOf",
                    FieldDescriptor.fromClass(Boolean.class), FieldDescriptor.fromBaseType(BaseType.Boolean))));
            env.getMethod().getCode().getInstructionList().add(new Goto(gotoLabel));
            System.out.println(new Goto(gotoLabel));

            hasState.addLabel(falseJmp);
            //Write false
            env.insertCode(new IConst(0));
            env.insertCode(new InvokeStatic(MethodRefInfo.make("java/lang/Boolean",
                    "valueOf",
                    FieldDescriptor.fromClass(Boolean.class), FieldDescriptor.fromBaseType(BaseType.Boolean))));
     

        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
        createSemantic();
    }
    /**
     * write ture and false instructions with stack map frame in the instuction list
     * and binds all PlaceHolderInstruction
     * @param stage3Env
     * @param gotoLabel jump to instruction after short circuit  
     */
    public void write(Stage3Environment stage3Env, Label gotoLabel, Type... verificationTypes) {
        if (verificationTypes.length == 0) {
            write(stage3Env, gotoLabel);
        } else {
            try {
                ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
                BlockState hasState = stage3Env.hasState(BlockState.class);
                hasState.addLabel(trueJmp);
                env.insertCodeWithVerificationStack(new IConst(1), verificationTypes);
                env.insertCode(new InvokeStatic(MethodRefInfo.make("java/lang/Boolean",
                        "valueOf",
                        FieldDescriptor.fromClass(Boolean.class), FieldDescriptor.fromBaseType(BaseType.Boolean))));
                env.getMethod().getCode().getInstructionList().add(new Goto(gotoLabel));
                System.out.println(new Goto(gotoLabel));

                hasState.addLabel(falseJmp);

                env.insertCodeWithVerificationStack(new IConst(0), verificationTypes);
                env.insertCode(new InvokeStatic(MethodRefInfo.make("java/lang/Boolean",
                        "valueOf",
                        FieldDescriptor.fromClass(Boolean.class), FieldDescriptor.fromBaseType(BaseType.Boolean))));
            
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
            createSemantic();
        }
    }

    public boolean released() {
        return this.released;
    }

    public void release() {
        this.released = true;
    }

    public InfixOpNode.InfixOps getOperator() {
        return operator;
    }

    public void setOperator(InfixOpNode.InfixOps operator) {
        this.operator = operator;
    }

    public void addBeginOr(PlaceholderInstruction orPlaceholder) {
        this.shortCircutSemanticList.add(new ShortCircuitSemantic(orPlaceholder, true));
    }

    public void addNextOr(PlaceholderInstruction orPlaceholder, NOP nop) {
        this.shortCircutSemanticList.add(new ShortCircuitSemantic(orPlaceholder, true, nop));
    }

    public void addBeginAnd(PlaceholderInstruction andPlaceholder) {
        this.shortCircutSemanticList.add(new ShortCircuitSemantic(andPlaceholder, false));
    }

    public void addNextAnd(PlaceholderInstruction andPlaceholder, NOP nop) {
        this.shortCircutSemanticList.add(new ShortCircuitSemantic(andPlaceholder, false, nop));
    }
    /**
     * binds the PlaceHolderInstructions
     */
    private void createSemantic() {
        boolean isOnlyOr = true;
        boolean isOnlyAnd = true;
        //Check is only or-operator or and-operator 
        for (ShortCircuitSemantic s : shortCircutSemanticList) {
            if (!s.isORSemantic) {
                isOnlyOr = false;

            }
            if (s.isORSemantic) {
                isOnlyAnd = false;
            }
        }
        if (isOnlyOr) {
            //binds all PlaceHoldeInstruction
            for (ShortCircuitSemantic s : shortCircutSemanticList) {
                s.getPlaceholder().setInstruction(new If0(IMode.ne, trueJmp));
            }
        } else if (isOnlyAnd) {
            //binds all PlaceHoldeInstruction
            for (ShortCircuitSemantic s : shortCircutSemanticList) {
                s.getPlaceholder().setInstruction(new If0(IMode.eq, falseJmp));
            }
        } else {
            processCompositeShortCircuit(shortCircutSemanticList.listIterator());
        }
    }
    /**
     * Binds the placeholder instructions
     * @param iter start position in list  
     */
    private void processCompositeShortCircuit(ListIterator<ShortCircuitSemantic> iter) {

        int index = iter.previousIndex();

        // go to end of Or Semantic
        ShortCircuitSemantic s = iter.next();
        boolean isORSemantic = true;
        if (s.isORSemantic) {
            while (iter.hasNext()) {
                s = iter.next();
                if (!s.isORSemantic) {
                    break;
                }
            }
        } else {
            // go to end of And Semantic
            while (iter.hasNext()) {
                s = iter.next();
                if (s.isORSemantic) {
                    break;
                }
            }
            isORSemantic = false;
        }

        ListIterator<ShortCircuitSemantic> iter2 = shortCircutSemanticList.listIterator(index + 1);
        ShortCircuitSemantic sem = iter2.next();

        Label l = null;
        // true || true && false 
        while (sem != s) {
            if (isORSemantic) {
                l = trueJmp;
                sem.getPlaceholder().setInstruction(new If0(IMode.ne, trueJmp));
            } else {
                l = falseJmp;
                sem.getPlaceholder().setInstruction(new If0(IMode.eq, s.nop));
            }

            sem = iter2.next();

        }
        if (l == null) {
            throw new Error("Fatal error: Label is null this is not possible!");
        }
        if (!iter2.hasNext()) {

            if (sem.isORSemantic) {
                sem.getPlaceholder().setInstruction(new If0(IMode.ne, trueJmp));
            } else {
                sem.getPlaceholder().setInstruction(new If0(IMode.eq, falseJmp));
            }
        } else {
            iter2.previous();
            processCompositeShortCircuit(iter2);
        }
    }

    public void withDup() {
        this.withDup = true;
    }
    /*
     * Manage the expression: exp && exp || exp 
     */
    private static class ShortCircuitSemantic {

        private final PlaceholderInstruction placeHolder;
        private final boolean isORSemantic;
        private final NOP nop;

        public ShortCircuitSemantic(PlaceholderInstruction placeholderInstruction, boolean isORSemantic, NOP nop) {
            this.placeHolder = placeholderInstruction;
            this.isORSemantic = isORSemantic;
            this.nop = nop;
        }

        public ShortCircuitSemantic(PlaceholderInstruction placeholderInstruction, boolean isORSemantic) {
            this.placeHolder = placeholderInstruction;
            this.isORSemantic = isORSemantic;
            this.nop = null;
        }

        public boolean isORSemantic() {
            return isORSemantic;
        }

        public boolean hasNOP() {
            return nop != null;
        }

        public PlaceholderInstruction getPlaceholder() {
            return this.placeHolder;
        }
    }
}
