/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.environment;

import abstractsyntaxtree.SyntaxNode;
import datastructs.Pair;
import exceptions.CompilerException;
import teajay.util.tjattribute.TeaJayAttribute;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import minicompiler.WorkingNode;
import symbol.SymbolFacade;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.util.io.Input;

/**
 *
 * @author Michael Burzan
 * Environment to mangage the entity name, package and imports
 */
public class Stage0Environment implements Comparable<Stage0Environment> {

    private SymbolFacade symbolManager;
    private HashSet<WorkingNode> queue = new HashSet<>();
    private Pair<String, SyntaxNode> ast;
    private ClassFile classFile;

    public Stage0Environment() {
        this.symbolManager = new SymbolFacade();
        this.ast = null;
    }

    public Stage0Environment(Pair<String, SyntaxNode> ast) {
        this.symbolManager = new SymbolFacade();
        this.ast = ast;
    }

    public void setSymbolFacade(SymbolFacade symbolManager) {
        this.symbolManager = symbolManager;
    }

    public SymbolFacade getSymbolFacade() {
        return this.symbolManager;
    }

    public void addStaticInputWildcardToWokingQueue(String name, Map<String, Input> findAll) throws CompilerException {
        throw new CompilerException("static imports not yet supported");
        //queue.add(new WorkingNode(symbolManager, name, findAll, true));

    }

    public void addInputWildcardToWorkinQueue(String name, Map<String, Input> findAll) {
        for (Entry<String, Input> entry : findAll.entrySet()) {
            queue.add(new WorkingNode(symbolManager, name + "/" + entry.getKey(), entry.getValue()));
        }
        //queue.add(new WorkingNode(symbolManager, name, findAll, false));
    }

    public void addStaticInputToWorkingQueue(String name, Input find) throws CompilerException {
        throw new CompilerException("static imports not yet supported");
    }

    public void addInputToWorkingQueue(String name, Input find) {
        queue.add(new WorkingNode(symbolManager, name, find));
    }

    @Deprecated
    public void addOwnPackageToWorkingQueue(Map<String, Input> findAll) {
        for (Entry<String, Input> entry : findAll.entrySet()) {
            queue.add(new WorkingNode(symbolManager, entry.getKey(), entry.getValue()));
        }
        //queue.add(new WorkingNode(symbolManager, "", findAll, false));
    }

    public HashSet<WorkingNode> getWorkingQueueForCompilationUnit() {
        return this.queue;
    }

    public void setAst(Pair<String, SyntaxNode> ast) {
        this.ast = ast;
    }

    public void setClassFile(ClassFile file) {
        this.classFile = file;
    }

    public ClassFile getClassFile() {
        return this.classFile;
    }

    @Override
    /*
     * compare Environment:
     * enum < interface< typedef < typeimpl
     */
    public int compareTo(Stage0Environment o) {
        AttributeInfo classAttribute = this.classFile.getAttribute(TeaJayAttribute.NAME);
        AttributeInfo oAttribute = o.classFile.getAttribute(TeaJayAttribute.NAME);
        if (classAttribute == null || oAttribute == null) {
            throw new Error("Fatal error: teajay attribute not found");
        }
        TeaJayAttribute.Type c1 = ((TeaJayAttribute) classAttribute).getType();
        TeaJayAttribute.Type c2 = ((TeaJayAttribute) oAttribute).getType();
        return c1.compare(c2);


    }

    public Pair<String, SyntaxNode> getAst() {
        return this.ast;
    }
}
