/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.instruction;

import static abstractsyntaxtree.AssignmentOperatorNode.Operators.SET;
import abstractsyntaxtree.IdentifierSuffixNode;
import abstractsyntaxtree.SyntaxNode;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.List;
import symbol.DerivationTree;
import symbol.FieldTypeResolver;
import symbol.LocalEntry;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.GetField;
import teajay.bytecode.attributes.code.GetStatic;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.attributes.code.PutStatic;
import teajay.bytecode.attributes.code.Swap;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.QIDTools;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 * create LValue access
 */
public class LValueVariableInstructionCreator extends VariableInstructionCreator {

    public LValueVariableInstructionCreator(Stage3Environment stage3Env) {
        super(stage3Env);
    }
    
    /**
     * Create writing access to class fields
     * 
     * @param ident ident.[identifierList] 
     * @param identifierList
     * @param node
     * @return number of operants on the stack
     * @throws CompilerException
     * @throws ByteCodeException
     * @throws IOException
     * @throws ResolveException 
     */
    @Override
    public int createClassInstruction(String ident, List<String> identifierList, SyntaxNode node) throws CompilerException, ByteCodeException, IOException, ResolveException {
        int pushCnt = 0;
        //is identifierList is empty then expression is Type; this make not sense
        if (identifierList == null || identifierList.isEmpty()) {
            throw new CompilerException("Variable " + ident + " not found", node.getSymbol());
        }
        /*
         *remove package identifiers: java.lang.String => String 
         */
        List<String> flatList = flatten(ident, identifierList);
        if (flatList == null || flatList.isEmpty()) {
            throw new CompilerException("Class not found " + ident + identifierList.toString(), node.getSymbol());
        }
        ident = flatList.get(0);
        flatList = flatList.subList(1, flatList.size());
        if (flatList == null || flatList.isEmpty()) {
            throw new CompilerException("Variable " + ident + " not found", node.getSymbol());
        }
        // get class file
        ClassFile file = ClassFinder.instance().find(ident).get();
        String selector = flatList.get(0);
        //get field
        FieldInfo field = FieldTypeResolver.find(file, selector);
        if (field == null) {
            throw new CompilerException("Field " + selector + "does not exist", node.getSymbol());
        }
        // only access to static field from typename
        if (!field.isStatic()) {
            throw new CompilerException("non static-variable " + selector + " cannot be referenced from a static-context", null);
        }
        // find owner from type of field
        ClassFile owner = FieldTypeResolver.findOwner(file, selector);
        FieldRefInfo ref = FieldRefInfo.make(file.getThis(), selector, field.getDescriptor());
        //if owner the current typeimplementation
        if (owner == env.getFile()) {
            if (env.isLvalue()) {
                //store put static into environment
                if (field.isFinal()) {
                    throw new CompilerException(field.getFieldName() + " is final", node.getSymbol());
                }
                env.addLValueInstruction(new PutStatic(ref), stage3Env.getTypeUtils().getFieldType(field));
            } else {
                //expression possible field += 10
                // get field
                env.insertCode(new GetStatic(ref));
                pushCnt++;
                env.setResultType(stage3Env.getTypeUtils().getFieldType(field));
            }
        } else {
            //check is acces allowed to field java compatibility
            ClassFileRef ownerRef = ClassFinder.instance().find(owner.getThis().getClassName());
            ClassFileRef fileRef = ClassFinder.instance().find(env.getClassName());
            if (DerivationTree.instance().isDerivated(ownerRef, fileRef)) {
                if (!field.isProtected() && !field.isPrivate() && !field.isPackagePrivate()) {
                    throw new CompilerException("Access to private field " + selector + " not allowed", node.getSymbol());
                }
                if (field.isPackagePrivate()) {
                    String ownerPackage = QIDTools.packageName(file.getThis().getClassName());
                    String classFilePackage = QIDTools.packageName(env.getClassName());
                    if (!QIDTools.samePackage(ownerPackage, classFilePackage)) {
                        throw new CompilerException("Access to package private field " + selector + "  is not allowed from other package", node.getSymbol());
                    }
                }
                if (env.isLvalue()) {
                if (field.isFinal()) {
                    throw new CompilerException(field.getFieldName() + " is final", node.getSymbol());
                }
                    env.addLValueInstruction(new PutStatic(ref), stage3Env.getTypeUtils().getFieldType(field));
                } else {
                    env.insertCode(new GetStatic(ref));
                    pushCnt++;
                    env.setResultType(stage3Env.getTypeUtils().getFieldType(field));
                }
            }
            if (field.isPrivate() || field.isProtected()) {
                throw new CompilerException("Access to private or protected field " + selector + "not allowed", node.getSymbol());
            }
            if (field.isPackagePrivate()) {
                String ownerPackage = QIDTools.packageName(file.getThis().getClassName());
                String classFilePackage = QIDTools.packageName(env.getClassName());
                if (!QIDTools.samePackage(ownerPackage, classFilePackage)) {
                    throw new CompilerException("Access to package private field " + selector + "  is not allowed from other package", node.getSymbol());
                }
            }
            if (env.isLvalue()) {
                if (field.isFinal()) {
                    throw new CompilerException(field.getFieldName() + " is final", node.getSymbol());
                }
                env.addLValueInstruction(new PutStatic(ref), stage3Env.getTypeUtils().getFieldType(field));
            } else {
                env.insertCode(new GetStatic(ref));
                pushCnt++;
                env.setResultType(stage3Env.getTypeUtils().getFieldType(field));
            }
        }
        flatList.remove(0);
        if (flatList.isEmpty()) {
            return pushCnt;
        }
        // access to rest identifiers
        return pushCnt + accessToField(stage3Env, field.getDescriptor().getClassInfo().getClassName(), flatList, node);
    }

    @Override
    public int createFieldInstruction(String fieldName, List<String> selectors, boolean isArrayAccess, SyntaxNode node) throws CompilerException, ByteCodeException, IOException, ResolveException {
        int pushCnt = 0;
        FieldInfo field = FieldTypeResolver.find(env.getFile(), fieldName);
        if (env.staticContext() && !field.isStatic()) {
            throw new CompilerException("Access to non-static field " + fieldName + " not allowed", node.getSymbol());
        }
        ClassFile owner = FieldTypeResolver.findOwner(env.getFile(), fieldName);
        if (owner.getThis().equals(env.getFile().getThis())) {
            if (!field.isStatic()) {
                // Non static member field
                //instructionList.add(new ALoad(0));
                //env.insertCode(new ALoad(0));
                //env.setResultType(stage3Env.getThis());
                //env.instructionTypes.add(env.getClassName());
                //env.setResultType(env.getClassName());
            }
        } else {
            if (field.isProtected() || field.isPublic()) {
                //instructionList.add(new ALoad(0));
                //env.insertCode(new ALoad(0));
                //env.setResultType(stage3Env.getThis());
            }
            if (field.isPackagePrivate()) {
                String ownerPackage = QIDTools.packageName(owner.getThis().getClassName());
                String classFilePackage = QIDTools.packageName(env.getClassName());
                if (QIDTools.samePackage(ownerPackage, classFilePackage)) {
                    //instructionList.add(new ALoad(0));
                    //env.insertCode(new ALoad(0));
                    //env.setResultType(stage3Env.getThis());
                } else {
                    throw new CompilerException("Access to package private field " + fieldName, node.getSymbol());
                }
            } else {
                throw new CompilerException("Access to private field " + fieldName, node.getSymbol());
            }
        }
        FieldRefInfo ref = FieldRefInfo.make(owner.getThis(), fieldName, field.getDescriptor());
        Type fieldType = stage3Env.getTypeUtils().getFieldType(field);
        if (selectors == null || selectors.isEmpty()) {
            if (isArrayAccess) {
                env.addLValueInstruction(new ALoad(0), stage3Env.getThis());
                env.addLValueInstruction(new GetField(ref), fieldType);
            } else if (!field.isStatic()) {
                if (field.isFinal()) {
                    throw new CompilerException(field.getFieldName() + " is final", node.getSymbol());
                }
                env.addLValueInstruction(new ALoad(0), stage3Env.getThis());
                env.setLValueInstruction(new Swap());
                env.addLValueInstruction(new PutField(ref), fieldType);
                if (env.getLValueOperator() != SET) {
                    env.insertCode(new ALoad(0));    
                    env.insertCode(new GetField(ref));
                    env.setResultType(fieldType);
                }
            } else {
                if (field.isFinal()) {
                    throw new CompilerException(field.getFieldName() + " is final", node.getSymbol());
                }
                env.addLValueInstruction(new PutStatic(ref), fieldType);
            }

        } else {
            // Access to field in local variable
            int pCnt = accessToField(stage3Env, owner.getThis().getClassName(), selectors, node);
            return pCnt + pushCnt;
        }
        return pushCnt;
    }
    /**
     * 
     * @param entry the local variable entry
     * @param ident the name of the variable
     * @param selector the selectors of the variable
     * @param node
     * @return 1
     * @throws CompilerException
     * @throws ByteCodeException
     * @throws IOException
     * @throws ResolveException 
     */
    @Override
    public int createLocalVariableInstruction(LocalEntry entry, String ident, List<String> selector, SyntaxNode node) throws CompilerException, ByteCodeException, IOException, ResolveException {
        if (selector == null || selector.isEmpty()) {
            env.addLValueInstruction(new AStore(entry.getIndex()), entry.getType());
            //+=,-=,*=,\= first RValue Instruction
            if (env.getLValueOperator() != SET) {
                env.insertCode(new ALoad(entry.getIndex()));
                env.setResultType(entry.getType());
            }

        } else {
            // Access to field in local variable
            //Put localvariavle on stack for accessing to field
            env.insertCode(new ALoad(entry.getIndex()));
            env.setResultType(entry.getType());
            return accessToField(stage3Env, entry.getQID(), selector, true, node);
        }
        return 1;
    }

    @Override
    public int createLocalVariableInstruction(LocalEntry entry, String ident, List<String> selector, IdentifierSuffixNode node, IdentifierSuffixNode.SuffixType suffixType) throws ByteCodeException, CompilerException, IOException, ResolveException {
        if (selector == null || selector.isEmpty()) {
            env.addLValueInstruction(new ALoad(entry.getIndex()), entry.getType());
            if (env.getLValueOperator() != SET) {
                env.insertCode(new ALoad(entry.getIndex()));
                env.setResultType(entry.getType());
            }
        } else {
            // Access to field in local variable
            //Put localvariavle on stack for accessing to field
            env.insertCode(new ALoad(entry.getIndex()));
            env.setResultType(entry.getType());
            return accessToField(stage3Env, entry.getQID(), selector, true, node);
        }
        return 1;

    }
}
