/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor;

import codegeneration.abstractvisitors.AbstractTypeVisitor;
import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.ClosureTypeDecNode;
import abstractsyntaxtree.EdgeClampsOpenCloseHelperNode;
import abstractsyntaxtree.ReferenceTypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractReferenceTypeVisitor;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.lang.model.util.AbstractTypeVisitor7;
import resolve.SymbolTable;
import symbol.NameResolver;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author cefour
 */
public class TypeVisitor extends AbstractTypeVisitor {

    public String typeName;
    private ReferenceTypeVisitor referenceTypeVisitor;

    @Override
    public void begin(BasicTypeNode node) {
    }

    @Override
    public void visit(BasicTypeNode node) {
        BasicTypeNode.BasicType type = node.getType();
        switch (type) {
            case BOOLEAN:
                this.typeName = "Boolean";
                break;
            case BYTE:
                this.typeName = "Byte";
                break;
            case CHAR:
                this.typeName = "Char";
                break;
            case DOUBLE:
                this.typeName = "Double";
                break;
            case FLOAT:
                this.typeName = "Float";
                break;
            case INT:
                this.typeName = "Int";
                break;
            case LONG:
                this.typeName = "Long";
                break;
            case SHORT:
                this.typeName = "Short";
                break;
            default:
                throw new Error("Fatal error: Base type was parse but no base type is found in TypeVisitor. Please send a email to teajay@outx.de");
        }
    }

    @Override
    public void end(BasicTypeNode node) {
    }

    @Override
    public void begin(ReferenceTypeNode node) {
    }

    @Override
    public Visitor visit(ReferenceTypeNode node) {
        return this.referenceTypeVisitor = new ReferenceTypeVisitor();
    }

    @Override
    public void end(ReferenceTypeNode node) {
        StringBuilder builder = new StringBuilder(node.getName());
        for (String s : this.referenceTypeVisitor.getList()) {
            builder.append("/");
            builder.append(s);
        }
        symbol.SymbolFacade facade = null;//symbol.SymbolFacade.instance();
        this.typeName = facade.getFullQualifiedId(builder.toString());
        try {
            if (this.typeName == null) {
                ClassFinder finder = ClassFinder.instance();

                if (finder.find(builder.toString()) != null) {
                    this.typeName = builder.toString();
                } else {
                    throw new CompilerException("Type " + builder.toString() + " does not exist", node.getSymbol());
                }
            }
        } catch (CompilerException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public String getTypeName() {
        return this.typeName;
    }

    @Override
    public void begin(EdgeClampsOpenCloseHelperNode node) {
    }

    @Override
    public void visit(EdgeClampsOpenCloseHelperNode node) {
    }

    @Override
    public void end(EdgeClampsOpenCloseHelperNode node) {
    }

    @Override
    public void begin(ClosureTypeDecNode node) {
    }

    @Override
    public Visitor visit(ClosureTypeDecNode node) {
        return null;
    }

    @Override
    public void end(ClosureTypeDecNode node) {
    }
}
