/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package codegeneration.javavisitor.expression.expressiontransform;

import codegeneration.environment.ExpressionEnvironment;
import abstractsyntaxtree.AssignmentOperatorNode;
import abstractsyntaxtree.Et1Node;
import abstractsyntaxtree.Expression1Node;
import abstractsyntaxtree.Expression1RestNode;
import abstractsyntaxtree.Expression2RestNode;
import abstractsyntaxtree.Expression3Node;
import abstractsyntaxtree.InfixOpNode;
import abstractsyntaxtree.PostfixOpNode;
import abstractsyntaxtree.PrefixOpNode;
import abstractsyntaxtree.PrimaryNode;
import abstractsyntaxtree.SelectorNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractExpression3Visitor;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.Deque;
import java.util.LinkedList;
import symbol.FieldTypeResolver;
import symbol.LocalEntry;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.runtime.exceptions.ResolveException;

/**
 *
 * @author cefour
 */
public class Expression3Visitor extends AbstractExpression3Visitor {

    private final ExpressionEnvironment env;
    private final Deque<InfixOpNode.InfixOps> stack;
    private SelectorVisitor selectorVisitor;
    private AssignmentOperatorNode.Operators operator = null;
    private Deque<InfixOpNode.InfixOps> exp1Stack;
    private ExpressionEnvironment exp1Env;
    private final boolean hasDup;
    private AssignmentOperatorNode exp1Symbol;

    public static enum ExpressionType {

        /* NON_CALL,
         IDENTIFIER_SUFFIX_CALL,
         SELECTOR_CALL, CREATOR_CALL,*/
        RVALUE, LVALUE
    }
    private final ExpressionType expressionCallerType;

    public Expression3Visitor(ExpressionEnvironment env, Deque<InfixOpNode.InfixOps> stack, boolean hasDup) {
        this.env = env;
        this.stack = stack;
        this.expressionCallerType = ExpressionType.LVALUE;
        this.hasDup = hasDup;
    }

    public Expression3Visitor(ExpressionType callerType, ExpressionEnvironment env, Deque<InfixOpNode.InfixOps> stack, boolean hasDup) {
        this.env = env;
        this.stack = stack;
        this.expressionCallerType = callerType;
        this.env.expressionType = this.expressionCallerType;
        this.hasDup = hasDup;
    }

    @Override
    public void begin(PrefixOpNode node) {
    }

    @Override
    public void begin(Expression3Node node) {
    }

    @Override
    /*
     * this.a = this.a +2
     */
    public void begin(PrimaryNode node) {
        try {
            if (node.isThis() || node.isSuper()) {
                if (env.staticContext()) {
                    throw new CompilerException("Not possible access in static-context", node.getSymbol());
                }
                this.env.insertCode(new ALoad(0));
                this.env.setResultType(env.getClassName());
            } else {
                String ident = node.getIdentifier();
                if (ident != null) {
                    LocalEntry entry = null;//SymbolFacade.instance().getLocalTable().getEntry(ident);
                    if (entry != null) { //Is ident a local variable
                   //     VTemp2.createLocalVariableInstruction(env, entry, ident, node.getIdentifierList(), node);
                    } else {
                        FieldInfo field = FieldTypeResolver.find(env.getFile(), ident);
                        if (field != null) {
                   //         VTemp2.createFieldInstruction(ident, node.getIdentifierList(), env, node);
                        } else {
                            if (node.getIdentifierList() == null && node.getIdentifierList().isEmpty()) {
                                throw new CompilerException("Local or field variable " + ident + " not found", node.getSymbol());
                            }
                   //         VTemp2.createClassInstruction(ident, node.getIdentifierList(), env, node);
                        }
                    }

                }
            }
        } catch (ByteCodeException | CompilerException | ResolveException | IOException e) {
            e.printStackTrace(System.err);
            System.exit(-1);
        }

    }

    @Override
    public void begin(SelectorNode node) {

        this.selectorVisitor = new SelectorVisitor(node.getIdentifier(), env);
    }

    @Override
    public void begin(PostfixOpNode node) {
    }

    @Override
    public void begin(Expression2RestNode node) {
    }

    @Override
    public void begin(Expression1RestNode node) {
    }

    @Override
    public void begin(AssignmentOperatorNode node) {
    }

    @Override
    public void begin(Expression1Node node) {
    }

    @Override
    public void begin(Et1Node node) {
    }

    @Override
    public void visit(PrefixOpNode node) {
    }

    @Override
    public Visitor visit(Expression3Node node) {
        return this;
    }

    @Override
    public Visitor visit(PrimaryNode node) {

        return new PrimaryVisitor(env);
    }

    @Override
    public Visitor visit(SelectorNode node) {
        return this.selectorVisitor;
    }

    @Override
    public void visit(PostfixOpNode node) {
    }

    @Override
    public Visitor visit(Expression2RestNode node) {
        return new Expression2RestVisitor(env, stack);
    }

    @Override
    public Visitor visit(Expression1RestNode node) {
        return this;
    }

    @Override
    public void visit(AssignmentOperatorNode node) {
        this.operator = node.getOperator();
        this.exp1Symbol = node;
    }

    @Override
    public Visitor visit(Expression1Node node) {
    //    this.exp1Env = new ExpressionEnvironment(env.getFile(), env.getMethod());
        this.exp1Stack = new LinkedList<>();
        return new Expression1Visitor(exp1Env, exp1Stack, true);
    }

    @Override
    public Visitor visit(Et1Node node) {
        return this;
    }

    @Override
    public void end(PrefixOpNode node) {
    }

    @Override
    public void end(Expression3Node node) {
    }

    @Override
    public void end(PrimaryNode node) {
    }

    @Override
    public void end(SelectorNode node) {
    }

    @Override
    public void end(PostfixOpNode node) {
    }

    @Override
    public void end(Expression2RestNode node) {
    }

    @Override
    public void end(Expression1RestNode node) {
    }

    @Override
    public void end(AssignmentOperatorNode node) {
        //env.info.getCode().getInstructionList().add(env.lValueInstruction);
    }

    @Override
    public void end(Expression1Node node) {
//        try {
//            if (env.expressionType == ExpressionType.RVALUE) {
//                throw new CompilerException("Expect: variable Found: value", exp1Symbol.getSymbol());
//            }
//          //  InstructionCreator.PutOperatorInstructions(exp1Env, env.getMethod(), exp1Stack);
//
//            if (hasDup) {
//
//                env.insertCode(new Dup());
//                env.setResultType(exp1Env.getResultType());
//            }
//          //  env.insertCode(env.lValueInstruction);
//            env.setResultType(exp1Env.getResultType(), false);
//       //     if (!SymbolFacade.instance().isDerivated(env.lValueResultExpected, exp1Env.getResultType())) {
//                throw new CompilerException("Expected: " + env.lValueResultExpected + " Found: " + exp1Env.getResultType(), exp1Symbol.getSymbol());
//         //   }
//        } catch (CompilerException | ByteCodeException ex) {
//            ex.printStackTrace(System.err);
//            System.exit(-1);
//        }
    }

    @Override
    public void end(Et1Node node) {
    }
}
// try {
//            if (node.isThis()) {
//                if (env.info.isStatic()) {
//                    throw new CompilerException("this is not allowed in static method ", node.getSymbol());
//                }
//                env.info.getCode().getInstructionList().add(new ALoad(0));
//                this.env.expressionType = this.env.file.getThis().getClassName();
//
//                InvokeFactory.instance().setInvokerType("virtual");
//            }
//            if (node.isSuper()) {
//                if (env.info.isStatic()) {
//                    throw new CompilerException("super is not allowed in static method ", node.getSymbol());
//                }
//                env.info.getCode().getInstructionList().add(new ALoad(0));
//                this.env.expressionType = this.env.file.getSuper().getClassName();
//                InvokeFactory.instance().setInvokerType("special");
//            } else {
//                String ident = node.getIdentifier();
//                IsIdentifierSuffixVisitor hasIdentSuffix = new IsIdentifierSuffixVisitor();
//                node.accept(hasIdentSuffix);
//
//                if (ident != null) {
//                    LocalEntry entry = SymbolFacade.instance().getLocalTable().getEntry(ident);
//                    if (entry != null) {
//                        switch (this.expressionCallerType) {
//                            case NON_CALL:
//                                break;
//                            case IDENTIFIER_SUFFIX_CALL:
//                            case CREATOR_CALL:
//                            case SELECTOR_CALL:
//                                env.info.getCode().getInstructionList().add(new ALoad(entry.getIndex()));
//                                env.expressionType = entry.getQID();
//                                break;
//                            default:/*Assignment ops durchführen*/
//                                env.varName = ident;
//                        }
//                        /*
//                         * var a = new var();
//                         * a.call();
//                         * noch eine Abfrage hinzufügen ob es sich um ein Closure handelt
//                         */
//                        if (hasIdentSuffix.isIdentifierSuffix) {
//                            List<String> identList = node.getIdentifierList();
//
//                            for (int i = 0; i < identList.size() - 1; i++) {
//                                String s = identList.get(i);
//                            }
//                            env.varName = identList.get(identList.size() - 1) == null ? "" : identList.get(identList.size());
//
//                        }
//
//
//                    } else {
//                        // check exist Field
//                        ClassFileRef info = FieldTypeResolver.find(env.file, ident);
//                        if (info != null) {
//                            ClassFile classInThereFieldExist = info.get();
//                            FieldInfo field = classInThereFieldExist.getField(ident);
//                            if (!field.isStatic() && env.info.isStatic()) {
//                                throw new CompilerException("Non-static variable " + ident + "cannot be referenced from a static-context", node.getSymbol());
//                            }
//                        }
//                        //identifier is not local variable or field 
//                        //possible class
//                        //Check is class
//                        String qID = SymbolFacade.instance().getFullQualifiedId(ident);
//
//                        if (qID != null) {
//                            ClassFileRef classFile = ClassFinder.instance().find(qID);
//                            List<String> identifierList = node.getIdentifierList();
//                            if (identifierList.isEmpty()) {
//                                throw new CompilerException("Not a statement", node.getSymbol());
//                            }
//                            String s = identifierList.get(0);
//                            FieldInfo field = classFile.get().getField(s);
//
//                            if (!field.isStatic()) {
//                                throw new CompilerException("Can't access on non static field", node.getSymbol());
//                            }
//                            JVMInstructionList instructionList = env.info.getCode().getInstructionList();
//                            FieldRefInfo fieldRef = FieldRefInfo.make(classFile.get().getThis(), s, field.getDescriptor());
//                            instructionList.add(new GetStatic(fieldRef));
//                            this.env.typeName = field.getDescriptor().getClassInfo().getClassName();
//                            for(int i = 1; i < identifierList.size()-1; i++){
//                                s = identifierList.get(i);
//                                classFile = ClassFinder.instance().find(this.env.typeName);
//                                field = classFile.get().getField(s);
//                                if(!field.isStatic()){
//                                    throw new CompilerException("Accessing static field "+s,node.getSymbol());
//                                }
//                                fieldRef = FieldRefInfo.make(classFile.get().getThis(), s, field.getDescriptor());
//                                instructionList.add(new GetField(fieldRef));
//                            }
//
//                            this.env.varName = identifierList.get(identifierList.size() - 1);
//
//                        } else {
//                            throw new CompilerException("Not a statement", node.getSymbol());
//                        }
//                    }
//                }
//            }
//
//        } catch (CompilerException | ByteCodeException ex) {
//            ex.printStackTrace(System.err);
//            System.exit(-1);
//        } catch (IOException ex) {
//            Logger.getLogger(Expression3Visitor.class.getName()).log(Level.SEVERE, null, ex);
//        } catch (ResolveException ex) {
//            Logger.getLogger(Expression3Visitor.class.getName()).log(Level.SEVERE, null, ex);
//        }
// Other code
//try {
//
//    if (node.isThis() || node.isSuper()) {
//        if (env.staticContext()) {
//            throw new CompilerException("Not possible access in static-context", node.getSymbol());
//        }
//        this.env.info.getCode().getInstructionList().add(new ALoad(0));
//        this.env.resultType = env.file.getThis().getClassName();
//    } else {
//        String ident = node.getIdentifier();
//        List<String> selectorList = node.getIdentifierList();
//        if (ident != null) {
//            if (env.expressionType == Expression3Visitor.ExpressionType.LVALUE) {
//                //Merken für put
//                env.varName = ident;
//                env.selectorList = selectorList;
//            } else {
//                LocalEntry entry = SymbolFacade.instance().getLocalTable().getEntry(ident);
//                if (entry != null) {
//                    this.env.info.getCode().getInstructionList().add(new ALoad(entry.getIndex()));
//                    if (selectorList != null) {
//                        ClassFile file = ClassFinder.instance().find(entry.getQID()).get();
//
//                        for (String selector : selectorList) {
//
//                            FieldInfo field = FieldTypeResolver.find(file, selector);
//                            if (field.isStatic()) {
//                                throw new CompilerException("Access to static field", node.getSymbol());
//                            }
//                            ClassFile fieldFile = ClassFinder.instance().find(field.getDescriptor().getClassInfo().getClassName()).get();
//                            if ((field.isPrivate() || field.isProtected()) && file != this.env.file) {
//                                throw new CompilerException("Access to private or protected field", node.getSymbol());
//                            }
//                            FieldRefInfo fieldRefInfo = FieldRefInfo.make(file.getThis(), selector, field.getDescriptor());
//                            this.env.info.getCode().getInstructionList().add(new GetField(fieldRefInfo));
//                            file = fieldFile;
//
//                        }
//                        env.resultType = file.getThis().getClassName();
//                    }else{
//                        env.resultType = entry.getQID();
//                    }
//                } else {
//                    FieldInfo field = FieldTypeResolver.find(env.file, ident);
//                    if (field != null) {
//                        if (env.staticContext() && !field.isStatic()) {
//                            throw new CompilerException("Try to access to non-static variable in static-context", node.getSymbol());
//                        }
//                        FieldRefInfo fieldRefInfo = FieldRefInfo.make(field.getDescriptor().getClassInfo(), ident, field.getDescriptor());
//                        if (field.isStatic()) {
//                            if (field.getDescriptor().getClassInfo() != env.file.getThis()) {
//                                if (field.isPrivate() || field.isProtected()) {
//                                    throw new CompilerException("Access to private or protected static field", node.getSymbol());
//                                }
//                                String fieldPackage = QIDTools.packageName(field.getDescriptor().getClassInfo().getClassName());
//                                if (field.isPackagePrivate() && !packageName.equals(fieldPackage)) {
//                                    throw new CompilerException("Access to private or protected static field", node.getSymbol());
//                                }
//                            }
//                            this.env.info.getCode().getInstructionList().add(new GetStatic(fieldRefInfo));
//                            this.env.resultType = field.getDescriptor().getClassInfo().getClassName();
//                        }else{ // Make this field 
//                            this.env.info.getCode().getInstructionList().add(new ALoad(0));
//                        }
//                        if (selectorList != null) {
//                            ClassFile file = ClassFinder.instance().find(entry.getQID()).get();
//
//                            for (String selector : selectorList) {
//
//                                FieldInfo fieldSel = FieldTypeResolver.find(file, selector);
//                                if (fieldSel.isStatic()) {
//                                    throw new CompilerException("Access to static field", node.getSymbol());
//                                }
//                                ClassFile fieldFile = ClassFinder.instance().find(fieldSel.getDescriptor().getClassInfo().getClassName()).get();
//                                if ((fieldSel.isPrivate() || fieldSel.isProtected()) && file != this.env.file) {
//                                    throw new CompilerException("Access to private or protected field", node.getSymbol());
//                                }
//                                FieldRefInfo fieldSelRefInfo = FieldRefInfo.make(file.getThis(), selector, fieldSel.getDescriptor());
//                                this.env.info.getCode().getInstructionList().add(new GetField(fieldSelRefInfo));
//                                file = fieldFile;
//                                env.resultType = file.getThis().getClassName();
//                            }
//                        }
//                    }
//
//                    // Class File
//                    int pos = 0;
//                    if (ClassFinder.instance().find(ident) == null) {
//                        if (selectorList != null) {
//                            for (int i = 0; i < selectorList.size(); i++) {
//                                if (ClassFinder.instance().find(ident + "/" + selectorList.get(i)) != null) {
//                                    pos = i;
//                                    break;
//                                }
//                            }
//                            if (pos == selectorList.size() - 1) {
//                                String dummy = "ident";
//                                for (String s : selectorList) {
//                                    dummy += "." + s;
//                                }
//                                throw new CompilerException("Variable " + dummy + " is not defined", node.getSymbol());
//                            }
//                        } else {
//                            throw new CompilerException("Variable " + ident + " is not defined", node.getSymbol());
//                        }
//                    }
//                    if (pos > 0) {
//                        ident = selectorList.get(pos);
//                        selectorList = selectorList.subList(0, pos + 1);
//                    }
//                    ClassFile file = ClassFinder.instance().find(ident).get();
//                    if (selectorList != null) {
//                        FieldInfo staticField = FieldTypeResolver.find(file, selectorList.get(0));
//                        if (staticField == null) {
//                            throw new CompilerException("Field " + selectorList.get(0) + "not found", null);
//                        }
//                        if (!staticField.isStatic()) {
//                            throw new CompilerException("Access to a not static ", node.getSymbol());
//                        }
//                        String fieldPackage = QIDTools.packageName(staticField.getDescriptor().getClassInfo().getClassName());
//                        if (staticField.isPackagePrivate() && !packageName.equals(fieldPackage)) {
//                            throw new CompilerException("Access to private or protected static field", node.getSymbol());
//                        }
//                        if (staticField.isPrivate() || staticField.isProtected() && env.file != file) {
//                            throw new CompilerException("Access to private or protected field", node.getSymbol());
//                        }
//                        FieldRefInfo ref = FieldRefInfo.make(file.getThis(), selectorList.get(0), field.getDescriptor());
//                        env.info.getCode().getInstructionList().add(new GetStatic(ref));
//                        env.resultType = staticField.getDescriptor().getClassInfo().getClassName();
//
//                        FieldInfo lastField = staticField;
//                        for (int i = 1; i < selectorList.size(); i++) {
//                            ClassFile selFile = ClassFinder.instance().find(lastField.getDescriptor().getClassInfo().getClassName()).get();
//                            FieldInfo selField = FieldTypeResolver.find(selFile, selectorList.get(i));
//                            if (selField == null) {
//                                throw new CompilerException("Field " + selectorList.get(i) + "not exist", node.getSymbol());
//                            }
//                            String selfieldPackage = QIDTools.packageName(selField.getDescriptor().getClassInfo().getClassName());
//                            if (selField.isPackagePrivate() && !packageName.equals(selfieldPackage)) {
//                                throw new CompilerException("Access to private or protected static field", node.getSymbol());
//                            }
//                            if (selField.isPrivate() || selField.isProtected() && env.file != file) {
//                                throw new CompilerException("Access to private or protected field", node.getSymbol());
//                            }
//                            FieldRefInfo r = FieldRefInfo.make(selFile.getThis(), selectorList.get(i),selField.getDescriptor());
//                            this.env.info.getCode().getInstructionList().add(new GetField(r));
//                            env.resultType = selField.getDescriptor().getClassInfo().getClassName();
//                        }
//                    } else {
//                        throw new CompilerException("Not a statemet", node.getSymbol());
//                    }
//                }
//
//
//
//            }
//        }
//    }
//
//} catch (ByteCodeException | CompilerException ex) {
//    ex.printStackTrace(System.err);
//    System.exit(-1);
//} catch (IOException ex) {
//    Logger.getLogger(Expression3Visitor.class.getName()).log(Level.SEVERE, null, ex);
//} catch (ResolveException ex) {
//    Logger.getLogger(Expression3Visitor.class.getName()).log(Level.SEVERE, null, ex);
//}