/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package lexer;

import java.util.Objects;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class Symbol {

    public final int sym;
    public final String value;
    public final int line;
    public final int column;

    public Symbol(int sym, int line, int column, String value) {
        this.sym = sym;
        this.value = value;
        this.line = line;
        this.column = column;
    }

    public Symbol(int sym, int line, int column) {
        this(sym, line, column, null);
    }

    public Symbol(int sym) {
        this(sym, -1, -1, null);
    }

    public Symbol(int sym, String value) {
        this(sym, -1, -1, value);
    }

    @Override
    public String toString() {
        return Sym.getNameByID(sym) + ", value = " + value;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 73 * hash + this.sym;
        hash = 73 * hash + Objects.hashCode(this.value);
        hash = 73 * hash + this.line;
        hash = 73 * hash + this.column;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Symbol) {
            Symbol s = (Symbol) obj;
            return s.sym == sym
                    && s.line == line
                    && s.column == column
                    && ((s.value == null && value == null) || (s.value != null && value != null && value.equals(s.value)));
        } else {
            return false;
        }
    }
}
