/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package lexer;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class UTF16 {

    /**
     * If <tt>a</tt> and <tt>b</tt> form a UTF-16 supplementary character this
     * method returns the corresponding code point.
     *
     * @param a high surrogate
     * @param b low surrogate
     * @return a java code point or <tt>null</tt> if <tt>a</tt> or <tt>b</tt>
     * aren't high- resp. low-surrogates.
     */
    public static Integer utf16Decode(char a, char b) {
        if ((a >>> 10) == 0b110110 && (b >>> 10) == 0b110111) {
            //reverse the utf-16 encoding
            return (((a & 0b1111111111) << 10) | (b & 0b1111111111)) + 0x10000;
        }
        return null;
    }
}
