/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage0;

import abstractsyntaxtree.AnnotationTypeDeclarationNode;
import abstractsyntaxtree.ClassDeclarationNode;
import abstractsyntaxtree.ClassOrInterfaceDeclaratioNode;
import abstractsyntaxtree.EnumDeclarationNode;
import abstractsyntaxtree.InterfaceDeclarationNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.NormalClassDeclarationNode;
import abstractsyntaxtree.NormalInterfaceDeclarationNode;
import abstractsyntaxtree.TjImplDeclarationNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractClassDeclarationVisitor;
import codegeneration.abstractvisitors.AbstractClassOrInterfaceDeclarationVisitor;
import codegeneration.abstractvisitors.AbstractInterfaceDeclarationVisitor;
import codegeneration.abstractvisitors.AbstractTypeDeclarationVisitor;
import teajay.util.tjattribute.TeaJayAttribute;
import codegeneration.environment.Stage0Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import teajay.bytecode.ClassAccessFlag;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.lang.TJComparable;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 * get the full qualified name of interfaces, typeimpl and enums and creates 
 * the class file
 */
class Stage0TypeDeclarationVisitor extends AbstractTypeDeclarationVisitor {

    private ClassFile file;
    private final String packageName;
    private final Stage0Environment stage0Env;

    Stage0TypeDeclarationVisitor(String packageName, Stage0Environment stage0Env) {
        this.packageName = packageName;
        this.stage0Env = stage0Env;
    }

    @Override
    public void begin(ClassOrInterfaceDeclaratioNode node)throws CompilerException {
    }

    @Override
    public Visitor visit(ClassOrInterfaceDeclaratioNode node)throws CompilerException {

        return new AbstractClassOrInterfaceDeclarationVisitor() {
            private final List<ClassAccessFlag> modis = new ArrayList<>();

            @Override
            public void begin(ModifierNode node)throws CompilerException {
                ModifierNode.Modifier modifier = node.getModifier();
                switch (modifier) {
                    case PUBLIC:
                        modis.add(ClassAccessFlag.PUBLIC);
                        break;
                    case PROTECTED:

                        break;
                    case PRIVATE:

                        break;
                    case STATIC:
                        break;
                    case ABSTRACT:
                        modis.add(ClassAccessFlag.ABSTRACT);
                        break;
                    case FINAL:
                        modis.add(ClassAccessFlag.FINAL);
                        break;
                    case NATIVE:
                        break;
                    case SYNCHRONIZED:
                        break;
                    case TRANSIENT:
                        break;
                    case VOLATILE:
                        break;
                    case STRICTFP:
                        break;
                    default:
                        throw new AssertionError(modifier.name());

                }

            }

            @Override
            public Visitor visit(ModifierNode node)throws CompilerException {
                return null;
            }

            @Override
            public void end(ModifierNode node)throws CompilerException {
            }

            @Override
            public void begin(ClassDeclarationNode node)throws CompilerException {
            }

            @Override
            public Visitor visit(ClassDeclarationNode node)throws CompilerException {
                return new AbstractClassDeclarationVisitor() {
                    @Override
                    public void begin(NormalClassDeclarationNode node)throws CompilerException {
                        try {
                            ClassFile file;
                            if (modis.isEmpty()) {
                                if (packageName != null) {
                                    file = new ClassFile(packageName + "/" + node.getClassName(), ClassAccessFlag.PACKAGE_PRIVATE,ClassAccessFlag.SUPER);
                                } else {
                                    file = new ClassFile(node.getClassName(), ClassAccessFlag.PACKAGE_PRIVATE,ClassAccessFlag.SUPER);
                                }
                              
                            } else {
                                if (modis.size() > 1) {
                                    throw new CompilerException("Modifier abstract is only allowed for class declaration in TeaJay", node.getSymbol());
                                }
                                if (modis.get(0) != ClassAccessFlag.ABSTRACT) {
                                    throw new CompilerException("Modifier " + modis.get(0) + " isn't allowed for class declaration", node.getSymbol());
                                }
                                if (packageName != null) {
                                    file = new ClassFile(packageName + "/" + node.getClassName(), ClassAccessFlag.PACKAGE_PRIVATE,ClassAccessFlag.SUPER ,ClassAccessFlag.ABSTRACT);
                                } else {
                                    file = new ClassFile(node.getClassName(), ClassAccessFlag.PACKAGE_PRIVATE, ClassAccessFlag.ABSTRACT, ClassAccessFlag.SUPER);
                                }

                             
                                
                            }
                            ClassFinder.instance().addClassFile(file);
                            stage0Env.setClassFile(file);
                            file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Class));

                        } catch (ByteCodeException | CompilerException ex) {
                            ex.printStackTrace(System.err);
                            System.exit(-1);
                        }

                    }

                    @Override
                    public Visitor visit(NormalClassDeclarationNode node)throws CompilerException {
                        return null;
                    }

                    @Override
                    public void end(NormalClassDeclarationNode node)throws CompilerException {
                    }

                    @Override
                    public Object getClassContainer() {
                        return null;
                    }

                    @Override
                    public void begin(EnumDeclarationNode node)throws CompilerException {
                        try {
                            ClassFile file;
                            if (modis.isEmpty()) {
                                file = new ClassFile(packageName + "/" + node.getName(),ClassAccessFlag.SUPER, ClassAccessFlag.PACKAGE_PRIVATE, ClassAccessFlag.ENUM,ClassAccessFlag.FINAL);
                                ClassFinder.instance().addClassFile(file);
                            } else {
                                if (modis.size() > 1) {
                                    throw new CompilerException("Enum declarations may only have one modifier.", node.getSymbol());
                                }
                                if (modis.get(0) != ClassAccessFlag.PUBLIC) {
                                    throw new CompilerException("Modifier " + modis.get(0) + " isn't allowed for enum declarations", node.getSymbol());
                                }
                                if (packageName != null) {
                                    file = new ClassFile(packageName + "/" + node.getName(),ClassAccessFlag.SUPER, ClassAccessFlag.PUBLIC, ClassAccessFlag.ENUM,ClassAccessFlag.FINAL);
                                } else {
                                    file = new ClassFile(node.getName(), ClassAccessFlag.SUPER,ClassAccessFlag.PUBLIC, ClassAccessFlag.ENUM,ClassAccessFlag.FINAL);
                                }
                                ClassFinder.instance().addClassFile(file);
                            }
                            file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Enum));
                            
                            file.setSuper("java/lang/Enum");
                            
                            file.addInterface(ClassInfo.fromClass(TJComparable.class));
                            Type.ClassType superType = new Type.ClassType(null, "java/lang/Enum",
                                    Arrays.<Type>asList(new Type.ClassType(null, file.getThis().getClassName(), null)));
                            Type.ClassType iface = new Type.ClassType(null, "teajay/lang/TJComparable",
                                    Arrays.<Type>asList(new Type.ClassType(null, file.getThis().getClassName(), null)));
                            
                            Type.ClassSigType sig = new Type.ClassSigType(null, superType, Arrays.<Type>asList(iface));
                            file.addAttribute(new Signature(SignatureEncoder.encode(sig)));
                            //file.addAttribute(new Signature("Ljava/lang/Enum<L" + file.getThis().getClassName() + ";>;"));
                            
                            stage0Env.setClassFile(file);

                        } catch (ByteCodeException | CompilerException ex) {
                            ex.printStackTrace(System.err);
                            System.exit(-1);
                        }
                    }

                    @Override
                    public Visitor visit(EnumDeclarationNode node)throws CompilerException {
                        return null;

                    }

                    @Override
                    public void end(EnumDeclarationNode node)throws CompilerException {
                    }
                };
            }

            @Override
            public void end(ClassDeclarationNode node)throws CompilerException {
            }

            @Override
            public void begin(InterfaceDeclarationNode node)throws CompilerException {
            }

            @Override
            public Visitor visit(InterfaceDeclarationNode node)throws CompilerException {
                return new AbstractInterfaceDeclarationVisitor() {
                    @Override
                    public void begin(NormalInterfaceDeclarationNode node)throws CompilerException {
                        try {
                            ClassFile file;
                            if (modis.isEmpty()) {
                                if (packageName != null) {
                                    file = new ClassFile(packageName + "/" + node.getInterfaceName(), ClassAccessFlag.PACKAGE_PRIVATE, ClassAccessFlag.INTERFACE, ClassAccessFlag.ABSTRACT);
                                } else {
                                    file = new ClassFile(node.getInterfaceName(), ClassAccessFlag.PACKAGE_PRIVATE, ClassAccessFlag.INTERFACE, ClassAccessFlag.ABSTRACT);
                                }
                                ClassFinder.instance().addClassFile(file);
                            } else {
                                if (modis.size() > 1) {
                                    throw new CompilerException("Modifier isn't allowed for interface declaration", node.getSymbol());
                                }
                                if (modis.get(0) != ClassAccessFlag.PUBLIC) {
                                    throw new CompilerException("Modifier " + modis.get(0) + " isn't allowed for interface declaration", node.getSymbol());
                                }
                                if (packageName != null) {
                                    file = new ClassFile(packageName + "/" + node.getInterfaceName(), ClassAccessFlag.PUBLIC, ClassAccessFlag.INTERFACE, ClassAccessFlag.ABSTRACT);
                                } else {
                                    file = new ClassFile(node.getInterfaceName(), ClassAccessFlag.PUBLIC, ClassAccessFlag.INTERFACE, ClassAccessFlag.ABSTRACT);
                                }
                                ClassFinder.instance().addClassFile(file);
                            }
                            stage0Env.setClassFile(file);
                            file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Interface));

                        } catch (ByteCodeException | CompilerException ex) {
                            ex.printStackTrace(System.err);
                            System.exit(-1);
                        }
                    }

                    @Override
                    public void begin(AnnotationTypeDeclarationNode node)throws CompilerException {
                    }

                    @Override
                    public Visitor visit(NormalInterfaceDeclarationNode node)throws CompilerException {
                        return null;
                    }

                    @Override
                    public Visitor visit(AnnotationTypeDeclarationNode node)throws CompilerException {
                        return null;
                    }

                    @Override
                    public void end(NormalInterfaceDeclarationNode node)throws CompilerException {
                    }

                    @Override
                    public void end(AnnotationTypeDeclarationNode node)throws CompilerException {
                    }
                };
            }

            @Override
            public void end(InterfaceDeclarationNode node)throws CompilerException {
            }

            @Override
            public Object getClassFileContainer() {
                return null;
            }
        };
    }

    @Override
    public void end(ClassOrInterfaceDeclaratioNode node)throws CompilerException {
    }

    @Override
    public void begin(TjImplDeclarationNode node)throws CompilerException {
        try {
            ClassFile file;
            if (packageName != null) {
                file = new ClassFile(this.packageName + "/" + node.getImplName(), ClassAccessFlag.FINAL ,ClassAccessFlag.PUBLIC,ClassAccessFlag.SUPER);
            } else {
                file = new ClassFile(node.getImplName(), ClassAccessFlag.PUBLIC,ClassAccessFlag.FINAL,ClassAccessFlag.SUPER);
            }
            file.addAttribute(new TeaJayAttribute(TeaJayAttribute.Type.Impl));
            ClassFinder.instance().addClassFile(file);
            this.stage0Env.setClassFile(file);
            this.stage0Env.getSymbolFacade().addNameFromImport(file.getThis().getClassName());


        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public Visitor visit(TjImplDeclarationNode node)throws CompilerException {
        return null;
    }

    @Override
    public void end(TjImplDeclarationNode node)throws CompilerException {
    }
}
