/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage1;

import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.ClosureTypeDecNode;
import abstractsyntaxtree.DotIdentifierTypeArgumentsHelperNode;
import abstractsyntaxtree.EdgeClampsOpenCloseHelperNode;
import abstractsyntaxtree.ReferenceTypeNode;
import abstractsyntaxtree.TypeArgumentsNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractReferenceTypeVisitor;
import codegeneration.abstractvisitors.AbstractTypeArgumentVisitor;
import codegeneration.abstractvisitors.AbstractTypeVisitor;
import codegeneration.environment.Stage1Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage1ReferenceTypeVisitor extends AbstractReferenceTypeVisitor {

    private final Stage1Environment stage1Env;
    private final List<String> identifierList;
    private boolean hasTypeArguments = false;
    private List<String> typeArgumentIdentifier;

    Stage1ReferenceTypeVisitor(Stage1Environment stage1Env, List<String> identifierList) {
        this.stage1Env = stage1Env;
        this.identifierList = identifierList;
    }

    @Override
    public void begin(TypeArgumentsNode node) throws CompilerException {
        hasTypeArguments = true;
    }

    @Override
    public Visitor visit(TypeArgumentsNode node) throws CompilerException {
        return new Stage1GenericTypeArgumentsVisitor(stage1Env, typeArgumentIdentifier = new ArrayList<String>());
    }

    @Override
    public void end(TypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {
        if (hasTypeArguments) {
            throw new CompilerException("Inner generic parameter is not allowed", node.getSymbol());
        }
        this.identifierList.add(node.getIdentifier());
    }

    @Override
    public Visitor visit(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {

        return new Stage1DotIdentifierTypeArgumentsHelperVisitor(stage1Env, identifierList);
    }

    @Override
    public void end(DotIdentifierTypeArgumentsHelperNode node) throws CompilerException {
    }
}
