/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage1;

import abstractsyntaxtree.CompilationUnitHelperNode;
import abstractsyntaxtree.CompilationUnitNode;
import abstractsyntaxtree.ImportDeclarationNode;
import abstractsyntaxtree.TjTypeDefNode;
import abstractsyntaxtree.TypeDeclarationNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCompilationUnitVisitor;
import codegeneration.environment.Stage0Environment;
import codegeneration.environment.Stage1Environment;
import codegeneration.javavisitor.compilationunit.ImportDeclarationVisitor;
import exceptions.CompilerException;

/**
 *
 * @author Michael Burzan
 * 
 * Imports must be visited once again, due to the import order
 */
public class Stage1Visitor extends AbstractCompilationUnitVisitor {

    private final Stage1Environment stage1Env;
    private final Stage0Environment stage0Env;

    public Stage1Visitor(Stage1Environment stage1Env) {
        this.stage1Env = stage1Env;
        this.stage0Env = stage1Env.getEnv();

    }

    @Override
    public void begin(CompilationUnitNode node)throws CompilerException {
    }

    @Override
    public void visit(CompilationUnitNode node)throws CompilerException {
    }

    @Override
    public void end(CompilationUnitNode node)throws CompilerException {
    }

    @Override
    public void begin(TypeDeclarationNode node)throws CompilerException {
        this.stage1Env.setFacade(stage0Env.getSymbolFacade());
    }

    @Override
    public Visitor visit(TypeDeclarationNode node)throws CompilerException {
        return new Stage1TypeDeclarationVisitor(stage1Env);
    }

    @Override
    public void end(TypeDeclarationNode node)throws CompilerException {
        stage1Env.dropGenerics();
    }

    @Override
    public void begin(ImportDeclarationNode node)throws CompilerException {
    }

    @Override
    public Visitor visit(ImportDeclarationNode node)throws CompilerException {
        return new ImportDeclarationVisitor(node.hasWildcard(), node.isStatic(), stage0Env);
    }

    @Override
    public void end(ImportDeclarationNode node)throws CompilerException {
    }

    @Override
    public void begin(CompilationUnitHelperNode node)throws CompilerException {
    }

    @Override
    public Visitor visit(CompilationUnitHelperNode node)throws CompilerException {
        return null;
    }

    @Override
    public void end(CompilationUnitHelperNode node)throws CompilerException {
    }

    @Override
    public void begin(TjTypeDefNode node)throws CompilerException {
        this.stage1Env.setFacade(stage0Env.getSymbolFacade());
    }

    @Override
    public Visitor visit(TjTypeDefNode node)throws CompilerException {
        return new Stage1TjTypeDefVisitor(stage1Env);
    }

    @Override
    public void end(TjTypeDefNode node)throws CompilerException {
    }

}
