/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ReferenceTypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractBoundVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2BoundVisitor extends AbstractBoundVisitor {

    private final Stage2Environment stage2Env;
    private final List<String> identifierList = new ArrayList<>();
    private Type type;
    private Stage2ReferenceTypeVisitor stage2ReferenceType;

    public Stage2BoundVisitor(Stage2Environment stage2Env) {
        this.stage2Env = stage2Env;
    }

    @Override
    public void begin(ReferenceTypeNode node) throws CompilerException {
        identifierList.add(node.getName());
    }

    @Override
    public Visitor visit(ReferenceTypeNode node) throws CompilerException {
        return stage2ReferenceType = new Stage2ReferenceTypeVisitor(stage2Env, identifierList);
    }
    /**
     * Create Type object from ReferenceType
     * @param node
     * @throws CompilerException 
     */
    @Override
    public void end(ReferenceTypeNode node) throws CompilerException {
        StringBuilder identifier = new StringBuilder();

        for (int i = 0; i < identifierList.size(); i++) {
            identifier.append(identifierList.get(i));
            if (i != (identifierList.size() - 1)) {
                identifier.append("/");
            }
        }

        String ident = identifier.toString();
        String qID = stage2Env.getSymbolFacade().getFullQualifiedId(ident);
        if (qID == null) {
            qID = ident;
        }
        if (stage2Env.isTypeVariable(ident)) {
            this.type = new Type.SimpleType(ident);
        } else {
            this.type = new Type.ClassType(null, qID, stage2ReferenceType.getTypeParameters());
        }
    }

    public Type getBound() {
        return this.type;
    }
}
