/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import codegeneration.javavisitor.enums.*;
import abstractsyntaxtree.EnumBodyNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TypeListNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractEnumDeclarationVisitor;
import codegeneration.environment.Stage2Environment;
import java.util.List;
import teajay.bytecode.BaseType;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.ILoad;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
public class Stage2EnumDeclarationVisitor extends AbstractEnumDeclarationVisitor {

    private Stage2EnumBodyVisitor enumBodyVisitor;
    private final Stage2Environment stage2Env;
    private final List<ModifierNode.Modifier> modifierList;
    

    Stage2EnumDeclarationVisitor(Stage2Environment stage2Env, List<ModifierNode.Modifier> modifierList) {
        this.stage2Env = stage2Env;
        this.modifierList = modifierList;
    }

    @Override
    public void begin(TypeListNode node) {
    }

    @Override
    public void begin(EnumBodyNode node) {
        
    }

    @Override
    public Visitor visit(TypeListNode node) {
        return this;
    }

    @Override
    public Visitor visit(EnumBodyNode node) {
        this.enumBodyVisitor = new Stage2EnumBodyVisitor();
        return enumBodyVisitor;
    }

    @Override
    public void end(TypeListNode node) {
    }

    @Override
    public void end(EnumBodyNode node) {
        ClassFile classFile = stage2Env.getClassFile();
        
        if (classFile.getMethods("<init>") == null) {
            try {
                MethodDescriptor baseConstr = new MethodDescriptor(FieldDescriptor.getVoid(),
                        FieldDescriptor.fromClass(String.class), FieldDescriptor.fromBaseType(BaseType.Int));
                MethodInfo minfo = new MethodInfo("<init>", baseConstr, classFile.getThis(), MethodAccessFlag.PRIVATE);
                classFile.addMethod(minfo);
                minfo.addAttribute(new Signature("()V"));
                NameAndTypeInfo init = new NameAndTypeInfo("<init>", baseConstr);
                MethodRefInfo mref = new MethodRefInfo(classFile.getSuper(), init);
                JVMInstructionList lst = minfo.getCode().getInstructionList();
                lst.add(new ALoad(0));
                lst.add(new ALoad(1));
                lst.add(new ILoad(2));
                lst.add(new InvokeSpecial(mref));
                lst.add(new Return());
            } catch (ByteCodeException e) {
                e.printStackTrace(System.err);
                System.exit(-1);
            }
        }
    }

    public List<String> getContainer() {
        return enumBodyVisitor.getContainer();
    }
}
