/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ClassDeclarationNode;
import abstractsyntaxtree.InterfaceDeclarationNode;
import abstractsyntaxtree.InterfaceGenericMethodDeclNode;
import abstractsyntaxtree.InterfaceMethodOrFieldDeclNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.VoidInterfaceMethodDeclaratorRestNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractInterfaceMemberDeclVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.lang.reflect.Method;
import java.util.List;
import stage2.utils.Converter;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2InterfaceMemberDeclVisitor extends AbstractInterfaceMemberDeclVisitor {
    private final Stage2Environment stage2Env;
    private final String methodName;
    private final List<ModifierNode.Modifier> flagList;
    private Stage2MethodHeadVisitor stage2VoidInterfaceMethod;

    public Stage2InterfaceMemberDeclVisitor(Stage2Environment stage2Env, String identifier, List<ModifierNode.Modifier> flagList) {
        this.stage2Env = stage2Env;
        this.methodName = identifier;
        this.flagList = flagList;
        
    }

    @Override
    public void begin(InterfaceMethodOrFieldDeclNode node) throws CompilerException {
        
    }

    @Override
    public void begin(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        
    }

    @Override
    public void begin(InterfaceGenericMethodDeclNode node) throws CompilerException {
        
    }

    @Override
    public void begin(ClassDeclarationNode node) throws CompilerException {
        
    }

    @Override
    public void begin(InterfaceDeclarationNode node) throws CompilerException {

    }

    @Override
    public Visitor visit(InterfaceMethodOrFieldDeclNode node) throws CompilerException {
        MethodAccessFlag[] accessFlags = Converter.convertModifierToMethodAccesFlags(flagList);
        if(flagList.isEmpty()){
           return new Stage2InterfaceMethodOrFieldDeclVisitor(stage2Env,node.getIdentifier(),MethodAccessFlag.PUBLIC, MethodAccessFlag.ABSTRACT);
        }
        if(accessFlags.length > 1){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        if(accessFlags[0] != MethodAccessFlag.PUBLIC){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        return new Stage2InterfaceMethodOrFieldDeclVisitor(stage2Env,node.getIdentifier(),accessFlags[0], MethodAccessFlag.ABSTRACT);
    }

    @Override
    public Visitor visit(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        MethodAccessFlag[] accessFlags = Converter.convertModifierToMethodAccesFlags(flagList);
        if(flagList.isEmpty()){
           return stage2VoidInterfaceMethod  = new Stage2MethodHeadVisitor(stage2Env);
        }
        if(accessFlags.length > 1){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        if(accessFlags[0] != MethodAccessFlag.PUBLIC){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        return stage2VoidInterfaceMethod = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public Visitor visit(InterfaceGenericMethodDeclNode node) throws CompilerException {
        MethodAccessFlag[] accessFlags = Converter.convertModifierToMethodAccesFlags(flagList);
        if(flagList.isEmpty()){
           return new Stage2InterfaceGenericMethodDeclVisitor(stage2Env,node.getIdentifier(),node.isVoid(), MethodAccessFlag.PUBLIC, MethodAccessFlag.ABSTRACT);
        }
        if(accessFlags.length > 1){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        if(accessFlags[0] != MethodAccessFlag.PUBLIC){
            throw new CompilerException("Only public is allowed", node.getSymbol());
        }
        
        return new Stage2InterfaceGenericMethodDeclVisitor(stage2Env,node.getIdentifier(),node.isVoid(), accessFlags[0], MethodAccessFlag.ABSTRACT);
        
    }

    @Override
    public Visitor visit(ClassDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner class declaration not supported", node.getSymbol());
    }

    @Override
    public Visitor visit(InterfaceDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner interface declaration not supported", node.getSymbol());
    }

    @Override
    public void end(InterfaceMethodOrFieldDeclNode node) throws CompilerException {

    }

    @Override
    public void end(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        MethodHeadBuilder.buildInterfaceMethodHead(stage2Env, new MethodAccessFlag[] {MethodAccessFlag.PUBLIC, MethodAccessFlag.ABSTRACT},new Type.SimpleType("void"), methodName,
                                stage2VoidInterfaceMethod.getMethodHead(), node, null, false);
    }

    @Override
    public void end(InterfaceGenericMethodDeclNode node) throws CompilerException {
        stage2Env.dropGenericFrame();
        stage2Env.getTypeUtils().removeFrame();
    }

    @Override
    public void end(ClassDeclarationNode node) throws CompilerException {

    }

    @Override
    public void end(InterfaceDeclarationNode node) throws CompilerException {

    }
    
}
