/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ClassDeclarationNode;
import abstractsyntaxtree.ConstructorDeclaratorRestHelperNode;
import abstractsyntaxtree.GenericMethodeOrConstructorDeclNode;
import abstractsyntaxtree.InterfaceDeclarationNode;
import abstractsyntaxtree.MethodeOrFieldDeclNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.VoidMethodeDeclaratorRestHelperNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractMemberDeclVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.List;

/**
 *
 * @author Michael Burzan
 */
class Stage2MemberDeclVisitor extends AbstractMemberDeclVisitor {

    private final Stage2Environment stage2Env;
    private final List<ModifierNode> modis;

    Stage2MemberDeclVisitor(Stage2Environment stage2Env, List<ModifierNode> modis) {
        this.stage2Env = stage2Env;
        this.modis = modis;
    }

    @Override
    public void begin(MethodeOrFieldDeclNode node) throws CompilerException {
    }

    @Override
    public void begin(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {
    }

    @Override
    public void begin(ConstructorDeclaratorRestHelperNode node) throws CompilerException {
    }

    @Override
    public void begin(GenericMethodeOrConstructorDeclNode node) throws CompilerException {
    }

    @Override
    public void begin(ClassDeclarationNode node) throws CompilerException {
    }

    @Override
    public void begin(InterfaceDeclarationNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(MethodeOrFieldDeclNode node) throws CompilerException {
        if (modis != null) {
            return new Stage2MethodeOrFieldDeclVisitor(this.stage2Env, modis, node.getIdentifier());
        } else {
            return new Stage2MethodeOrFieldDeclVisitor(this.stage2Env, node.getIdentifier());
        }

    }

    @Override
    public Visitor visit(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {
        if (modis != null) {
            return new Stage2VoidMethodeDeclaratorRestHelperVisitor(stage2Env, modis, node.getMethodName());
        } else {
            return new Stage2VoidMethodeDeclaratorRestHelperVisitor(stage2Env, node.getMethodName());
        }
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestHelperNode node) throws CompilerException {
        if (modis != null) {
            return new Stage2ConstructorDeclaratorRestHelperVisitor(stage2Env, modis, node.getIdentifier());
        } else {
            return new Stage2ConstructorDeclaratorRestHelperVisitor(stage2Env, node.getIdentifier());
        }

    }

    @Override
    public Visitor visit(GenericMethodeOrConstructorDeclNode node) throws CompilerException {
        if (modis != null) {
            return new Stage2GenericMethodOrConstructorDeclVisitor(stage2Env, modis);
        } else {
            return new Stage2GenericMethodOrConstructorDeclVisitor(stage2Env);
        }
    }

    @Override
    public Visitor visit(ClassDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner class declaration not allowed", node.getSymbol());
    }

    @Override
    public Visitor visit(InterfaceDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner interface declaration not allowed",node.getSymbol());
    }

    @Override
    public void end(MethodeOrFieldDeclNode node) throws CompilerException {
    }

    @Override
    public void end(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {
    }

    @Override
    public void end(ConstructorDeclaratorRestHelperNode node) throws CompilerException {
    }

    @Override
    public void end(GenericMethodeOrConstructorDeclNode node) throws CompilerException {
        stage2Env.dropGenericFrame();
        stage2Env.getTypeUtils().removeFrame();
    }

    @Override
    public void end(ClassDeclarationNode node) throws CompilerException {
    }

    @Override
    public void end(InterfaceDeclarationNode node) throws CompilerException {
    }
}
