/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.FieldDeclaratorsRestNode;
import abstractsyntaxtree.MethodeDeclaratorRestNode;
import abstractsyntaxtree.ModifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractMethodeOrFieldRestVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;

/**
 *
 * @author Michael Burzan
 */
class Stage2MethodeOrFieldRestVisitor extends AbstractMethodeOrFieldRestVisitor {
    private final Stage2Environment stage2Env;
    private final List<ModifierNode> modis;
    private final Type type;
    private final String identifier;
    private Stage2FieldDeclaratorsRestVisitor fieldDeclVistor;
    private Stage2MethodHeadVisitor stage2MethodDeclVisitor;

    public Stage2MethodeOrFieldRestVisitor(Stage2Environment stage2Env, List<ModifierNode> modis, Type type, String identifier) {
        this.stage2Env = stage2Env;
        this.modis = modis;
        this.type = type;
        this.identifier = identifier;
    }

    @Override
    public void begin(FieldDeclaratorsRestNode node) throws CompilerException {
        
    }

    @Override
    public void begin(MethodeDeclaratorRestNode node) throws CompilerException {
        
    }

    @Override
    public Visitor visit(FieldDeclaratorsRestNode node) throws CompilerException {
        return fieldDeclVistor = new Stage2FieldDeclaratorsRestVisitor(stage2Env);
    }

    @Override
    public Visitor visit(MethodeDeclaratorRestNode node) throws CompilerException {
        return stage2MethodDeclVisitor = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(FieldDeclaratorsRestNode node) throws CompilerException {
        //checks field in typeimplementation
        FieldDescriptor makeFieldDescriptor = stage2Env.getTypeUtils().makeFieldDescriptor(type);
        FieldAccessFlag[] flag = MethodHeadBuilder.getFieldAccessFlags(identifier, modis);
        try {
            FieldInfo info = new FieldInfo(identifier, makeFieldDescriptor, flag);
            if(stage2Env.getClassFile().getField(identifier)!= null){
                throw new CompilerException("Duplicate field "+ identifier, node.getSymbol());
            }
            info.addAttribute(new Signature(SignatureEncoder.encode(type)));
            this.stage2Env.getClassFile().addField(info);
        } catch (ByteCodeException ex) {
            
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(MethodeDeclaratorRestNode node) throws CompilerException {
        
        MethodHeadBuilder.buildMethodHeadClass(node, stage2Env, type, identifier, stage2MethodDeclVisitor.getMethodHead(),MethodHeadBuilder.getMethodAccessFlags(identifier, modis) );
    }
    
}
