/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ClassBodyNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TypeListNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractNormalClassDeclarationVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.List;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 *
 * @author Michael Burzan
 */
class Stage2NormalClassDeclarationVisitor extends AbstractNormalClassDeclarationVisitor {

    private final Stage2Environment stage2Env;
    private final String className;
    private Stage2TypeParametersVisitor genericParameters;
    private Stage2TypeVisitor extendsTypeVisitor;
    private Stage2TypeListVisitor implementsVisitor;

    public Stage2NormalClassDeclarationVisitor(Stage2Environment stage2Env, List<ModifierNode.Modifier> modifierList, String className) {
        this.stage2Env = stage2Env;
        this.className = className;
        this.stage2Env.setTypeUtils(new TypeUtils());

    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        stage2Env.addGenericFrame();
        stage2Env.getTypeUtils().addFrame();
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {
        return genericParameters = new Stage2TypeParametersVisitor(stage2Env);
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return extendsTypeVisitor = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
    }

    @Override
    public void begin(TypeListNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeListNode node) throws CompilerException {
        return implementsVisitor = new Stage2TypeListVisitor(stage2Env);
    }

    @Override
    public void end(TypeListNode node) throws CompilerException {
    }

    @Override
    public void begin(ClassBodyNode node) throws CompilerException {
        Type extendsType;
        List<Type.TypeParamType> generics;
        List<Type> implementsList;
        if (genericParameters == null) {
            generics = null;
        } else {
            generics = genericParameters.getTypeParams();
        }
        if (extendsTypeVisitor == null) {
            extendsType = new Type.ClassType(null, "java/lang/Object", null);
        } else {
            extendsType = extendsTypeVisitor.getType();
            checkIsExtendsClass(extendsType, node);
        }
        if (implementsVisitor == null) {
            implementsList = null;
        } else {
            implementsList = implementsVisitor.getImplementsList();
        }

        Type.ClassSigType sigType = new Type.ClassSigType(generics, extendsType, implementsList);
        ClassFile classFile = stage2Env.getClassFile();
        try {
            classFile.addAttribute(new Signature(SignatureEncoder.encode(sigType)));

            classFile.setSuper(((Type.ClassType) extendsType).name);
            if (implementsList != null) {
                for (Type classType : implementsList) {
                    Type.ClassType c = (Type.ClassType) classType;
                    classFile.addInterface(ClassFinder.instance().find(c.name).get().getThis());
                }
            }
            stage2Env.setClassSigType(sigType);
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public Visitor visit(ClassBodyNode node) throws CompilerException {
        return new Stage2ClassBodyVisitor(stage2Env);
    }

    @Override
    public void end(ClassBodyNode node) throws CompilerException {
    }

    private void checkIsExtendsClass(Type extendsType, ClassBodyNode node) throws CompilerException {
        if (extendsType instanceof Type.ClassType) {
            Type.ClassType classType = (Type.ClassType) extendsType;
            String className = classType.name;
            try {
                AttributeInfo attribute = ClassFinder.instance().find(className).get().getAttribute(TeaJayAttribute.NAME);
                if (attribute == null) {
                    throw new CompilerException("Can only extend classes", node.getSymbol());
                }
                if (((TeaJayAttribute) attribute).getType() != TeaJayAttribute.Type.Class) {
                    throw new CompilerException("Can only extend classes", node.getSymbol());
                }
            } catch (ByteCodeException | IOException | ResolveException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        } else {
            throw new CompilerException("Can not extend from type variable " + extendsType, node.getSymbol());
        }
    }
}
