/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.InterfaceBodyNode;
import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TypeListNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractNormalInterfaceDeclarationVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage2NormalInterfaceDeclarationVisitor extends AbstractNormalInterfaceDeclarationVisitor {
    private final Stage2Environment stage2Env;
    private final List<ModifierNode.Modifier> modifierList;
    private final String interfaceName;
    private Stage2TypeParametersVisitor stage2TypeParameters;
    private Stage2TypeListVisitor stage2TypeList;

    public Stage2NormalInterfaceDeclarationVisitor(Stage2Environment stage2Env, List<ModifierNode.Modifier> modifierList, String interfaceName) {
        this.stage2Env = stage2Env;
        this.modifierList = modifierList;
        this.interfaceName = interfaceName;
        this.stage2Env.setTypeUtils(new TypeUtils());
    }

    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        this.stage2Env.getTypeUtils().addFrame();
        this.stage2Env.addGenericFrame();
    }

    @Override
    public void begin(TypeListNode node) throws CompilerException {
        
    }

    @Override
    public void begin(InterfaceBodyNode node) throws CompilerException {
        Type extendsType;
        List<Type.TypeParamType> generics;
        List<Type> implementsList;
        //is Interface = interface <T  extends Foo>
        if (stage2TypeParameters == null) {
            generics = null;
        } else {
            generics = stage2TypeParameters.getTypeParams();
        }
        // all interfaces extends java/lang/Object in TeaJay
        extendsType = new Type.ClassType(null, "java/lang/Object", null);
        
        //is Interface = interface implements Foo
        if (stage2TypeList == null) {
            implementsList = null;
        } else {
            implementsList = stage2TypeList.getImplementsList();
        }
        // create interface signature
        Type.ClassSigType sigType = new Type.ClassSigType(generics, extendsType, implementsList);
        ClassFile classFile = stage2Env.getClassFile();
        try {
            //add signature
            classFile.addAttribute(new Signature(SignatureEncoder.encode(sigType)));
            //add extendsType
            classFile.setSuper(((Type.ClassType) extendsType).name);
            //add implements List
            if (implementsList != null) {
                for (Type classType : implementsList) {
                    Type.ClassType c = (Type.ClassType) classType;
                    classFile.addInterface(ClassFinder.instance().find(c.name).get().getThis());
                }
            }
            //store signature type in environment
            stage2Env.setClassSigType(sigType);
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {
        return stage2TypeParameters = new Stage2TypeParametersVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TypeListNode node) throws CompilerException {
        return stage2TypeList = new Stage2TypeListVisitor(stage2Env);
    }

    @Override
    public Visitor visit(InterfaceBodyNode node) throws CompilerException {
        return new Stage2InterfaceBodyVisitor(stage2Env);
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {

    }

    @Override
    public void end(TypeListNode node) throws CompilerException {

    }

    @Override
    public void end(InterfaceBodyNode node) throws CompilerException {

    }
    
}
