/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.MemberDeclNode;
import abstractsyntaxtree.ModifierNode;
import static abstractsyntaxtree.ModifierNode.Modifier.SYNCHRONIZED;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractClassBodyDeclarationVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjImplClassBodyDeclarationVisitor extends AbstractClassBodyDeclarationVisitor {

    private final Stage2Environment stage2Env;
    private final Collection<List<MethodInfo>> methods;
    private List<ModifierNode.Modifier> flagList = new ArrayList<>();
    private final boolean isStaticBlock;

    public Stage2TjImplClassBodyDeclarationVisitor(Stage2Environment stage2Env, boolean isStaticBlock) {
        this.stage2Env = stage2Env;
        this.methods = this.stage2Env.getClassFile().getMethods();
        this.isStaticBlock = isStaticBlock;
    }

    @Override
    public void begin(ModifierNode node) throws CompilerException {
        //Check Modifier of field or method
        ModifierNode.Modifier modifier = node.getModifier();
        switch (modifier) {
            case PUBLIC:
                flagList.add(ModifierNode.Modifier.PUBLIC);
                break;
            //throw new CompilerException("public attribute isn't useful", node.getSymbol());
            case PROTECTED:
                throw new CompilerException("protected attribute is not supported", node.getSymbol());
            case PRIVATE:
                flagList.add(ModifierNode.Modifier.PRIVATE);
                break;
            case STATIC:
                flagList.add(ModifierNode.Modifier.STATIC);
                break;
            case SYNCHRONIZED:
                flagList.add(ModifierNode.Modifier.SYNCHRONIZED);
                break;
            case ABSTRACT:
            case FINAL:
            case NATIVE:
            case TRANSIENT:
            case VOLATILE:
            case STRICTFP:
                throw new CompilerException(modifier.toString() + " attribute is not supported", node.getSymbol());
            default:
                throw new AssertionError(modifier.name());

        }
    }

    @Override
    public void begin(MemberDeclNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        //create static constructor
        if (isStaticBlock && stage2Env.getClassFile().getMethods("<clinit>") == null) {
            try {
                
                MethodDescriptor desc = new MethodDescriptor(FieldDescriptor.getVoid());
                MethodInfo staticConstructor = new MethodInfo("<clinit>", desc, stage2Env.getClassFile().getThis(), MethodAccessFlag.STATIC);
                stage2Env.getClassFile().addMethod(staticConstructor);
                stage2Env.pushMethodInfo(staticConstructor);
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }
        } else {
            throw new CompilerException("can only have one static construtor", node.getSymbol());
        }
    }

    @Override
    public Visitor visit(ModifierNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(MemberDeclNode node) throws CompilerException {

        return new Stage2TjImplMemberDeclVisitor(stage2Env, this.flagList, this.methods);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(ModifierNode node) throws CompilerException {
    }

    @Override
    public void end(MemberDeclNode node) throws CompilerException {
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
    }
}
