/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ConstructorDeclaratorRestNode;
import abstractsyntaxtree.ModifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractConstructorDeclaratorRestHelperVisitor;
import codegeneration.environment.Stage2Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import stage2.utils.MethodHeadBuilder;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjImplConstructorDeclaratorRestHelperVisitor extends AbstractConstructorDeclaratorRestHelperVisitor {

    private final Stage2Environment stage2Env;
    private final List<ModifierNode.Modifier> flagList;
    private String constructorName;
    private final Collection<List<MethodInfo>> methods;
    private Stage2MethodHeadVisitor stage2ConstructorDeclaratorRest;
    private final String methodName = "$init$";
    private final Type.SimpleType type;

    public Stage2TjImplConstructorDeclaratorRestHelperVisitor(Stage2Environment stage2Env, List<ModifierNode.Modifier> flagList, String constructorName, Collection<List<MethodInfo>> methods) throws CompilerException {
        this.stage2Env = stage2Env;
        this.flagList = flagList;
        this.constructorName = constructorName;
        this.methods = methods;
        this.type = new Type.SimpleType("void");
    }

    @Override
    public void begin(ConstructorDeclaratorRestNode node) throws CompilerException {
        //check is constructor name = typeimpl name
        if (!stage2Env.getClassFile().getThis().getSimpleName().equals(constructorName)) {
            throw new CompilerException("Invalid constructor name " + this.constructorName, node.getSymbol());
        }
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestNode node) throws CompilerException {
        return stage2ConstructorDeclaratorRest = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(ConstructorDeclaratorRestNode node) throws CompilerException {
        // flag the user defined constructo synthetic
        LinkedList<ModifierNode.Modifier> flags = new LinkedList<>();
        flags.add(ModifierNode.Modifier.SYNTHETIC);
        flags.add(ModifierNode.Modifier.PRIVATE);
        MethodHeadBuilder.buildMethodHeadTypeImpl(node, stage2Env, type, methodName, stage2ConstructorDeclaratorRest.getMethodHead(), flags);
        
        //Formal Parameters and ThrowsList
        Pair<List<Pair<Type, String>>, List<String>> methodHead = stage2ConstructorDeclaratorRest.getMethodHead();
        
        methodHead.value1.add(0, new Pair<>( (Type) new Type.ClassType(null, stage2Env.getClassFile().getSuper().getClassName(), null),"$"));
        //Build constructor head
        if (flagList.contains(ModifierNode.Modifier.STATIC)) {
             throw new CompilerException("Constructors cannot have static modifier", node.getSymbol());
        }
        MethodHeadBuilder.buildMethodHeadTypeImpl(node, stage2Env, type, "<init>",methodHead, flagList);
        methodHead.value1.remove(0);
    }
}
