/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.ModifierNode;
import abstractsyntaxtree.TjTypeDefMemberDeclNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTjTypeDefBodyDeclarationVisitor;
import codegeneration.environment.Stage2Environment;
import exceptions.CompilerException;
import java.util.LinkedList;
import java.util.List;
import teajay.bytecode.methods.MethodAccessFlag;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjTypeDefBodyDeclarationVisitor extends AbstractTjTypeDefBodyDeclarationVisitor {
    private final Stage2Environment stage2Env;
    private List<MethodAccessFlag> flags = new LinkedList<>();
    public Stage2TjTypeDefBodyDeclarationVisitor(Stage2Environment stage2Env) {
        this.stage2Env = stage2Env;
    }

    @Override
    public void begin(ModifierNode node) throws CompilerException {
        ModifierNode.Modifier modifier = node.getModifier();
        switch(modifier){
            case PUBLIC:
                    throw new CompilerException("All methods in typedef are public", node.getSymbol());
               
            case PROTECTED:
                    throw new CompilerException("protected methods in typedef not allowed", node.getSymbol());
            case PRIVATE:
                    throw new CompilerException("private methods in typedef not allowed", node.getSymbol());
            case STATIC:
                    flags.add(  MethodAccessFlag.STATIC );
                    break;
            case ABSTRACT:
                     throw new CompilerException("abstract methods in typedef not allowed", node.getSymbol());
            case FINAL:
                throw new CompilerException("final methods in typedef not allowed", node.getSymbol());
            case NATIVE:
                throw new CompilerException("native methods in teajay not allowed", node.getSymbol());
            case SYNCHRONIZED:
                  flags.add(  MethodAccessFlag.SYNCHRONIZED );
                    break;
                //throw new CompilerException("synchronized methods in typedef not allowed", node.getSymbol());
            case TRANSIENT:
                throw new CompilerException("transient methods in typedef not supported", node.getSymbol());
            case VOLATILE:
                throw new CompilerException("voltaile methods in typedef not supported", node.getSymbol());
            case STRICTFP:
                throw new CompilerException("strictfp methods in typedef not supported", node.getSymbol());
            default:
                throw new AssertionError(modifier.name());
            
        }
    }

    @Override
    public Visitor visit(ModifierNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(ModifierNode node) throws CompilerException {

    }

    @Override
    public void begin(TjTypeDefMemberDeclNode node) throws CompilerException {

    }

    @Override
    public Visitor visit(TjTypeDefMemberDeclNode node) throws CompilerException {
        return new Stage2TjTypeDefMemberDeclVisitor(stage2Env,node.getIdentifier(),flags);
    }

    @Override
    public void end(TjTypeDefMemberDeclNode node) throws CompilerException {

    }
    
}
