/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage2;

import abstractsyntaxtree.InterfaceGenericMethodDeclNode;
import abstractsyntaxtree.InterfaceMethodDeclaratorRestNode;
import abstractsyntaxtree.SyntaxNode;
import abstractsyntaxtree.TjTypeDefConstructorDeclNode;
import abstractsyntaxtree.TypeNode;
import abstractsyntaxtree.VoidInterfaceMethodDeclaratorRestNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTjTypeDefMemberDeclVisitor;
import codegeneration.environment.Stage2Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import stage2.utils.MethodHeadBuilder;
import symbol.DerivationTree;
import teajay.bytecode.attributes.Exceptions;
import teajay.bytecode.attributes.bootstrap.BootstrapMethodEntry;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.InvokeDynamic;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.code.ReturnType;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.InvokeDynamicInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodAccessFlag;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan
 */
class Stage2TjTypeDefMemberDeclVisitor extends AbstractTjTypeDefMemberDeclVisitor {

    private final Stage2Environment stage2Env;
    private Stage2TypeVisitor stage2Type;
    private Type returnType;
    private String methodName;
    private Stage2MethodHeadVisitor stage2InterfaceMethodDeclaratorRest;
    private Stage2TjTypeDefConstructorDeclVisitor stage2TjTypeDefConstructorDecl;
    private Stage2MethodHeadVisitor stage2VoidInterfaceMethodDeclarator;
    private List<MethodAccessFlag> methodAccessFlag;
    private boolean isVoid = false;
    private Stage2TjTypeDefInterfaceGenericMethodDeclVisitor stage2GenericIfaceVisitor;

    Stage2TjTypeDefMemberDeclVisitor(Stage2Environment stage2Env, String identifier, List<MethodAccessFlag> flags) {
        this.stage2Env = stage2Env;

        this.methodName = identifier;
        this.methodAccessFlag = flags;
        this.methodAccessFlag.add(MethodAccessFlag.ABSTRACT);
    }
    

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public void begin(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public void begin(TjTypeDefConstructorDeclNode node) throws CompilerException {
        if (!node.getIdentifier().equals(stage2Env.getClassFile().getThis().getSimpleName())) {
            throw new CompilerException("Invalid constructor declaration ", node.getSymbol());
        }
        //set constructor name
        this.methodName = "<init>";
        this.returnType = new Type.SimpleType("void");
        if (this.methodAccessFlag.contains(MethodAccessFlag.STATIC)) {
            throw new CompilerException("static not allowed for constructors", node.getSymbol());
        }
        //abstract was set by previous visitor
        //constructors in typedef have a default implementation
        this.methodAccessFlag.remove(MethodAccessFlag.ABSTRACT);
        this.methodAccessFlag.add(MethodAccessFlag.PUBLIC);
    }

    @Override
    public void begin(InterfaceGenericMethodDeclNode node) throws CompilerException {
        if (!methodAccessFlag.contains(MethodAccessFlag.STATIC)) {
            throw new CompilerException("Generic methods must be static for now. (To be implemented)");
        }
        //create new frame for generic context, 
        //because the old generic context should be disregarded
        stage2Env.addGenericFrame();
        stage2Env.getTypeUtils().addFrame();
    }

    @Override
    public void begin(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        this.isVoid = true;
        this.returnType = new Type.SimpleType("void");
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return stage2Type = new Stage2TypeVisitor(stage2Env);
    }

    @Override
    public Visitor visit(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
        return stage2InterfaceMethodDeclaratorRest = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public Visitor visit(TjTypeDefConstructorDeclNode node) throws CompilerException {
        return stage2TjTypeDefConstructorDecl = new Stage2TjTypeDefConstructorDeclVisitor(stage2Env);
    }

    @Override
    public Visitor visit(InterfaceGenericMethodDeclNode node) throws CompilerException {
        return stage2GenericIfaceVisitor = new Stage2TjTypeDefInterfaceGenericMethodDeclVisitor(stage2Env, node.getIdentifier(), node.isVoid(), methodAccessFlag);
    }

    @Override
    public Visitor visit(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        return stage2VoidInterfaceMethodDeclarator = new Stage2MethodHeadVisitor(stage2Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        this.returnType = stage2Type.getType();
    }

    @Override
    public void end(InterfaceMethodDeclaratorRestNode node) throws CompilerException {
        createMethod(stage2InterfaceMethodDeclaratorRest.getMethodHead(), node);
    }

    @Override
    public void end(TjTypeDefConstructorDeclNode node) throws CompilerException {
        createMethod(stage2TjTypeDefConstructorDecl.getMethodHead(), node);
    }

    @Override
    public void end(InterfaceGenericMethodDeclNode node) throws CompilerException {
        this.returnType = stage2GenericIfaceVisitor.getReturnType();
        this.isVoid = stage2GenericIfaceVisitor.isVoid();
        this.methodName = stage2GenericIfaceVisitor.getMethodName();
        createMethod(stage2GenericIfaceVisitor.getMethodHead(), stage2GenericIfaceVisitor.getTypeParams(), node);
        stage2Env.dropGenericFrame();
        stage2Env.getTypeUtils().removeFrame();

    }

    @Override
    public void end(VoidInterfaceMethodDeclaratorRestNode node) throws CompilerException {
        createMethod(stage2VoidInterfaceMethodDeclarator.getMethodHead(), node);
    }

    private void createMethod(Pair<List<Pair<Type, String>>, List<String>> methodHead, SyntaxNode node) throws CompilerException {
        createMethod(methodHead, null, node);
    }
    /**
     * 
     * @param methodHead the parameters and the throws list
     * @param typeParams generic parameters
     * @param node
     * @throws CompilerException 
     */
    private void createMethod(Pair<List<Pair<Type, String>>, List<String>> methodHead, List<Type.TypeParamType> typeParams, SyntaxNode node) throws CompilerException {

        List<Pair<Type, String>> l = methodHead.value1;
        List<Type> typeList = new ArrayList<>();
        boolean vararg = false;
        //Build from first pair attribute a Type list
        for (Pair<Type, String> pair : l) {
            if (pair.value1 instanceof Type.ArrayType) {
                vararg = true;
            }
            typeList.add(pair.value1);
        }
        //generate a MethodType 
        Type.MethodType methodType = new Type.MethodType(typeParams, typeList, returnType, null);
        //Type.MethodType methodType = new Type.MethodType(typeList, returnType);
        //build method descriptor from MethodType
        MethodDescriptor makeMethodDescriptor = stage2Env.getTypeUtils().makeMethodDescriptor(methodType);
        try {
            MethodInfo minfo;
            //add public access flag
            methodAccessFlag.add(MethodAccessFlag.PUBLIC);
            //if method static, then is the abstract flag not allowed
            //the abstract flag was set by previous Visitor 
            if (methodAccessFlag.contains(MethodAccessFlag.STATIC)) {
                methodAccessFlag.remove(MethodAccessFlag.ABSTRACT);
            }
            //Create MethodInfo 
            if (vararg) {
                this.methodAccessFlag.add(MethodAccessFlag.VARARGS);
                minfo = new MethodInfo(methodName, makeMethodDescriptor,
                        stage2Env.getClassFile().getThis(), this.methodAccessFlag);
            } else {

                minfo = new MethodInfo(methodName, makeMethodDescriptor,
                        stage2Env.getClassFile().getThis(), this.methodAccessFlag);
            }
            //Check and Add MethodInfo
            MethodHeadBuilder.checkExistMethodInClassFile(stage2Env.getClassFile(), methodName, minfo, node);
            stage2Env.getClassFile().addMethod(minfo);
            //add signature to method info this represents the generic parameters
            minfo.addAttribute(new Signature(SignatureEncoder.encode(methodType)));
            //add throws list to method info
            if (!methodHead.value2.isEmpty()) {
                Exceptions exceptions = new Exceptions();
                ClassFileRef throwable = ClassFinder.instance().find("java/lang/Throwable");
                for (String e : methodHead.value2) {
                    String qid = stage2Env.getSymbolFacade().getFullQualifiedId(e);
                    if (qid == null) {
                        qid = e;
                    }
                    
                    exceptions.addException(new ClassInfo(qid));
                    ClassFileRef ref = ClassFinder.instance().find(qid);
                    if (ref == null) {
                        throw new CompilerException("Class not found: " + qid, node.getSymbol());
                    }
                    /* too early in the stage, no class infos avaiable yet
                    if (!DerivationTree.instance().isAssignable(throwable, ref)) {
                        throw new CompilerException("Expecting java.lang.Thowable", node.getSymbol());
                    }*/
                }
                //add exception attribute to method info
                minfo.addAttribute(exceptions);
            }
            //Build static method with call the runtime to invoke the static method from a implementation 
            if (methodAccessFlag.contains(MethodAccessFlag.STATIC)) {
                if (minfo.getCode() == null) {
                    throw new Error("Fatal error:" + minfo + " in file " + stage2Env.getClassFile().getThis().getClassName() + " has no code attribute");
                }
                //load all parameter from the register
                JVMInstructionList lst = minfo.getCode().getInstructionList();
                for (int i = 0; i < minfo.getDescriptor().getParameterCount(); i++) {
                    lst.add(new ALoad(i));
                }
                //write invoke dynamic to instruction list
                lst.add(new InvokeDynamic(stage2Env.invokeDynamic(minfo)));
                //add return to instruction list
                if (isVoid) {
                    lst.add(new Return());
                } else {
                    //minfo.getCode().getInstructionList().add(new AConstNull());
                    lst.add(new Return(ReturnType.Reference));
                }
            }
            //build the default constructor , if the method is a constructor
            if (methodName.equals("<init>")) {

                minfo.getCode().getInstructionList().add(new ALoad(0));
                minfo.getCode().getInstructionList().add(new InvokeSpecial(MethodRefInfo.make(stage2Env.getClassFile().getSuper(), "<init>", FieldDescriptor.getVoid())));
                minfo.getCode().getInstructionList().add(new Return());
            }
        } catch (ByteCodeException /*| ResolveException | IOException*/ ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }
}
