/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package stage3;

import abstractsyntaxtree.BlockStatementsNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractBlockVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.MethodState;
import stage3.states.UnreachableCodeState;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Return;
import teajay.bytecode.attributes.code.ReturnType;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
public class Stage3BlockVisitor extends AbstractBlockVisitor {

    private final Stage3Environment stage3Env;
    private Stage3BlockStatementsVisitor stage3BlockStatementsVisitor;
    private final boolean closure;

    public Stage3BlockVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.closure = false;
    }

    public Stage3BlockVisitor(Stage3Environment stage3Env, boolean closure) {
        this.stage3Env = stage3Env;
        this.closure = closure;

    }

    @Override
    public void begin(BlockStatementsNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(BlockStatementsNode node) throws CompilerException {

        return stage3BlockStatementsVisitor = new Stage3BlockStatementsVisitor(stage3Env);
    }

    @Override
    public void end(BlockStatementsNode node) throws CompilerException {
        if (closure) {
            JVMInstructionList instructionList = this.stage3Env.getMethodInfo().getCode().getInstructionList();
            MethodState mstate = stage3Env.hasState(MethodState.class);
            boolean isVoidClosure = false;
            if (mstate != null) {
                isVoidClosure = mstate.isVoidClosure();
            }
            if (isVoidClosure) {
                if (instructionList.size() > 0) {
                    if (stage3Env.hasState(EndState.class) == null && stage3Env.hasState(UnreachableCodeState.class) == null) {
                        insertVoidClosureReturn();
                    }
                } else {
                    insertVoidClosureReturn();
                }
            } else if (stage3Env.hasState(UnreachableCodeState.class) == null && stage3Env.hasState(EndState.class) == null) {
                throw new CompilerException("missing return statement", node.getSymbol());
            }
        }
    }

    private void insertVoidClosureReturn() {
        stage3Env.getExpressionEnvironment().insertCode(new AConstNull());
        stage3Env.getExpressionEnvironment().insertCode(new Return(ReturnType.Reference));
    }
    /**
     * 
     * @return true if block is empty
     */
    public boolean isEmptyBlock() {
        return stage3BlockStatementsVisitor.hasNoStatement();
    }
}
