/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.QualifiedIdentifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCatchTypeVisitor;
import codegeneration.environment.Stage3Environment;
import codegeneration.javavisitor.QualifiedIdentifierVisitor;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3CatchTypeVisitor extends AbstractCatchTypeVisitor {

    private final Stage3Environment stage3Env;
    private QualifiedIdentifierVisitor qVisitor;
    private List<Type> types = new ArrayList<>();

    public Stage3CatchTypeVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(QualifiedIdentifierNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(QualifiedIdentifierNode node) throws CompilerException {
        return qVisitor = new QualifiedIdentifierVisitor();
    }

    @Override
    /*
     * Checks is QualifiedIdentifier assignable to Throwable
     */
    public void end(QualifiedIdentifierNode node) throws CompilerException {
        //Build full qualified Identifier
        List<String> qualifiedNameList = qVisitor.getQualifiedNameList();
        String name = "";
        for (int i = 0; i < qualifiedNameList.size(); i++) {
            name += qualifiedNameList.get(i);
            if (i < qualifiedNameList.size() - 1) {
                name += "/";
            }
        }
        //check exist qualified identifier in NameResolver
        String qid = stage3Env.getSymbolFacade().getFullQualifiedId(name);
        if (qid == null) { 
            //Catch types werden nicht in stage 2 gesucht!
            throw new CompilerException("Type " + name + " not found", node.getSymbol());
        }
        //Create Type from qId
        Type type = new Type.ClassType(null, qid, null);
        //Create Throwable type
        Type throwable = new Type.ClassType(null, "java/lang/Throwable", null);
        try {
            //check the assignability 
            stage3Env.getTypeUtils().checkAssignability(throwable, type);
        } catch (TypeException ex) {
            throw new CompilerException("Incompatible types. Requiered: " + throwable + ", Found:" + type, node.getSymbol());
        }
        //add type to list of catched types
        types.add(type);
    }
    /**
     * 
     * @return list of catched types 
     */
    public List<Type> getTypes() {
        return types;
    }
    /**
     * 
     * @return the first type of catch clausel
     */
    public Type getType() {
        return types.get(0);
    }
}
