/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.CatchClauseNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractCatchesVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.TryState;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;

/**
 *
 * @author Michael Burzan
 */
class Stage3CatchesVisitor extends AbstractCatchesVisitor {

    private final Stage3Environment stage3Env;
    private Stage3CatchClauseVisitor catchClause;
    private final Label jmpOverCatches;
    private final JVMInstruction endPC;

    public Stage3CatchesVisitor(Stage3Environment stage3Env, Label jmpOverCatches, JVMInstruction ins) {
        this.stage3Env = stage3Env;
        this.jmpOverCatches = jmpOverCatches;
        this.endPC = ins;
    }

    @Override
    public void begin(CatchClauseNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(CatchClauseNode node) throws CompilerException {

        return catchClause = new Stage3CatchClauseVisitor(stage3Env, node.getIdentifier());
    }

    @Override
    public void end(CatchClauseNode node) throws CompilerException {
        TryState tryState = stage3Env.hasState(TryState.class);
        if (tryState != null) {
            if(endPC == null){
                //Write exception table
                tryState.commit(stage3Env);
            }else{
                //if last statement in try block is throw then is endPc not set
                tryState.setEndLabel(stage3Env.getMethodInfo().getCode().getInstructionList().getSuccessor(endPC));
                //Write exception table
                tryState.commit(stage3Env);
            }
            if (catchClause.hasEndStateReached()) {
                //stage3Env.changeState(new UnreachableCodeState(), node);
            } else {
                //is possible, because last statement in try-block can be return or throw
                if(jmpOverCatches != null) 
                    stage3Env.getExpressionEnvironment().insertCode(new Goto(jmpOverCatches));
            }
        } else {
            throw new Error("Fatal error");
        }

    }
    /**
     * 
     * @return true is last statement in catch block break,continue,return or throw 
     */
    public boolean hasEndStateReached() {
        return this.catchClause.hasEndStateReached();
    }
}
