/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.MemberDeclNode;
import abstractsyntaxtree.ModifierNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractClassBodyDeclarationVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.StaticConstructorState;
import symbol.LocalTable;
import teajay.bytecode.attributes.code.Return;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3ClassBodyDeclarationVisitor extends AbstractClassBodyDeclarationVisitor {

    private final Stage3Environment stage3Env;
    private final boolean isStaticBlock;
    private boolean isFieldDecl = false;
    private Stage3MemberDeclVisitor memberDeclVisitor;

    public Stage3ClassBodyDeclarationVisitor(Stage3Environment stage3Env) {
        this(stage3Env, false);
    }

    public Stage3ClassBodyDeclarationVisitor(Stage3Environment stage3Env, boolean hasStaticBlock) {
        this.stage3Env = stage3Env;
        this.isStaticBlock = hasStaticBlock;
    }

    @Override
    public void begin(ModifierNode node) throws CompilerException {
    }

    @Override
    public void begin(MemberDeclNode node) throws CompilerException {
    }

    @Override
    //visit static constructor
    public void begin(BlockNode node) throws CompilerException {
        //TODO: Anderes type util für statische methoden
        if (isStaticBlock) {
            stage3Env.initState(new StaticConstructorState());
            stage3Env.changeState(new BlockState(), node);
            stage3Env.setCurrentMethodExpectedType(new Type.SimpleType("void"));
            stage3Env.setExpressionEnvironment(new ExpressionEnvironment(stage3Env));
            stage3Env.setLocalTable(new LocalTable(stage3Env.getMethodInfo(), stage3Env.getTypeUtils()));
            stage3Env.getLocalTable().addFrame();
            stage3Env.writeFields(node);
        } else {
            throw new CompilerException("instance initializer not supported", node.getSymbol());
        }
    }

    @Override
    public Visitor visit(ModifierNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(MemberDeclNode node) throws CompilerException {
        return memberDeclVisitor = new Stage3MemberDeclVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(ModifierNode node) throws CompilerException {
    }

    @Override
    public void end(MemberDeclNode node) throws CompilerException {
        this.isFieldDecl = memberDeclVisitor.isFieldDeclaration();
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        if (isStaticBlock) {
            stage3Env.getExpressionEnvironment().insertCode(new Return());
            stage3Env.initState(null);
            stage3Env.removeMethodInfo();
        }
    }
    
    public boolean isFieldDeclaration(){
        return this.isFieldDecl;
    }
}
