/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArgumentsNode;
import abstractsyntaxtree.ClassBodyNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractClassCreatorRestVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.MethodInvokeCreator;
import datastructs.Pair;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.AAStore;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.ANewArray;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Swap;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;
import teajay.methods.MethodFinder;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFinder;

/**
 *
 * @author Michael Burzan | Kußmann
 */
class Stage3ClassCreatorRestVisitor extends AbstractClassCreatorRestVisitor {

    private final Stage3Environment stage3Env;
    private Type typeOfTypeDef;
    private Type resultType;
    private final boolean hasNull;

    public Stage3ClassCreatorRestVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.resultType = this.stage3Env.getExpressionEnvironment().typeOfResult();
        throw new Error("unsupported");
    }

    Stage3ClassCreatorRestVisitor(Stage3Environment stage3Env, Type type, boolean isTypedef, boolean hasNull) {
        this.stage3Env = stage3Env;
        if (isTypedef) {
            this.typeOfTypeDef = type;
        } else {
            this.resultType = type;
        }
        this.hasNull = hasNull;
    }

    @Override
    public void begin(ArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(ClassBodyNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ArgumentsNode node) throws CompilerException {
        return new Stage3ArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ClassBodyNode node) throws CompilerException {
        return new Stage3ClassBodyVisitor(stage3Env);
    }

    @Override
    public void end(ArgumentsNode node) throws CompilerException {
        
        try {
            //TODO: Aufräumen, die fälle sollten sich mit MethodInvokeCreator.makeConstructorCall(); beide realisieren lassen
            ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
            if (typeOfTypeDef != null) {// InvokeDynamic
                if (!(typeOfTypeDef instanceof Type.ClassType)) {
                    throw new CompilerException("Invalid type " + typeOfTypeDef + " for new operator", node.getSymbol());
                }
                ClassFile cType = ClassFinder.instance().find(((Type.ClassType) typeOfTypeDef).name).get();
                //MethodInfo constructorMethod = MethodResolver.existMethod(stage3Env.getClassFile(), cType.getThis().getClassName(), "<init>", env.getCallParameters());
                Pair<MethodInfo, MethodFinder.DistanceTailPair> findMethod = MethodFinder.findMethod("<init>", cType.getThis().getClassName(), stage3Env.getClassFile().getThis().getClassName(), node, MethodFinder.QueryType.NON_VIRTUAL_NON_STATIC, env.getCallParameters());

                if (findMethod == null) {
                    throw new CompilerException("Constructor not found in typedef " + cType.getThis().getSimpleName(), node.getSymbol());
                }
                MethodInfo constructorMethod = findMethod.value1;
                Type.MethodType methodType = stage3Env.getTypeUtils().getMethodSignatureByEnv(constructorMethod, (Type.ClassType) typeOfTypeDef, Collections.EMPTY_LIST);

                if (findMethod.value2.tailLength != 0) { //Handle vararg constructors
                    int tailLengthForVarArgs = findMethod.value2.tailLength;
                    JVMInstructionList ins = stage3Env.getMethodInfo().getCode().getInstructionList();
                    ins.pushInt(tailLengthForVarArgs);
                    List<FieldDescriptor> params = constructorMethod.getDescriptor().getParameters();
                    FieldDescriptor varArgArray = params.get(params.size() - 1);
                    ins.add(new ANewArray(varArgArray.getElementType().getClassInfo()));
                    int arrayIndex = env.getStage3Env().getLocalTable().getNextId();
                    ins.add(new AStore(arrayIndex));
                    for (int i = tailLengthForVarArgs - 1; i >= 0; i--) {
                        ins.add(new ALoad(arrayIndex));
                        ins.add(new Swap());
                        ins.pushInt(i);
                        ins.add(new Swap());
                        ins.add(new AAStore());
                    }
                    ins.add(new ALoad(arrayIndex));

                    List<Type> paramTypes = new ArrayList<>();
                    paramTypes.addAll(methodType.paramTypes.subList(0, methodType.paramTypes.size() - 1));

                    Type.ArrayType last = (Type.ArrayType) methodType.paramTypes.get(methodType.paramTypes.size() - 1);
                    for (int i = 0; i < tailLengthForVarArgs; i++) {
                        paramTypes.add(last.elemType);
                    }
                    methodType = new Type.MethodType(methodType.typeParamTypes, paramTypes, methodType.returnType, methodType.throwsTypes);
                }
                stage3Env.getTypeUtils().checkCallability(methodType, env.getCallTypeParameters());
                {
                    MethodInfo oldInfo = constructorMethod;
                    MethodDescriptor descr = oldInfo.getDescriptor();
                    FieldDescriptor[] newParams = new FieldDescriptor[descr.getParameterCount() + 1];
                    newParams[0] = FieldDescriptor.fromClassFile(cType);
                    for (int i = 1; i < newParams.length; i++) {
                        newParams[i] = descr.getParameters().get(i - 1);
                    }
                    MethodDescriptor desc = new MethodDescriptor(stage3Env.getTypeUtils().makeFieldDescriptor(typeOfTypeDef), newParams);
                    stage3Env.invokeDynamic(this.stage3Env.getMethodInfo(), desc, cType.getThis());
                    
                    env.popType();
                    
                    env.setResultType(typeOfTypeDef);
                }

            } else {
                if (!(resultType instanceof Type.ClassType)) {
                    throw new CompilerException("Invalid type " + resultType + " for new operator", node.getSymbol());
                }
                Type.ClassType t = (Type.ClassType) resultType;
                MethodInvokeCreator.makeConstructorCall(env, t.name, false, node);
                if (hasNull) {
                    env.popType();
                }
                env.setResultType(resultType);

            }
            env.resetCallParameters();
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        } catch (TypeException ex) {
            //ex.printStackTrace(System.err);
            throw new CompilerException("Invalid parameters for constructor call: " + ex.getMessage(), node.getSymbol());
        }

    }

    @Override
    public void end(ClassBodyNode node) throws CompilerException {
    }
}
