/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.AssignmentOperatorNode;
import abstractsyntaxtree.BasicTypeNode.BasicType;
import abstractsyntaxtree.Et1Node;
import abstractsyntaxtree.Expression1Node;
import abstractsyntaxtree.Expression1RestNode;
import abstractsyntaxtree.Expression2RestNode;
import abstractsyntaxtree.Expression3Node;
import abstractsyntaxtree.InfixOpNode;
import abstractsyntaxtree.PostfixOpNode;
import abstractsyntaxtree.PrefixOpNode;
import abstractsyntaxtree.PrimaryNode;
import abstractsyntaxtree.SelectorNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractExpression3Visitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import codegeneration.instruction.MethodInvokeCreator;
import codegeneration.instruction.VariableInstructionCreator;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;
import stage3.states.BlockState;
import stage3.states.MethodState;
import symbol.FieldTypeResolver;
import symbol.LocalEntry;
import teajay.bytecode.attributes.code.Dup;
import teajay.bytecode.attributes.code.GetStatic;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.Ldc;
import teajay.bytecode.attributes.code.NOP;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldInfo;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;

/**
 *
 * @author Michael Burzan
 */
class Stage3Expression3Visitor extends AbstractExpression3Visitor {

    private final Stage3Environment stage3Env;
    private final boolean hasDup;
    private final Deque<InfixOpNode.InfixOps> stack;
    private Deque<InfixOpNode.InfixOps> exp1Stack;
    private Deque<InfixOpNode.InfixOps> exp3Stack = new LinkedList<>();
    private boolean hasSelector = false;
    private AssignmentOperatorNode.Operators assignmentOperator;
    private boolean isPrimaryThisOrSuper;
    private Stage3PrimaryVisitor stage3PrimaryVisitor;
    private Stage3SelectorVisitor stage3SelectorVisitor;

    Stage3Expression3Visitor(Stage3Environment stage3Env, Deque<InfixOpNode.InfixOps> stack, boolean hasDup) {
        this.stage3Env = stage3Env;
        this.stack = stack;
        this.hasDup = hasDup;
    }

    @Override
    public void begin(PrefixOpNode node) throws CompilerException {

        switch (node.getPrefix()) {
            case ADD_ADD:
                throw new CompilerException("Operator ++ not suported in teajay", node.getSymbol());
            case SUB_SUB:
                throw new CompilerException("Operator -- not suported in teajay", node.getSymbol());
            case INVERT:
                throw new CompilerException("Operator ~ not suported in teajay", node.getSymbol());
            case ADD:
                break;
            case NOT:
            case SUB:
                //Save Prefixoperator
                this.stage3Env.getExpressionEnvironment().pushPrefix(node.getPrefix());
                break;
            default:
                throw new AssertionError(node.getPrefix().name());
        }
    }

    @Override
    public void begin(Expression3Node node) throws CompilerException {
        //rename stack, if prefix operator 
        exp3Stack = stack;
        stage3Env.getExpressionEnvironment().setRValue();
    }

    @Override
    public void begin(PrimaryNode node) throws CompilerException {
        if (node.isPlainIdent()) {
            stage3Env.writeSuperConstructorCall(node);
        }
        try {
            ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
            if (node.isThis() || node.isSuper()) {
                
                if (env.staticContext()) {
                    throw new CompilerException("Cannot access non static methods from a static context", node.getSymbol());
                }
                //Load this or super field
                env.writeThisOrSuper(node.isThis());
                if (node.isThis()) {
                    MethodState mstate = stage3Env.hasState(MethodState.class);
                    if (mstate != null && mstate.isClosure()) {
                        env.popType();
                        env.setResultType(mstate.getClosureType());
                    }
                }
            } else if (node.isVoidClass()) {
                FieldRefInfo fref = FieldRefInfo.make("java/lang/Void", "TYPE", "java/lang/Class");
                env.insertCode(new GetStatic(fref));
                env.setResultType(new Type.ClassType(null, "java/lang/Class",
                        Arrays.<Type>asList(new Type.ClassType(null, "java/lang/Void", null))));
            } else if (node.hasDotClass()) {
                BasicType type = node.getBasicType();
                int dim = node.getDim();
                if (dim == 0) {
                    throw new CompilerException(type + ".class not supported", node.getSymbol());
                }
                FieldDescriptor makeArray = FieldDescriptor.makeArray(FieldDescriptor.fromString(type.toString()), dim);
                env.insertCode(new Ldc(makeArray.getClassInfo()));
                env.setResultType(new Type.ClassType(null, "java/lang/Class",
                        Arrays.<Type>asList(new Type.WildcardType())));
            } else {
                //Load Or Store variable
                String ident = node.getIdentifier();
                VariableInstructionCreator varInstructionCreator = env.getVariableInstructionCreator();
                if (varInstructionCreator == null) {
                    throw new CompilerException("varInstructionIs Null", node.getSymbol());
                }
                if (ident != null) {
                    LocalEntry entry = stage3Env.getLocalTable().getEntry(ident);
                    if (entry != null) { //Is identifier a local variable
                        varInstructionCreator.createLocalVariableInstruction(entry, ident, node.getIdentifierList(), node);
                    } else {//is identifier a field 
                        FieldInfo field = FieldTypeResolver.find(env.getFile(), ident);
                        if (field != null) {
                            varInstructionCreator.createFieldInstruction(ident, node.getIdentifierList(), false, node);
                        } else {
                            //check is identifier only a name
                            if (node.getIdentifierList() == null && node.getIdentifierList().isEmpty()) {
                                throw new CompilerException("Local or field variable " + ident + " not found", node.getSymbol());
                            }
                            //last resort: Is a ClassName (Java support)
                            varInstructionCreator.createClassInstruction(ident, node.getIdentifierList(), node);
                        }
                    }
                }
            }
        } catch (ByteCodeException | ResolveException | IOException e) {
            e.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void begin(SelectorNode node) throws CompilerException {
        this.hasSelector = true;
    }

    @Override
    public void begin(PostfixOpNode node) throws CompilerException {
    }

    @Override
    public void begin(Expression2RestNode node) throws CompilerException {
    }

    @Override
    public void begin(Expression1RestNode node) throws CompilerException {
    }

    @Override
    public void begin(AssignmentOperatorNode node) throws CompilerException {
        this.assignmentOperator = node.getOperator();
    }

    @Override
    public void begin(Expression1Node node) throws CompilerException {
        this.stage3Env.getExpressionEnvironment().writePrefix(node);

        switch (this.assignmentOperator) {
            case SET:
                //NOT USE in this case
                break;
            case ADD_SET:
                stack.push(InfixOpNode.InfixOps.ADD);
                break;
            case SUB_SET:
                stack.push(InfixOpNode.InfixOps.SUB);
                break;
            case MULT_SET:
                stack.push(InfixOpNode.InfixOps.MULT);
                break;
            case DIV_SET:
                stack.push(InfixOpNode.InfixOps.DIV);
                break;
            case AND_SET:
                stack.push(InfixOpNode.InfixOps.BIT_AND);
                break;
            case OR_SET:
                stack.push(InfixOpNode.InfixOps.BIT_OR);
                break;
            case INV_SET:
            case MOD_SET:
            case LEFT_SHIFT_SET:
            case RIGHT_SHIFT_SET:
            case DOUBLE_RIGHT_SHIFT_SET:
            default:
                throw new CompilerException("Operator " + this.assignmentOperator.name() + " not supported", node.getSymbol());

        }
    }

    @Override
    public void begin(Et1Node node) throws CompilerException {
    }

    @Override
    public void visit(PrefixOpNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(Expression3Node node) throws CompilerException {
        return new Stage3Expression3Visitor(stage3Env, exp3Stack, true);
    }

    @Override
    public Visitor visit(PrimaryNode node) throws CompilerException {
        this.stage3PrimaryVisitor = new Stage3PrimaryVisitor(stage3Env, stack);
        return this.stage3PrimaryVisitor;
    }

    @Override
    public Visitor visit(SelectorNode node) throws CompilerException {
        return this.stage3SelectorVisitor = new Stage3SelectorVisitor(stage3Env, node.getIdentifier(), node.isClosureCall());
    }

    @Override
    public void visit(PostfixOpNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(Expression2RestNode node) throws CompilerException {
        return new Stage3Expression2RestVisitor(stage3Env, stack);
    }

    @Override
    public Visitor visit(Expression1RestNode node) throws CompilerException {

        return new Stage3Expression1RestVisitor(stage3Env);
    }

    @Override
    public void visit(AssignmentOperatorNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(Expression1Node node) throws CompilerException {
        if (!this.hasSelector) {
            this.stage3Env.getExpressionEnvironment().writePrefix(node);
        }

        this.exp1Stack = new LinkedList<>();
        stage3Env.createNewExpressionEnvironment();
        return new Stage3Expression1Visitor(stage3Env, exp1Stack, true);
    }

    @Override
    public Visitor visit(Et1Node node) throws CompilerException {
        return new Stage3Et1Visitor(stage3Env);
    }

    @Override
    public void end(PrefixOpNode node) throws CompilerException {
    }

    @Override
    public void end(Expression3Node node) throws CompilerException {

        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();

        if (!this.hasSelector) {
            env.writePrefix(node);
        }

        InstructionCreator.PutOperatorInstructions(env, exp3Stack, node);

    }

    @Override
    public void end(PrimaryNode node) throws CompilerException {
        this.isPrimaryThisOrSuper = node.isSuper() || node.isThis() || this.stage3PrimaryVisitor.hasSuperSuffix();
    }

    @Override
    public void end(SelectorNode node) throws CompilerException {
        //!!! fix this.a.b.c is being ignored without ()
        //This doesn't fix 'this;' being a valid primary statement, but that is a grammar bug
        if (!this.stage3SelectorVisitor.hasArguments()) {
            throw new CompilerException("Cannot access fields via this", node.getSymbol());
        }
        stage3Env.getExpressionEnvironment().writePrefix(node);
    }

    @Override
    public void end(PostfixOpNode node) throws CompilerException {
    }

    @Override
    public void end(Expression2RestNode node) throws CompilerException {
        this.stage3Env.getExpressionEnvironment().writePrefix(node);
    }

    @Override
    public void end(Expression1RestNode node) throws CompilerException {
    }

    @Override
    public void end(AssignmentOperatorNode node) throws CompilerException {
    }

    @Override
    public void end(Expression1Node node) throws CompilerException {
        try {
            ExpressionEnvironment exp1Env = stage3Env.getExpressionEnvironment();
            stage3Env.restoreOldExpressionEnvironment();
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            //write prefix if exists
            if (!this.hasSelector) {
                this.stage3Env.getExpressionEnvironment().writePrefix(node);
            }
            //Check is expression : a = exp op exp | -exp | method invoke= b
            if (env.isRvalue()) {
                throw new CompilerException("Expected variable but found value.", node.getSymbol());
            }
            //write operator to instruction list
            InstructionCreator.PutOperatorInstructions(exp1Env, exp1Stack, node);
            try {
                //exp:= a += 2
                if (this.assignmentOperator != null
                        && this.assignmentOperator != AssignmentOperatorNode.Operators.SET
                        && !env.isLValueCall()) {
                    if (this.isPrimaryThisOrSuper) {
                        throw new CompilerException("Cannot access fields via this or super", node.getSymbol());
                    }
                    env.setResultType(exp1Env.typeOfResult());
                    //Write +,-,* or \
                    InstructionCreator.PutOperatorInstructions(env, stack, node);
                    exp1Env.setResultType(env.typeOfResult());
                    //exp:= b = a +=2
                    if (hasDup) {
                        env.insertCode(new Dup());
                    }
                    //Store value
                    env.writeLValueInstruction();
                    stage3Env.getTypeUtils().checkAssignability(env.getLValueResultType(), exp1Env.typeOfResult());
                    //consume type from stack
                    env.popType();

                } else {
                    // exp:= a[exp] = exp
                    if (env.isLValueCall()) {
                        env.setLValueCall(false);
                        Type toSetParam;
                        //exp:= a[exp] += exp | a[exp] -= exp | a[exp] *= exp | a[exp] \= exp
                        if (env.getLValueOperator() != AssignmentOperatorNode.Operators.SET) {
                            env.setResultType(exp1Env.typeOfResult());
                            InstructionCreator.PutOperatorInstructions(env, stack, node);
                            toSetParam = env.popType();
                        } else {
                            toSetParam = exp1Env.typeOfResult();
                        }
                        //push type result:= [exp]
                        env.pushCallParameter(env.getLValueCallTypeParam());
                        env.pushCallParameter(toSetParam);
                        //Call operator_set
                        MethodInvokeCreator.CallContext callContext = new MethodInvokeCreator.CallContext(stage3Env, env.typeOfResult(), "operator_set", 1);
                        MethodInvokeCreator.createMethodCall(callContext, env, "operator_set", null, node);
                    } else {
                        // exp := exp = exp op = exp
                        if (hasDup) {
                            if (!exp1Env.isShortCircuit()) {

                                env.insertCode(new Dup());
                                env.setResultType(exp1Env.typeOfResult());
                                env.setResultType(exp1Env.typeOfResult());
                            } else {
                                //exp := exp = exp && exp | exp = exp || exp
                                BlockState hasState = stage3Env.hasState(BlockState.class);
                                Label label = new Label(" Assignement short circuit gotoLabel");
                                List<Type> merged = new ArrayList<>();
                                merged.addAll(env.getOperandStack());
                                exp1Env.getShortCircuit().write(stage3Env, label, merged.toArray(new Type[0]));
                                hasState.addLabel(label);
                                merged.add(new Type.ClassType(null, "java/lang/Boolean", null));
                                stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(), merged.toArray(new Type[0]));
                                env.insertCode(new Dup());
                                env.setResultType(exp1Env.typeOfResult());
                                env.setResultType(exp1Env.typeOfResult());
                                env.setShortCircuit(null);
                            }
                        } else if (exp1Env.isShortCircuit()) {
                            // exp := func(exp && exp)
                            BlockState hasState = stage3Env.hasState(BlockState.class);
                            Label label = new Label(" Assignement short circuit gotoLabel");
                            List<Type> merged = new ArrayList<>();
                            merged.addAll(env.getOperandStack());
                            // merged.addAll(exp1Env.getOperandStack());
                            exp1Env.getShortCircuit().write(stage3Env, label, merged.toArray(new Type[0]));
                            hasState.addLabel(label);
                            merged.add(new Type.ClassType(null, "java/lang/Boolean", null));
                            stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(), merged.toArray(new Type[0]));
                            env.setResultType(exp1Env.typeOfResult());
                            env.setShortCircuit(null);
                        }

                        //write Lvalue = exp
                        env.writeLValueInstruction();

                        env.setResultType(exp1Env.typeOfResult(), false);
                        try {
                            stage3Env.getTypeUtils().checkAssignability(env.getLValueResultType(), exp1Env.typeOfResult());
                        } catch (NoSuchElementException ex) {
                            throw new CompilerException("Cannot access fields via this or super", node.getSymbol());
                        }
                        //remove LValue result type
                        env.popType();

                    }
                }

            } catch (TypeException ex) {
                //ex.printStackTrace(System.err);
                throw new CompilerException("Expected: " + env.getLValueResultType() + " Found: " + exp1Env.typeOfResult(), node.getSymbol());
            }

        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

    }

    @Override
    public void end(Et1Node node) throws CompilerException {
    }
}
/*
 @Override
 public void begin(Expression3Node node) throws CompilerException {
        
 /*if (stack.isEmpty() || this.stack.contains(InfixOpNode.InfixOps.EQL)
 || this.stack.contains(InfixOpNode.InfixOps.NEQL)
 || this.stack.contains(InfixOpNode.InfixOps.LESS)
 || this.stack.contains(InfixOpNode.InfixOps.LESS_EQUAL)
 || this.stack.contains(InfixOpNode.InfixOps.GREATER_EQUAL)
 || this.stack.contains(InfixOpNode.InfixOps.GREATER)) {
 exp3Stack = new LinkedList<>();
 hasAssignment = true;
 } else {*/
//      exp3Stack = stack;
//}
//    stage3Env.getExpressionEnvironment().setRValue();
   // }
