/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.ForControlNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractForStatementVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.LoopState;
import teajay.bytecode.attributes.code.Goto;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Burzan
 */
public class Stage3ForStatementVisitor extends AbstractForStatementVisitor {

    private final Stage3Environment stage3Env;
    private Stage3ForControlVisitor forControlVisitor;
    private Label breakLabel;
    private Label continueLabel;

    public Stage3ForStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(ForControlNode node) throws CompilerException {
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        stage3Env.changeState(new BlockState(), node);
    }

    @Override
    public Visitor visit(ForControlNode node) throws CompilerException {
        stage3Env.getLocalTable().addFrame();
        return forControlVisitor = new Stage3ForControlVisitor(stage3Env);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return new Stage3BlockVisitor(stage3Env);
    }

    @Override
    public void end(ForControlNode node) throws CompilerException {
        this.breakLabel = forControlVisitor.getLabel();
        this.continueLabel = forControlVisitor.getLoopLabel();
        LoopState state = stage3Env.hasState(LoopState.class);
        state.setLoopLabel(continueLabel);
        state.setBreakLabel(breakLabel);
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        /*EndState endState = stage3Env.hasState(EndState.class);
        if (endState != null) {
            stage3Env.prevState();
        }
        stage3Env.prevState();
        stage3Env.getExpressionEnvironment().insertCode(new Goto(continueLabel));
        stage3Env.getLocalTable().removeFrame();*/
        EndState endState = stage3Env.hasState(EndState.class);
        if (endState != null) {
            stage3Env.prevState();
            stage3Env.prevState();
            stage3Env.getLocalTable().removeFrame();
        } else {
            //last statement in block is not break,continue,return or throw
            stage3Env.prevState();
            //wriet jump to loop label
            stage3Env.getExpressionEnvironment().insertCode(new Goto(continueLabel));
            stage3Env.getLocalTable().removeFrame();
        }

    }

    public Label getLabel() {
        return this.breakLabel;
    }
}
