/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.ForUpdateNode;
import abstractsyntaxtree.ForVariableDeclaratorsRestNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractForVarControlRestVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.util.Arrays;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import symbol.LocalEntry;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.CheckCast;
import teajay.bytecode.attributes.code.IMode;
import teajay.bytecode.attributes.code.If0;
import teajay.bytecode.attributes.code.InvokeInterface;
import teajay.bytecode.attributes.code.InvokeStatic;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.cpool.ClassInfo;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.InterfaceMethodRefInfo;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.cpool.NameAndTypeInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;
import teajay.lang.TJIterator;
import teajay.runtime.StringType;

/**
 *
 * @author Michael (Kußmann | Burzan)
 */
class Stage3ForVarControlRestVisitor extends AbstractForVarControlRestVisitor {

    private final Stage3Environment stage3Env;
    private final Type type;
    private final String name;
    private Label label;
    private Label loopLabel;

    public Stage3ForVarControlRestVisitor(Stage3Environment stage3Env, Type type, String name) {
        this.stage3Env = stage3Env;
        this.type = type;
        this.name = name;
    }

    @Override
    public void begin(ForVariableDeclaratorsRestNode node) throws CompilerException {
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        if (stage3Env.getLocalTable().getEntry(name) != null) {
            throw new CompilerException("Variable " + name + " already defined", node.getSymbol());
        }
        label = new Label("jmpOver");
        loopLabel = new Label("loopLabel");
        this.stage3Env.setExpressionEnvironment(new ExpressionEnvironment(stage3Env));
    }

    @Override
    public void begin(ForUpdateNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ForVariableDeclaratorsRestNode node) throws CompilerException {
        return new Stage3ForVariableDeclaratorsRestVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        Deque<InfixOpNode.InfixOps> stack = new LinkedList<>();
        return new Stage3ExpressionVisitor(this.stage3Env, stack, false);
    }

    @Override
    public Visitor visit(ForUpdateNode node) throws CompilerException {
        return null;
    }

    @Override
    public void end(ForVariableDeclaratorsRestNode node) throws CompilerException {
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        Type typeOfResult = stage3Env.getExpressionEnvironment().typeOfResult();
        List<Type> param = Arrays.<Type>asList(new Type.WildcardType(Type.WildcardType.Kind.EXTENDS, type));
        
        Type.ClassType iterableType = new Type.ClassType(null, "java/lang/Iterable", param);
        Type.ClassType tjIterableType = new Type.ClassType(null, "teajay/lang/TJIterable", param);
        InterfaceMethodRefInfo iteratorRef;
        try {
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            JVMInstruction iteratorGet;
            String iteratorType;
            FieldDescriptor returnType = FieldDescriptor.fromString("java/lang/Boolean");
            if (stage3Env.getTypeUtils().makeFieldDescriptor(typeOfResult).getClassInfo().getClassName().equals("java/lang/String")) {
                stage3Env.getTypeUtils().checkAssignability(type, new Type.ClassType(null, "java/lang/String", null));
                iteratorType = "teajay/lang/TJIterator";
                iteratorGet = new InvokeStatic(MethodRefInfo.make(ClassInfo.fromClass(StringType.class),
                        "iterator", FieldDescriptor.fromClass(TJIterator.class), FieldDescriptor.fromClass(String.class)));
            } else {
                try {
                    iteratorType = "teajay/lang/TJIterator";
                    stage3Env.getTypeUtils().checkAssignability(tjIterableType, typeOfResult);
                    iteratorRef = InterfaceMethodRefInfo.make(tjIterableType.name, "iterator", iteratorType);
                } catch (TypeException ex) {
                    //ex.printStackTrace(System.err);
                    iteratorType = "java/util/Iterator";
                    returnType = FieldDescriptor.fromClass(boolean.class);
                    stage3Env.getTypeUtils().checkAssignability(iterableType, typeOfResult);
                    iteratorRef = InterfaceMethodRefInfo.make(iterableType.name, "iterator", iteratorType);

                }
                iteratorGet = new InvokeInterface(iteratorRef);
            }

            InterfaceMethodRefInfo hasNext = new InterfaceMethodRefInfo(iteratorType,
                    new NameAndTypeInfo<>("hasNext",
                    new MethodDescriptor(returnType)));
            InterfaceMethodRefInfo next = new InterfaceMethodRefInfo(iteratorType,
                    new NameAndTypeInfo<>("next",
                    new MethodDescriptor(FieldDescriptor.fromString("java/lang/Object"))));

            env.insertCode(iteratorGet);

            stage3Env.getLocalTable().addEntry("$" + name + "_Iterator$", new Type.ClassType(null, iteratorType, null));
            LocalEntry entry = stage3Env.getLocalTable().getEntry("$" + name + "_Iterator$");
            env.insertCode(new AStore(entry.getIndex()));

            JVMInstructionList instructionList = stage3Env.getMethodInfo().getCode().getInstructionList();
            stage3Env.getLocalTable().addStackMapFrame(instructionList.add(this.loopLabel, new ALoad(entry.getIndex())));

            env.insertCode(new InvokeInterface(hasNext));
            if (iteratorType.equals("teajay/lang/TJIterator")) {
                MethodRefInfo booleanValue = MethodRefInfo.make("java/lang/Boolean", "booleanValue", "boolean");
                env.insertCode(new InvokeVirtual(booleanValue));
            }
            env.insertCode(new If0(IMode.eq, this.label));
            env.insertCode(new ALoad(entry.getIndex()));
            env.insertCode(new InvokeInterface(next));
            
            FieldDescriptor erasure = stage3Env.getTypeUtils().makeFieldDescriptor(type);
            env.insertCode(new CheckCast(erasure.getClassInfo()));
            
            stage3Env.getLocalTable().addEntry(name, type);
            env.insertCode(new AStore(stage3Env.getLocalTable().getEntry(name).getIndex()));

        } catch (TypeException ex) {
            throw new CompilerException("Type " + typeOfResult + " is not iterable", node.getSymbol());
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(ForUpdateNode node) throws CompilerException {
    }

    public Label getLabel() {
        return this.label;
    }

    public Label getLoopLabel() {
        return this.loopLabel;
    }
}
