/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package stage3;

import abstractsyntaxtree.GenericMethodeOrConstructorRestNode;
import abstractsyntaxtree.TypeParametersNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractGenericMethodeOrConstructorDeclVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.util.List;
import stage3.states.MethodState;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
class Stage3GenericMethodeOrConstructorDeclNode extends AbstractGenericMethodeOrConstructorDeclVisitor {
    
    private final Stage3Environment stage3Env;
    private List<Type.TypeParamType> typeParams;
    private Stage3GenericMethodeOrConstructorRestVisitor restVisitor;

    public Stage3GenericMethodeOrConstructorDeclNode(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }
    
    @Override
    public void begin(TypeParametersNode node) throws CompilerException {
        stage3Env.addGenericFrame();
        this.typeParams = node.getGeneric();
        TypeUtils util = stage3Env.getTypeUtils();
        util.addFrame();
        for (Type.TypeParamType type : typeParams) {
            util.addTypeToContext(type);
            stage3Env.addTypeVariable(type.name);
        }
    }

    @Override
    public void begin(GenericMethodeOrConstructorRestNode node) throws CompilerException {
        stage3Env.getMethodInfo();
        stage3Env.initState(new MethodState());
    }

    @Override
    public Visitor visit(TypeParametersNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(GenericMethodeOrConstructorRestNode node) throws CompilerException {
        return restVisitor = new Stage3GenericMethodeOrConstructorRestVisitor(stage3Env);
    }

    @Override
    public void end(TypeParametersNode node) throws CompilerException {
    }

    @Override
    public void end(GenericMethodeOrConstructorRestNode node) throws CompilerException {
        stage3Env.dropGenericFrame();
        stage3Env.getTypeUtils().removeFrame();
        //stage3Env.removeMethodInfo();
    }

}
