/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package stage3;

import abstractsyntaxtree.ConstructorDeclaratorRestNode;
import abstractsyntaxtree.MethodeDeclaratorRestNode;
import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractGenericMethodeOrConstructorRestVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
class Stage3GenericMethodeOrConstructorRestVisitor extends AbstractGenericMethodeOrConstructorRestVisitor {
    private final Stage3Environment stage3Env;
    private Type returnType;

    public Stage3GenericMethodeOrConstructorRestVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
        this.returnType = node.getType();
    }

    @Override
    public void begin(MethodeDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public void begin(ConstructorDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return null;
    }

    @Override
    public Visitor visit(MethodeDeclaratorRestNode node) throws CompilerException {
        MethodInfo info = stage3Env.getMethodInfo();
        if (returnType == null) {
            returnType = new Type.SimpleType("void");
        }
        stage3Env.setCurrentMethodExpectedType(returnType);
        return new Stage3MethodDeclaratorRestVisitor(stage3Env, info.getName().getValue(), returnType);
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestNode node) throws CompilerException {
        throw new Error("unsupported");
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
    }

    @Override
    public void end(MethodeDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public void end(ConstructorDeclaratorRestNode node) throws CompilerException {
    }

}
