/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ClassDeclarationNode;
import abstractsyntaxtree.ConstructorDeclaratorRestHelperNode;
import abstractsyntaxtree.GenericMethodeOrConstructorDeclNode;
import abstractsyntaxtree.InterfaceDeclarationNode;
import abstractsyntaxtree.MethodeOrFieldDeclNode;
import abstractsyntaxtree.VoidMethodeDeclaratorRestHelperNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractMemberDeclVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.ConstructorState;
import stage3.states.VoidMethodState;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3MemberDeclVisitor extends AbstractMemberDeclVisitor {
    private final Stage3Environment stage3Env;
    private boolean isFieldDecl = false;
    private Stage3MethodOrFieldDeclVisitor methodOrFieldDeclVisitor;

    public Stage3MemberDeclVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }
    
    @Override
    public void begin(MethodeOrFieldDeclNode node) throws CompilerException {
    }

    @Override
    public void begin(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {

    }

    @Override
    public void begin(ConstructorDeclaratorRestHelperNode node) throws CompilerException {

    }

    @Override
    public void begin(GenericMethodeOrConstructorDeclNode node) throws CompilerException {

    }

    @Override
    public void begin(ClassDeclarationNode node) throws CompilerException {

    }

    @Override
    public void begin(InterfaceDeclarationNode node) throws CompilerException {

    }

    @Override
    public Visitor visit(MethodeOrFieldDeclNode node) throws CompilerException {
        return methodOrFieldDeclVisitor = new Stage3MethodOrFieldDeclVisitor(stage3Env,node.getIdentifier());
    }

    @Override
    public Visitor visit(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {
        stage3Env.initState(new VoidMethodState());
        stage3Env.setCurrentMethodExpectedType(new Type.SimpleType("void"));
        return new Stage3MethodDeclaratorRestVisitor(stage3Env,node.getMethodName(), new Type.SimpleType("void"));
    }

    @Override
    public Visitor visit(ConstructorDeclaratorRestHelperNode node) throws CompilerException {
        ConstructorState state = new ConstructorState();
        stage3Env.initState(state);
        stage3Env.setCurrentMethodExpectedType(new Type.SimpleType("void"));
        return new Stage3MethodDeclaratorRestVisitor(stage3Env,"<init>",new Type.SimpleType("void"));
    }

    @Override
    public Visitor visit(GenericMethodeOrConstructorDeclNode node) throws CompilerException {
        return new Stage3GenericMethodeOrConstructorDeclNode(stage3Env);
    }

    @Override
    public Visitor visit(ClassDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner class declaration not allowed",node.getSymbol());
    }

    @Override
    public Visitor visit(InterfaceDeclarationNode node) throws CompilerException {
        throw new CompilerException("Inner interface declaration not allowed",node.getSymbol());
    }

    @Override
    public void end(MethodeOrFieldDeclNode node) throws CompilerException {
        this.isFieldDecl = methodOrFieldDeclVisitor.isFielDeclaration();
    }

    @Override
    public void end(VoidMethodeDeclaratorRestHelperNode node) throws CompilerException {

    }

    @Override
    public void end(ConstructorDeclaratorRestHelperNode node) throws CompilerException {

    }

    @Override
    public void end(GenericMethodeOrConstructorDeclNode node) throws CompilerException {

    }

    @Override
    public void end(ClassDeclarationNode node) throws CompilerException {

    }

    @Override
    public void end(InterfaceDeclarationNode node) throws CompilerException {

    }

    public boolean isFieldDeclaration() {
        return this.isFieldDecl;
    }
    
}
