/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArgumentsNode;
import abstractsyntaxtree.ExplicitGenericInvocationNode;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InnerCreatorNode;
import abstractsyntaxtree.NonWildcardTypeArgumentsNode;
import abstractsyntaxtree.SuperSuffixNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractSelectorVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.MethodInvokeCreator;
import exceptions.CompilerException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;

/**
 *
 * @author Michael Burzan
 */
class Stage3SelectorVisitor extends AbstractSelectorVisitor {

    private final Stage3Environment stage3Env;
    private final String identifier;
    private final boolean isClosureCall;
    private boolean hasArguments = false;

    public boolean hasArguments() {
        return hasArguments;
    }

    Stage3SelectorVisitor(Stage3Environment stage3Env, String identifier, boolean closureCall) {
        this.stage3Env = stage3Env;
        this.identifier = identifier;
        this.isClosureCall = closureCall;
    }

    @Override
    public void begin(ArgumentsNode node) throws CompilerException {
    }

    @Override
    public void begin(ExplicitGenericInvocationNode node) throws CompilerException {
        throw new CompilerException(" explicit generic method invocation after method invocation not supported yet",node.getSymbol());
    }

    @Override
    public void begin(SuperSuffixNode node) throws CompilerException {
         throw new CompilerException(" super acces after method invocation not allowed in TeaJay",node.getSymbol());
    }

    @Override
    public void begin(NonWildcardTypeArgumentsNode node) throws CompilerException {
        
    }

    @Override
    public void begin(InnerCreatorNode node) throws CompilerException {
         throw new CompilerException(" InnerCreation after method invocation not allowed in TeaJay",node.getSymbol());
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        throw new CompilerException(" [ Expression ] after method invocation not allowed in TeaJay",node.getSymbol());
    }

    @Override
    public Visitor visit(ArgumentsNode node) throws CompilerException {
        return new Stage3ArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ExplicitGenericInvocationNode node) throws CompilerException {
        return new Stage3ExplicitGenericInvocationVisitor(stage3Env);
    }

    @Override
    public Visitor visit(SuperSuffixNode node) throws CompilerException {
        return new Stage3SuperSuffixVisitor(stage3Env, identifier);
    }

    @Override
    public Visitor visit(NonWildcardTypeArgumentsNode node) throws CompilerException {
        return new Stage3NonWildcardTypeArgumentsVisitor(stage3Env);
    }

    @Override
    public Visitor visit(InnerCreatorNode node) throws CompilerException {
        return new Stage3InnerCreatorVisitor(stage3Env);
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return null;
        //return new Stage3ExpressionVisitor(stage3Env);
    }

    @Override
    public void end(ArgumentsNode node) throws CompilerException {
        // call exp.a() or closure call
        this.hasArguments = true;
        ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
        Type typeOfResult = env.typeOfResult();
        
        try {
            //check is closure call
            if (isClosureCall) {
                try {
                    Type.ClassType cType = null;
                    if(typeOfResult instanceof Type.ClassType){
                       cType  = (Type.ClassType) typeOfResult;
                       if(!cType.name.equals("teajay/lang/Closure")){
                           cType = null;
                       }
                    }
                    if(cType == null){
                        throw new CompilerException("For closure call must be type of closure and not type of "+typeOfResult, node.getSymbol());
                    }
                    //call the method call
                    MethodInvokeCreator.CallContext context = new MethodInvokeCreator.CallContext(stage3Env, cType, "execute",1);
                    MethodInvokeCreator.createMethodCall(context, stage3Env.getExpressionEnvironment(), "execute", Collections.EMPTY_LIST, node);
                } catch (ByteCodeException | IOException | ResolveException ex) {
                    ex.printStackTrace(System.err);
                    System.exit(-1);
                }
            } else {
                MethodInvokeCreator.CallContext context = new MethodInvokeCreator.CallContext(stage3Env, typeOfResult, identifier,1);
                MethodInvokeCreator.createMethodCall(context, env, identifier, Arrays.asList("..."), node); //anonymous selector (must be ...)
            }
        } catch (ByteCodeException | IOException | ResolveException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    @Override
    public void end(ExplicitGenericInvocationNode node) throws CompilerException {
    }

    @Override
    public void end(SuperSuffixNode node) throws CompilerException {
    }

    @Override
    public void end(NonWildcardTypeArgumentsNode node) throws CompilerException {
    }

    @Override
    public void end(InnerCreatorNode node) throws CompilerException {
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
    }
}
