/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractStatementExpressionVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.ShortCircuitEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.Deque;
import java.util.LinkedList;
import stage3.states.BlockState;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.NOP;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage3StatementExpressionVistor extends AbstractStatementExpressionVisitor {

    private final Stage3Environment stage3Env;
    private Deque<InfixOpNode.InfixOps> stack;

    public Stage3StatementExpressionVistor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.stage3Env.setExpressionEnvironment(new ExpressionEnvironment(stage3Env));
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        stack = new LinkedList<>();
        return new Stage3ExpressionVisitor(this.stage3Env, stack, false);
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
        InstructionCreator.PutOperatorInstructions(env, stack, node);
        ShortCircuitEnvironment shortCircuit = env.getShortCircuit();
        if (shortCircuit != null) {
            BlockState hasState = stage3Env.hasState(BlockState.class);
            Label label = new Label("ParExpression short circuit gotoLabel");
            env.popType(); 
            
            shortCircuit.write(stage3Env, label, env.getOperandStack().toArray(new Type[0]));
            hasState.addLabel(label);

            env.setResultType(new Type.ClassType(null, "java/lang/Boolean", null));
            stage3Env.getExpressionEnvironment().insertCodeWithVerificationStack(new NOP(), env.getOperandStack());
            env.setShortCircuit(null);
        }
        /*
         * expression without assignment;
         * Pop type from jvm stack
         */
        if (env.existItemOnStack()) {
            env.writePop();
        }
        stage3Env.setExpectedResultType(null);
    }
}