/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.AssertStatementNode;
import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.BreakStatementNode;
import abstractsyntaxtree.ContinueStatementNode;
import abstractsyntaxtree.DoStatementNode;
import abstractsyntaxtree.ForStatementNode;
import abstractsyntaxtree.IdentDoublePointStatementNode;
import abstractsyntaxtree.IfStatementNode;
import abstractsyntaxtree.ReturnStatementNode;
import abstractsyntaxtree.StatementExpressionNode;
import abstractsyntaxtree.SwitchStatementNode;
import abstractsyntaxtree.SynchronizedStatementNode;
import abstractsyntaxtree.ThrowStatementNode;
import abstractsyntaxtree.TryStatementNode;
import abstractsyntaxtree.TypeSwitchStatementNode;
import abstractsyntaxtree.WhileStatementNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractStatementVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.bridgeimplementation.BaseStatementVisitorImpl;
import stage3.bridgeimplementation.StatementVisitorImpl;

/**
 *
 * @author Michael Burzan
 * delegate all calls to BaseStatementVisitorImpl
 */
public class Stage3StatementVisitor extends AbstractStatementVisitor {

    private final Stage3Environment stage3Env;
    private Stage3IfStatementVisitor ifStatementVistor;
    private Stage3WhileStatementVisitor whileStatementVisitor;
    private Stage3DoStatementVisitor doStatement;
    private Stage3ForStatementVisitor forStatement;
    private Stage3AssertStatementVisitor assertStatementVisitor;
    private Stage3TryStatementVisitor tryVisitor;
    private Stage3TypeSwitchStatementVisitor typeSwitchStatement;
    private Stage3SynchronizedStatementVisitor synchronizedVisitor;
    private StatementVisitorImpl impl;

    public Stage3StatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        this.impl = new BaseStatementVisitorImpl(stage3Env);
        if(stage3Env.getExpressionEnvironment() != null){
            this.stage3Env.getExpressionEnvironment().cleanOperandStack();
        }
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(IdentDoublePointStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(StatementExpressionNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(IfStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(AssertStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(SwitchStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(WhileStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(DoStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(ForStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(BreakStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(ContinueStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(ReturnStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(ThrowStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(SynchronizedStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(TryStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public void begin(TypeSwitchStatementNode node) throws CompilerException {
        impl.beginImpl(node);
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(IdentDoublePointStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(StatementExpressionNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(IfStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(AssertStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(SwitchStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(WhileStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(DoStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(ForStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public void visit(BreakStatementNode node) throws CompilerException {
        impl.visitImpl(node);
    }

    @Override
    public void visit(ContinueStatementNode node) throws CompilerException {
        impl.visitImpl(node);
    }

    @Override
    public Visitor visit(ReturnStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(ThrowStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(SynchronizedStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(TryStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public Visitor visit(TypeSwitchStatementNode node) throws CompilerException {
        return impl.visitImpl(node);
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(IdentDoublePointStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(StatementExpressionNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(IfStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(AssertStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(SwitchStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(WhileStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(DoStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(ForStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(BreakStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(ContinueStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(ReturnStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(ThrowStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(SynchronizedStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(TryStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }

    @Override
    public void end(TypeSwitchStatementNode node) throws CompilerException {
        impl.endImpl(node);
    }
}
