/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArgumentsNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractSuperSuffixVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import stage3.states.ConstructorState;
import stage3.states.MethodState;
import stage3.utils.InformationCenter;
import teajay.bytecode.ClassFile;
import teajay.bytecode.attributes.code.AAStore;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.ANewArray;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.GetField;
import teajay.bytecode.attributes.code.InvokeSpecial;
import teajay.bytecode.attributes.code.InvokeVirtual;
import teajay.bytecode.attributes.code.JVMInstructionList;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.attributes.code.Swap;
import teajay.bytecode.attributes.signature.Signature;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.cpool.MethodRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.bytecode.fields.FieldAccessFlag;
import teajay.bytecode.fields.FieldInfo;
import teajay.bytecode.methods.MethodInfo;
import teajay.generics.check.TypeException;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;
import teajay.generics.writer.SignatureEncoder;
import teajay.methods.MethodFinder;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Closure;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Hidden;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Impl;
import static teajay.util.tjattribute.TeaJayAttribute.Type.TJBuiltinType;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Typedef;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
class Stage3SuperSuffixVisitor extends AbstractSuperSuffixVisitor {

    private final Stage3Environment stage3Env;
    private ClassFile superType;
    private final String identifier;

    public Stage3SuperSuffixVisitor(Stage3Environment stage3Env, String identifier) {
        this.stage3Env = stage3Env;
        this.identifier = identifier;
    }

    @Override
    public void begin(ArgumentsNode node) throws CompilerException {
        try {
            //super Arguments
            if (identifier == null) {
                ConstructorState state = stage3Env.hasState(ConstructorState.class);
                if (state == null) {
                    throw new CompilerException("super(...) call outside constructor", node.getSymbol());
                }
                if (state.hasSuper()) {
                    throw new CompilerException("super(...) calls must be the first statement in a constructor", node.getSymbol());
                }
                stage3Env.setIsInSuperConstructorCall();
                stage3Env.getExpressionEnvironment().insertCode(new ALoad(0)); //THIS
                stage3Env.getExpressionEnvironment().insertCode(new ALoad(0)); // First parameter of constructor
                FieldRefInfo fieldRefInfo = stage3Env.getClassFile().getField("this").getFieldRefInfo(stage3Env.getClassFile().getThis());
                stage3Env.getExpressionEnvironment().insertCode(new GetField(fieldRefInfo));
                this.superType = stage3Env.getSuper(node);
            } else {
                MethodState mstate = stage3Env.hasState(MethodState.class);
                if (mstate != null && mstate.isClosure()) {
                    throw new CompilerException("no calls to super-methods allowed within closures", node.getSymbol());
                }
                // super.identifier Arguments
                ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
                env.insertCode(new ALoad(0)); //THIS
                FieldInfo field = this.stage3Env.getClassFile().getField("super");
                if (field == null) {
                    throw new Error("missing super field");
                }
                env.insertCode(new GetField(field.getFieldRefInfo(this.stage3Env.getClassFile().getThis())));
                env.setResultType(stage3Env.getTypeUtils().getFieldType(field));
            }
            //stage3Env.getExpressionEnvironment().pushCallParameter(new Type.ClassType(null,this.superType.getThis().getClassName(), null));

        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }


    }

    @Override
    public Visitor visit(ArgumentsNode node) throws CompilerException {
        return new Stage3ArgumentsVisitor(stage3Env);
    }

    @Override
    public void end(ArgumentsNode node) throws CompilerException {
        try {
            if (identifier == null) {
                JVMInstructionList instructionList = this.stage3Env.getMethodInfo().getCode().getInstructionList();
                FieldDescriptor[] cParameters = stage3Env.getExpressionEnvironment().getCallParameters();
                List<FieldDescriptor> callParameters = new ArrayList<>(Arrays.asList(cParameters));
                //callParameters.add(0, FieldDescriptor.fromClassFile(superType));
                Pair<MethodInfo, MethodFinder.DistanceTailPair> findMethod = MethodFinder.findMethod("<init>", callParameters, superType.getThis().getClassName(),
                        stage3Env.getClassFile().getThis().getClassName(), node, MethodFinder.QueryType.NON_VIRTUAL_NON_STATIC);
                if (findMethod == null) {
                    throw new CompilerException("no such super constructor: <init>" + callParameters, node.getSymbol());
                }
                Type.MethodType methodSignatureByEnv = stage3Env.getTypeUtils().getMethodSignatureByEnv(findMethod.value1, stage3Env.getThis(), null);
                if (findMethod.value1.isVarArgs()) {
                    instructionList.pushInt(findMethod.value2.tailLength);
                    List<FieldDescriptor> params = findMethod.value1.getDescriptor().getParameters();
                    FieldDescriptor varArgArray = params.get(params.size() - 1);
                    instructionList.add(new ANewArray(varArgArray.getElementType().getClassInfo()));
                    int arrayIndex = stage3Env.getLocalTable().getNextId();
                    instructionList.add(new AStore(arrayIndex));
                    for (int i = findMethod.value2.tailLength - 1; i >= 0; i--) {
                        instructionList.add(new ALoad(arrayIndex));
                        instructionList.add(new Swap());
                        instructionList.pushInt(i);
                        instructionList.add(new Swap());
                        instructionList.add(new AAStore());
                    }
                    instructionList.add(new ALoad(arrayIndex));

                    List<Type> paramTypes = new ArrayList<>();
                    paramTypes.addAll(methodSignatureByEnv.paramTypes.subList(0, methodSignatureByEnv.paramTypes.size() - 1));

                    Type.ArrayType last = (Type.ArrayType) methodSignatureByEnv.paramTypes.get(methodSignatureByEnv.paramTypes.size() - 1);
                    for (int i = 0; i < findMethod.value2.tailLength; i++) {
                        paramTypes.add(last.elemType);
                    }
                    methodSignatureByEnv = new Type.MethodType(methodSignatureByEnv.typeParamTypes, paramTypes, methodSignatureByEnv.returnType, methodSignatureByEnv.throwsTypes);
                    List<FieldDescriptor> subList = callParameters.subList(0, callParameters.size() - findMethod.value2.tailLength);
                    subList.add(stage3Env.getTypeUtils().makeFieldDescriptor(last));
                    callParameters = subList;
                }
                callParameters.add(0, FieldDescriptor.fromClassFile(superType));

                try {
                    stage3Env.getTypeUtils().checkCallability(methodSignatureByEnv, stage3Env.getExpressionEnvironment().getCallTypeParameters());
                } catch (TypeException ex) {
                    throw new CompilerException("Parameters missmatch (" + findMethod.value1 + "): " + ex.getMessage(), node.getSymbol());
                }
                if (stage3Env.getClassFile().getField("super") == null) {
                    Type.ClassSigType classSignature = TypeUtils.getClassSignature(stage3Env.getClassFile().getSuper().getClassName());
                    FieldInfo superField = new FieldInfo("super", FieldDescriptor.fromClassFile(superType), FieldAccessFlag.PRIVATE, FieldAccessFlag.SYNTHETIC);
                    superField.addAttribute(new Signature(SignatureEncoder.encode(classSignature.superclassType)));
                    stage3Env.getClassFile().addField(superField);
                }

                switch (InformationCenter.getTeaJayAttributeType(superType)) {
                    case Class:
                    case TJBuiltinType: {

                        List<FieldDescriptor> parameters = new ArrayList<>();
                        parameters.addAll(findMethod.value1.getDescriptor().getParameters());

                        MethodRefInfo ref = MethodRefInfo.make(this.superType.getThis(), "<init>", FieldDescriptor.getVoid(), parameters.toArray(new FieldDescriptor[parameters.size()]));
                        instructionList.add(new InvokeSpecial(ref));
                        instructionList.add(new ALoad(0));
                        FieldRefInfo thisRef = stage3Env.getClassFile().getField("this").getFieldRefInfo(stage3Env.getClassFile().getThis());
                        instructionList.add(new GetField(thisRef));
                        instructionList.add(new PutField(
                                FieldRefInfo.make(this.stage3Env.getClassFile().getThis(),
                                "super", FieldDescriptor.fromClassFile(superType))));
                    }
                    break;

                    case Typedef: {
                        List<FieldDescriptor> parameters = new ArrayList<>();
                        parameters.add(FieldDescriptor.fromClassFile(superType));
                        parameters.addAll(findMethod.value1.getDescriptor().getParameters());
                        //System.err.println(callParameters.toArray(new FieldDescriptor[0]));
                        stage3Env.invokeDynamic(this.stage3Env.getMethodInfo(), this.superType.getThis(), parameters.toArray(new FieldDescriptor[parameters.size()]));
                        instructionList.add(new PutField(
                                FieldRefInfo.make(this.stage3Env.getClassFile().getThis(),
                                "super", FieldDescriptor.fromClassFile(superType))));
                    }
                    break;
                    case Closure:
                    case Hidden:
                    case Enum:
                    case Interface:
                    case Impl:
                        throw new Error("Fatal error: super call is impossible");
                    default:
                        throw new AssertionError("Missing case");

                }

                stage3Env.writeFields(node);
                ConstructorState cState = stage3Env.hasState(ConstructorState.class);
                cState.setHasSuper();
                stage3Env.resetIsInSuperConstructorCall();
            } else {
                ExpressionEnvironment env = this.stage3Env.getExpressionEnvironment();
                Type typeOfResult = env.typeOfResult();
                MethodInfo info;
                FieldDescriptor superDesc = stage3Env.getTypeUtils().makeFieldDescriptor(typeOfResult);
                Pair<MethodInfo, MethodFinder.DistanceTailPair> result = MethodFinder.findMethod(identifier, Arrays.asList(env.getCallParameters()),
                        superDesc.getClassInfo().getClassName(), stage3Env.getClassFile().getThis().getClassName(),
                        node, MethodFinder.QueryType.NON_STATIC);
                if (result == null) {
                    throw new CompilerException("Method " + identifier + " not found in " + typeOfResult, node.getSymbol());
                }
                info = result.value1;
                env.insertCode(new InvokeVirtual(info.makeMethodRef()));
                Type.MethodType methodSignatureByEnv =
                        stage3Env.getTypeUtils().getMethodSignatureByEnv(info, (Type.ClassType) typeOfResult, Collections.EMPTY_LIST);
                env.setResultType(methodSignatureByEnv.returnType);
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        } catch (TypeException ex) {
            throw new CompilerException("Incompatible signature " + ex.getMessage(), node.getSymbol());
        }

    }
}
