/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractThrowStatementVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.Deque;
import java.util.LinkedList;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class Stage3ThrowStatementVisitor extends AbstractThrowStatementVisitor {

    private final Stage3Environment stage3Env;
    private ExpressionEnvironment argEnv;
    private Deque<InfixOpNode.InfixOps> stack;
    private static final Type.ClassType THROWABLE = new Type.ClassType(null, "java/lang/Throwable", null);

    public Stage3ThrowStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        stack = new LinkedList<>();
        argEnv = new ExpressionEnvironment(stage3Env);
        this.stage3Env.setExpressionEnvironment(argEnv);
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        return new Stage3ExpressionVisitor(stage3Env, this.stack, true);
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        //write operators of expression to instruction list
        InstructionCreator.PutOperatorInstructions(argEnv, stack, node);
        //check is result type assignable to Throwable
        Type typeOfResult = argEnv.typeOfResult();
        if (typeOfResult instanceof Type.ClassType || typeOfResult instanceof Type.SimpleType) {
            try {
                stage3Env.getTypeUtils().checkAssignability(THROWABLE, typeOfResult);
                //Throw call in StatementVisitor
            } catch (TypeException ex) {
                throw new CompilerException("throw expects Throwable", node.getSymbol());
            }
        } else {
            throw new CompilerException("throw expects Throwable", node.getSymbol());
        }
    }
}
