/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BlockNode;
import abstractsyntaxtree.CatchesNode;
import abstractsyntaxtree.FinallyNode;
import abstractsyntaxtree.ResourceSpecificationNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTryStatementVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import stage3.states.BlockState;
import stage3.states.EndState;
import stage3.states.TryState;
import stage3.states.UnreachableCodeState;
import teajay.bytecode.attributes.code.JVMInstruction;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.attributes.code.PlaceholderInstruction;
import teajay.bytecode.exceptions.ByteCodeException;

/**
 *
 * @author Michael Burzan
 */
public class Stage3TryStatementVisitor extends AbstractTryStatementVisitor {

    private final Stage3Environment stage3Env;
    private final Label labelStartPC;
    private Stage3BlockVisitor blockVisitor;
    private JVMInstruction endPC;
    private Label jmpOverCatches;
    private int unreachableCode = 0;
    private int cntCatchClause = 0;
    private final static int TRYBLOCKCNT = 1;
    private Stage3CatchesVisitor catchesVisitor;
    private PlaceholderInstruction placeholderPc;
    private boolean isUnreachableCode;

    public Stage3TryStatementVisitor(Stage3Environment stage3Env) {
        this.stage3Env = stage3Env;
        //create start label for exception table
        this.labelStartPC = new Label("StartPC");
    }

    @Override
    public void begin(BlockNode node) throws CompilerException {
        BlockState oldBlock = stage3Env.hasState(BlockState.class);
        if (oldBlock == null) {
            throw new Error("Fatal error: Surrounding block missing befor try state");
        }
        BlockState state = new BlockState();
        //bind start position to first instruction in try block
        state.addLabel(labelStartPC);
        //copy old labels from old block
        state.addLabels(oldBlock.getLabels());
        oldBlock.resetLabelList();
        stage3Env.changeState(state, node);
        stage3Env.getLocalTable().addFrame();
    }

    @Override
    public void begin(CatchesNode node) throws CompilerException {
        cntCatchClause++;
    }

    @Override
    public void begin(FinallyNode node) throws CompilerException {
    }

    @Override
    public void begin(ResourceSpecificationNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(BlockNode node) throws CompilerException {
        return blockVisitor = new Stage3BlockVisitor(this.stage3Env);
    }

    @Override
    public Visitor visit(CatchesNode node) throws CompilerException {
        stage3Env.getLocalTable().removeFrame();
        return catchesVisitor = new Stage3CatchesVisitor(stage3Env, jmpOverCatches, isUnreachableCode ? endPC : null);
    }

    @Override
    public Visitor visit(FinallyNode node) throws CompilerException {
        throw new CompilerException("finally not supported in TeaJay",node.getSymbol());
        //return null;
    }

    @Override
    public Visitor visit(ResourceSpecificationNode node) throws CompilerException {
        throw new CompilerException("Resource specification in try not supported in TeaJay",node.getSymbol());
        //return null;
    }

    @Override
    public void end(BlockNode node) throws CompilerException {
        EndState endState = stage3Env.hasState(EndState.class);
        UnreachableCodeState unreachableCodeState = stage3Env.hasState(UnreachableCodeState.class);

        stage3Env.prevState();
        if (blockVisitor.isEmptyBlock()) {
            throw new CompilerException("Empty try block", node.getSymbol());
        }
        try {
            TryState tryState = stage3Env.hasState(TryState.class);
            //save first position in try state
            tryState.setStartLabel(labelStartPC);
            // get last instruction from instruction list
            this.endPC = this.stage3Env.getMethodInfo().getCode().getInstructionList().getLast();
            /*
             * case one:
             * try{
             *  ...
             *  goto jumpOverCatches;
             * }catch(Type t){
             * }
             * jmpOverCatches: ...
             * 
             */ 
            if (endState == null && unreachableCodeState == null) {
                // save end position without goto
                this.jmpOverCatches = new Label();
                stage3Env.getExpressionEnvironment().writeGoto(this.jmpOverCatches);
                this.isUnreachableCode = false;
                tryState.setEndLabel(this.endPC);
                this.endPC = null;
            } else {
                //otherwise end position is calculatet in Stage3CatchesVisitor
                unreachableCode++;
                stage3Env.prevState();
                this.isUnreachableCode = true;
            }
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }


    }

    @Override
    public void end(CatchesNode node) throws CompilerException {
        if (catchesVisitor.hasEndStateReached()) {
            this.unreachableCode++;
        }



    }

    @Override
    public void end(FinallyNode node) throws CompilerException {
    }

    @Override
    public void end(ResourceSpecificationNode node) throws CompilerException {
    }

    public boolean hasUnreachableCodeReached() {
        //check have try and catch all unreachable state
        if (unreachableCode == (cntCatchClause + TRYBLOCKCNT)) {
            return true;
        }
        return false;
    }

    public Label getLabel() {
        return jmpOverCatches;
    }
}
