/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.TypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeSwitchLabelVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.io.IOException;
import stage3.utils.InformationCenter;
import symbol.DerivationTree;
import teajay.bytecode.ClassFile;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.parser.Type;
import teajay.runtime.exceptions.ResolveException;
import teajay.util.io.ClassFileRef;
import teajay.util.io.ClassFinder;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Class;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Closure;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Enum;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Hidden;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Impl;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Interface;
import static teajay.util.tjattribute.TeaJayAttribute.Type.TJBuiltinType;
import static teajay.util.tjattribute.TeaJayAttribute.Type.Typedef;

/**
 *
 * @author Michael Burzan
 */
class Stage3TypeSwitchLabelVisitor extends AbstractTypeSwitchLabelVisitor {

    private final Stage3Environment stage3Env;
    private Stage3TypeVisitor typeVisitor;
    private Type type;
    private final String origType;

    public Stage3TypeSwitchLabelVisitor(Stage3Environment stage3Env, String origType) {
        this.stage3Env = stage3Env;
        this.origType = origType;
    }

    @Override
    public void begin(TypeNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(TypeNode node) throws CompilerException {
        return typeVisitor = new Stage3TypeVisitor(stage3Env);
    }

    @Override
    public void end(TypeNode node) throws CompilerException {
        try {
            type = typeVisitor.getType();
            String className = stage3Env.getTypeUtils().makeFieldDescriptor(type).getClassInfo().getClassName();

            ClassFile file = ClassFinder.instance().find(className).get();
            switch (InformationCenter.getTeaJayAttributeType(file)) {
                case Closure:
                    throw new CompilerException("no closures allowed as case label (in typeswitch)", node.getSymbol());
                case Impl:
                    throw new CompilerException("no implementations allowed as case label (in typeswitch)", node.getSymbol());
                case Enum:
                case Typedef:
                case Class:
                case Interface:
                case TJBuiltinType:
                case Hidden:
                    break;
                default:
                    throw new AssertionError(InformationCenter.getTeaJayAttributeType(file).name());
            }

            if (type instanceof Type.ClassType) {
                Type.ClassType classType = (Type.ClassType) type;
                ClassFileRef orig = ClassFinder.instance().find(origType);
                ClassFileRef caseType = ClassFinder.instance().find(classType.name);
                if (DerivationTree.instance().isDerivated(caseType, orig)) {
                    throw new CompilerException("superfluous case " + classType, node.getSymbol());
                }
                if (classType.typeArgs != null && !classType.typeArgs.isEmpty()) {
                    for (Type t : classType.typeArgs) {
                        if (t instanceof Type.WildcardType) {
                            Type.WildcardType wildcardType = (Type.WildcardType) t;
                            if (wildcardType.kind == Type.WildcardType.Kind.UNBOUNDED) {
                                break; //OK
                            }
                        }
                        throw new CompilerException("case labels can only accept unbounded generic arguments", node.getSymbol());
                    }
                }
            } else {
                throw new CompilerException("case labels cannot accept primitives or type variables", node.getSymbol());
            }
        } catch (ByteCodeException | ResolveException | IOException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
    }

    public Type.ClassType getType() throws CompilerException {
        return (Type.ClassType) type;
    }
}
