/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.BasicTypeNode;
import abstractsyntaxtree.ClosureTypeDecNode;
import abstractsyntaxtree.EdgeClampsOpenCloseHelperNode;
import abstractsyntaxtree.ReferenceTypeNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractTypeVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.List;
import stage3.utils.TypeChecker;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3TypeVisitor extends AbstractTypeVisitor {

    private final Stage3Environment stage3Env;
    private List<String> identifierList = new ArrayList<>();
    private Stage3ReferenceTypeVisitor stage3ReferenceType;
    private Type type;
    private Stage3ClosureTypeDecVisitor closureTypeDec;
    private final boolean allowBasicTypes;
    private int edgeClampCount = 0;
    
    public Stage3TypeVisitor(Stage3Environment stage3Env) {
        this(stage3Env, false);
    }

    public Stage3TypeVisitor(Stage3Environment stage3Env, boolean allowBasicTypes) {
        this.stage3Env = stage3Env;
        this.allowBasicTypes = allowBasicTypes;
    }

    @Override
    public void begin(BasicTypeNode node) throws CompilerException {
        if (allowBasicTypes) {
            this.type = new Type.SimpleType(node.getType().toString());
        } else {
            throw new CompilerException("Java basic type not allowed in teajay", node.getSymbol());
        }
    }

    @Override
    public void visit(BasicTypeNode node) throws CompilerException {
    }

    @Override
    public void end(BasicTypeNode node) throws CompilerException {
    }

    @Override
    public void begin(ReferenceTypeNode node) throws CompilerException {
        identifierList.add(node.getName());
    }

    @Override
    public Visitor visit(ReferenceTypeNode node) throws CompilerException {

        return stage3ReferenceType = new Stage3ReferenceTypeVisitor(stage3Env, identifierList);
    }

    @Override
    public void end(ReferenceTypeNode node) throws CompilerException {
        StringBuilder identifier = new StringBuilder();
        //Make Identifier . Identifier to Identifier/Identifier
        for (int i = 0; i < identifierList.size(); i++) {
            identifier.append(identifierList.get(i));
            if (i != (identifierList.size() - 1)) {
                identifier.append("/");
            }
        }
        //check exist qID in NameResolver
        String ident = identifier.toString();
        String qID = stage3Env.getSymbolFacade().getFullQualifiedId(ident);
        if (qID == null) {
            qID = ident;
        }
        if (stage3Env.isTypeVariable(ident)) {
            this.type = new Type.SimpleType(ident);
        } else {
            this.type = new Type.ClassType(null, qID, stage3ReferenceType.getTypeParameters());
            TypeChecker.check(type, node, stage3Env);
        }
    }

    @Override
    public void begin(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
        edgeClampCount++;
    }

    @Override
    public void visit(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
    }

    @Override
    public void end(EdgeClampsOpenCloseHelperNode node) throws CompilerException {
    }

    @Override
    public void begin(ClosureTypeDecNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(ClosureTypeDecNode node) throws CompilerException {
        return closureTypeDec = new Stage3ClosureTypeDecVisitor(stage3Env);
    }

    @Override
    public void end(ClosureTypeDecNode node) throws CompilerException {
        this.type = closureTypeDec.getClassType();
    }

    @Override
    public String getTypeName() throws CompilerException {
        return null;
    }

    public Type getType() throws CompilerException {
        return this.type;
    }
    
    public int getDim() {
        return this.edgeClampCount;
    }
}
