/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.VariableDeclaratorRestNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractVariableDeclaratorVisitor;
import codegeneration.environment.Stage3Environment;
import exceptions.CompilerException;
import symbol.LocalEntry;
import teajay.bytecode.attributes.code.AConstNull;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3VariableDeclaratorVisitor extends AbstractVariableDeclaratorVisitor {

    private final Stage3Environment stage3Env;
    private final Type type;
    private final String name;
    private Stage3VariableDeclaratorRestVisitor variableDeclaratorRestVisitor;

    public Stage3VariableDeclaratorVisitor(Stage3Environment stage3Env, Type type, String name) {
        this.stage3Env = stage3Env;
        this.type = type;
        this.name = name;
    }

    @Override
    public void begin(VariableDeclaratorRestNode node) throws CompilerException {
    }

    @Override
    public Visitor visit(VariableDeclaratorRestNode node) throws CompilerException {
        return variableDeclaratorRestVisitor = new Stage3VariableDeclaratorRestVisitor(stage3Env, name, type);
    }

    @Override
    public void end(VariableDeclaratorRestNode node) throws CompilerException {
        TypeUtils.checkTypes(type, stage3Env.getThis(), node.getSymbol());
        
        if (node.getDim() > 0) {
            throw new CompilerException("Cannot declare Java-Arrays", node.getSymbol());
        }
        //if local variable is not init then init with null 
        if (!variableDeclaratorRestVisitor.hasInit()) {
            if (stage3Env.getLocalTable().getEntry(name) != null) {
                throw new CompilerException("Variable " + name + " already defined", node.getSymbol());
            }
            try {
                //push null reference on bytecode stack
                stage3Env.getExpressionEnvironment().insertCode(new AConstNull());
                //add entry to local table
                stage3Env.getLocalTable().addEntry(name, type);
                LocalEntry entry = stage3Env.getLocalTable().getEntry(name);
                // store in the next free register
                stage3Env.getExpressionEnvironment().insertCode(new AStore(entry.getIndex()));
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            }

        }

    }
}
