/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3;

import abstractsyntaxtree.ArrayInitializerNode;
import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.Visitor;
import codegeneration.abstractvisitors.AbstractVariableInitializerVisitor;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import exceptions.CompilerException;
import java.util.Deque;
import java.util.LinkedList;
import stage3.states.BlockState;
import stage3.states.LocalVariableDeclarationState;
import stage3.utils.FieldInitBuilder;
import symbol.LocalEntry;
import symbol.LocalVariableState;
import teajay.bytecode.attributes.code.AStore;
import teajay.bytecode.attributes.code.Label;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
class Stage3VariableInitializerVisitor extends AbstractVariableInitializerVisitor {

    private final Stage3Environment stage3Env;
    private final String varName;
    private final FieldInitBuilder fieldBuilder;
    private final Type type;
    Deque<InfixOpNode.InfixOps> stack = new LinkedList<>();

    public Stage3VariableInitializerVisitor(Stage3Environment stage3Env, String fieldName,
            FieldInitBuilder fieldBuilder, Type type) {
        this.stage3Env = stage3Env;
        this.varName = fieldName;
        this.fieldBuilder = fieldBuilder;
        this.type = type;
        if(stage3Env.getExpressionEnvironment() != null){
            stage3Env.getExpressionEnvironment().cleanOperandStack();
        }

    }

    @Override
    public void begin(ArrayInitializerNode node) throws CompilerException {
    }

    @Override
    public void begin(ExpressionNode node) throws CompilerException {
        if (fieldBuilder != null) {
            this.fieldBuilder.addVariable(type, varName, node);
        } else {
            LocalVariableDeclarationState state = stage3Env.hasState(LocalVariableDeclarationState.class);
            if (state == null) {
                throw new Error("Fatal error: state is not possible");
            }
            state.setType(type);
        }
    }

    @Override
    public Visitor visit(ArrayInitializerNode node) throws CompilerException {
        throw new CompilerException("Array initializing not supported in teajay", node.getSymbol());
    }

    @Override
    public Visitor visit(ExpressionNode node) throws CompilerException {
        if (this.fieldBuilder == null) {
            ExpressionEnvironment env = new ExpressionEnvironment(stage3Env);

            stage3Env.setExpectedResultType(type);
            stage3Env.setExpressionEnvironment(env);
            //is a RValue Expression
            return new Stage3ExpressionVisitor(this.stage3Env, stack, true); //true
        } else {
            return null;
        }
    }

    @Override
    public void end(ArrayInitializerNode node) throws CompilerException {
    }

    @Override
    public void end(ExpressionNode node) throws CompilerException {
        if (this.fieldBuilder == null) {
            ExpressionEnvironment env = stage3Env.getExpressionEnvironment();
            try {
                if (stage3Env.getLocalTable().getEntry(varName) != null) {
                    throw new CompilerException("Variable " + varName + " already defined", node.getSymbol());
                }
                // write operators to instruction list
                InstructionCreator.PutOperatorInstructions(env, stack, node);
                // check is statement Boolean a = exp && exp...
                if (env.isShortCircuit()) {
                    
                    Label gotoLabel = new Label("gotoLabel");
                
                    env.writeShortCircuit(gotoLabel);
                    BlockState hasState = stage3Env.hasState(BlockState.class);
                    hasState.addLabel(gotoLabel);
                  
                    //Store variable to next free register the stack map frame can't be calculate from StackMapGenerator
                    env.insertCodeWithVerificationStack(new AStore(stage3Env.getLocalTable().getNextId()), env.getOperandStack());
                    
                    stage3Env.getTypeUtils().checkAssignability(type, env.typeOfResult());
                    stage3Env.getLocalTable().addEntry(varName, type);

                } else {
                    stage3Env.getLocalTable().addEntry(varName, type);

                    LocalEntry entry = stage3Env.getLocalTable().getEntry(varName);
                    
                    stage3Env.getTypeUtils().checkAssignability(type, env.typeOfResult());
                    //store variable to free regisiter
                    env.insertCode(new AStore(entry.getIndex()));
                }
            } catch (ByteCodeException ex) {
                ex.printStackTrace(System.err);
                System.exit(-1);
            } catch (TypeException ex) {
                throw new CompilerException("Expected: " + type + " Found:" + env.typeOfResult(), node.getSymbol());
            }

        }
    }
}
