/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package stage3.utils;

import abstractsyntaxtree.ExpressionNode;
import abstractsyntaxtree.InfixOpNode;
import codegeneration.environment.ExpressionEnvironment;
import codegeneration.environment.Stage3Environment;
import codegeneration.instruction.InstructionCreator;
import datastructs.Pair;
import exceptions.CompilerException;
import java.util.ArrayList;
import java.util.Deque;
import java.util.LinkedList;
import java.util.List;
import stage3.Stage3ExpressionVisitor;
import teajay.bytecode.attributes.code.ALoad;
import teajay.bytecode.attributes.code.PutField;
import teajay.bytecode.cpool.FieldRefInfo;
import teajay.bytecode.exceptions.ByteCodeException;
import teajay.generics.check.TypeException;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Burzan
 */
public class ConstructorFieldInitBuilder extends FieldInitBuilder {

    private List<Pair<Type, Pair<String, ExpressionNode>>> variables = new ArrayList<>();
    /**
     * stores the field and the expression node
     * @param type of the field
     * @param name of the field
     * @param node expression 
     */
    @Override
    public void addVariable(Type type, String name, ExpressionNode node) {
        variables.add(new Pair<>(type, new Pair<>(name, node)));
    }
    /**
     * writes stored fields in the constructor instruction list
     * @param stage3Env the current environment
     * @throws CompilerException 
     */
    @Override
    public void writeToInstructionList(Stage3Environment stage3Env) throws CompilerException {
        try {
            ExpressionEnvironment oldEnv = stage3Env.getExpressionEnvironment();
            //iterate over all stored fields
            for (Pair<Type, Pair<String, ExpressionNode>> p : variables) {
                //build field reference
                FieldRefInfo ref = FieldRefInfo.make(stage3Env.getClassFile().getThis(), p.value2.value1, stage3Env.getClassFile().getField(p.value2.value1).getDescriptor());
                stage3Env.setExpectedResultType(p.value1);
                //load this reference from register 0
                stage3Env.getMethodInfo().getCode().getInstructionList().add(new ALoad(0));
                ExpressionEnvironment expEnv = new ExpressionEnvironment(stage3Env);
                stage3Env.setExpressionEnvironment(expEnv);
                //build new infix operator stack
                Deque<InfixOpNode.InfixOps> operatorStack = new LinkedList<>();
                //visit field expression
                p.value2.value2.accept(new Stage3ExpressionVisitor(stage3Env, operatorStack,true));
                //write infix operator to instriction list
                InstructionCreator.PutOperatorInstructions(expEnv,operatorStack,p.value2.value2);
                try {
                    //check is field type assignable to expression type
                    stage3Env.getTypeUtils().checkAssignability(p.value1, expEnv.typeOfResult());
                } catch (TypeException ex) {
                    throw new CompilerException("Incompatible types " + p.value1 + " : " + expEnv.typeOfResult(), p.value2.value2.getSymbol());
                }
                stage3Env.getMethodInfo().getCode().getInstructionList().add(new PutField(ref));

            }
            //env.setExpectedResultType(null);
            stage3Env.setExpressionEnvironment(oldEnv);
        } catch (ByteCodeException ex) {
            ex.printStackTrace(System.err);
            System.exit(-1);
        }
       // variables = Collections.EMPTY_LIST;


    }
}
