/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package symbol;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.generics.check.TypeUtils;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael (Burzan | Kußmann)
 */
public class Frame implements Iterable<LocalEntry> {

    private Map<String, LocalEntry> table = new HashMap<>();
    private int currentIndex;
    private final int origStart;
    private final TypeUtils typeUtils;

    Frame(int lowIndex, TypeUtils typeUtils) {
        this.currentIndex = lowIndex;
        this.origStart = lowIndex;
        this.typeUtils = typeUtils;
    }

    boolean contains(String name) {
        return table.containsKey(name);
    }

    LocalEntry get(String name) {
        return table.get(name);
    }

    void addEntry(String name, Type type) {
        FieldDescriptor descr = typeUtils.makeFieldDescriptor(type);
        String typeName = descr.getClassInfo().getClassName();
        table.put(name, new LocalEntry(this.currentIndex++, typeName, type, typeUtils,LocalVariableState.Uninitialized));
    }

    int getMaxIndex() {
        return this.currentIndex;
    }

    int getMinIndex() {
        return this.currentIndex - this.table.size();
    }

    @Override
    public Iterator<LocalEntry> iterator() {
        Collection<LocalEntry> c = this.table.values();
        List<LocalEntry> l = new ArrayList<>(c.size());
        l.addAll(c);
        Collections.sort(l);
        return l.iterator();
    }

    @Override
    public String toString() {
        return toString("");
    }

    public String toString(String tab) {
        StringBuilder buf = new StringBuilder();
        buf.append("-- Frame ");
        buf.append(origStart);
        buf.append('\n');
        List<Map.Entry<String, LocalEntry>> lst = new ArrayList<>(table.size());
        lst.addAll(table.entrySet());
        Collections.sort(lst, new Comparator<Map.Entry<String, LocalEntry>>() {
            @Override
            public int compare(Map.Entry<String, LocalEntry> o1, Map.Entry<String, LocalEntry> o2) {
                return o1.getValue().compareTo(o2.getValue());
            }
        });

        for (Map.Entry<String, LocalEntry> entry : lst) {
            buf.append(tab).append("  ");
            buf.append(entry.getKey());
            buf.append(", ");
            buf.append(entry.getValue());
            buf.append('\n');
        }
        return buf.toString();
    }
}