/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class Context {

    private ContextFrame currentFrame;
    private final Context parent;

    public Context() {
        this.currentFrame = new ContextFrame();
        this.parent = null;
    }

    public Context(Context parent) {
        this.parent = parent;
        this.currentFrame = new ContextFrame();
    }

    public void addFrame() {
        this.currentFrame = new ContextFrame(currentFrame);
    }

    public void removeFrame() {
        this.currentFrame = currentFrame.prev;
    }

    public void checkNull() {
        if (currentFrame == null) {
            throw new IllegalStateException("there is no current frame");
        }
    }

    void put(String name, Type erased, Type.TypeParamType realType) {
        checkNull();
        currentFrame.put(name, erased, realType);
    }

    public Type getErasedType(String name) {
        checkNull();
        Type result = currentFrame.getErasedType(name);
        if (result == null && parent != null) {
            result = parent.getErasedType(name);
        }
        if (result == null) {
            throw new TypeException("unknown type parameter " + name);
        }
        return result;
    }

    public Type.TypeParamType getUnerasedType(String name) {
        checkNull();
        Type.TypeParamType result = currentFrame.getUnerasedType(name);
        if (result == null && parent != null) {
            result = parent.getUnerasedType(name);
        }
        if (result == null) {
            throw new TypeException("unknown type parameter " + name);
        }
        return result;
    }
}
