/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.util.HashMap;
import java.util.Map;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
class ContextFrame {

    private static class TypePair {

        private final Type erasedType;
        private final Type.TypeParamType realType;

        private TypePair(Type erasedType, Type.TypeParamType realType) {
            this.erasedType = erasedType;
            this.realType = realType;
        }
    }
    final ContextFrame prev;
    private final Map<String, TypePair> erasures = new HashMap<>();

    ContextFrame(ContextFrame prev) {
        this.prev = prev;
    }

    ContextFrame() {
        this.prev = null;
    }

    void put(String name, Type erased, Type.TypeParamType realType) {
        erasures.put(name, new TypePair(erased, realType));
    }

    Type getErasedType(String name) {
        TypePair pair = erasures.get(name);
        Type result = null;
        if (pair != null) {
            result = pair.erasedType;
        }
        if (result == null && prev != null) {
            result = prev.getErasedType(name);
        }
        return result;
    }

    Type.TypeParamType getUnerasedType(String name) {
        TypePair pair = erasures.get(name);
        Type.TypeParamType result = null;
        if (pair != null) {
            result = pair.realType;
        }
        if (result == null && prev != null) {
            result = prev.getUnerasedType(name);
        }
        return result;
    }
}
