/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.util.LinkedList;
import java.util.List;
import teajay.bytecode.cpool.FieldDescriptor;
import teajay.bytecode.cpool.MethodDescriptor;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
class DescriptorVisitor implements Type.Visitor<FieldDescriptor, Void> {

    private MethodDescriptor mdescr;

    public MethodDescriptor getMethodDescriptor() {
        if (mdescr == null) {
            throw new IllegalStateException();
        }
        return mdescr;
    }

    @Override
    public FieldDescriptor visitSimpleType(Type.SimpleType type, Void p) {
        if (type.isPrimitiveType()) {
            return FieldDescriptor.fromString(type.name);
        } else {
            throw new TypeException("expecting erased types");
        }
    }

    @Override
    public FieldDescriptor visitArrayType(Type.ArrayType type, Void p) {
        return FieldDescriptor.makeArray(type.elemType.accept(this, null), 1);
    }

    @Override
    public FieldDescriptor visitMethodType(Type.MethodType type, Void p) {
        FieldDescriptor returnType = type.returnType.accept(this, null);
        List<FieldDescriptor> params = new LinkedList<>();
        if (type.paramTypes != null) {
            for (Type t : type.paramTypes) {
                params.add(t.accept(this, null));
            }
        }
        this.mdescr = new MethodDescriptor(returnType, params);
        return null;
    }

    @Override
    public FieldDescriptor visitClassSigType(Type.ClassSigType type, Void p) {
        throw new TypeException("cannot accept class signature here");
    }

    @Override
    public FieldDescriptor visitClassType(Type.ClassType type, Void p) {
        FieldDescriptor result;
        if (type.outerType != null) {
            FieldDescriptor outerType = type.outerType.accept(this, null);
            result = FieldDescriptor.fromString(outerType.getClassInfo().getClassName() + "$" + type.name);
        } else {
            result = FieldDescriptor.fromString(type.name);
        }
        return result;
    }

    @Override
    public FieldDescriptor visitTypeParamType(Type.TypeParamType type, Void p) {
        throw new TypeException("expecting erased types");
    }

    @Override
    public FieldDescriptor visitWildcardType(Type.WildcardType type, Void p) {
        throw new TypeException("expecting erased types");
    }
}
