/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class JavaArrayVisitor implements Type.Visitor<Type, Void> {

    private final Context context;

    public JavaArrayVisitor(Context context) {
        this.context = context;
    }

    @Override
    public Type visitSimpleType(Type.SimpleType type, Void p) {
        Type t = context.getErasedType(type.name);
        if (t == null) {
            t = TypeUtils.object;
        }
        return t;
    }

    @Override
    public Type visitArrayType(Type.ArrayType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitMethodType(Type.MethodType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitClassSigType(Type.ClassSigType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitClassType(Type.ClassType type, Void p) {
        if (type.outerType != null) {
            System.err.println("Warning: Outer types not allowed for Array");
        }
        if (type.name.equals("teajay/lang/Array")) {
            Type arg = TypeUtils.object;
            if (type.typeArgs != null && type.typeArgs.size() == 1) {
                arg = type.typeArgs.get(0);
            } else if (type.typeArgs != null && !type.typeArgs.isEmpty()) {
                throw new TypeException("cannot convert to array: " + type);
            }
            return new Type.ArrayType(arg.accept(this, null));
        } else {
            if (type.typeArgs != null && !type.typeArgs.isEmpty()) {
                System.err.println("Warning ignoring type arguments");
            }
            return new Type.ArrayType(type);
        }
    }

    @Override
    public Type visitTypeParamType(Type.TypeParamType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitWildcardType(Type.WildcardType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }
}
