/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import java.util.Arrays;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class TJToJavaTypeVisitor implements Type.Visitor<Type, Void> {

    private final Context context;

    public TJToJavaTypeVisitor(Context context) {
        this.context = context;
    }

    @Override
    public Type visitSimpleType(Type.SimpleType type, Void p) {
        Type t = context.getErasedType(type.name);
        if (t == null) {
            t = TypeUtils.object;
        }
        return t;
    }

    @Override
    public Type visitArrayType(Type.ArrayType type, Void p) {
        return new Type.ClassType(null, "teajay/lang/Array", Arrays.asList(type.elemType.accept(this, null)));
    }

    @Override
    public Type visitMethodType(Type.MethodType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitClassSigType(Type.ClassSigType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitClassType(Type.ClassType type, Void p) {
        return type;
    }

    @Override
    public Type visitTypeParamType(Type.TypeParamType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public Type visitWildcardType(Type.WildcardType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }
}
