/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.generics.check;

import teajay.bytecode.cpool.FieldDescriptor;
import teajay.generics.parser.Type;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class TeaJayArrayVisitor implements Type.Visitor<FieldDescriptor, Void> {

    private final TypeUtils utils;

    TeaJayArrayVisitor(TypeUtils utils) {
        this.utils = utils;
    }

    @Override
    public FieldDescriptor visitSimpleType(Type.SimpleType type, Void p) {
        return FieldDescriptor.makeArray(utils.makeFieldDescriptor(type), 1);
    }

    @Override
    public FieldDescriptor visitArrayType(Type.ArrayType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public FieldDescriptor visitMethodType(Type.MethodType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public FieldDescriptor visitClassSigType(Type.ClassSigType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public FieldDescriptor visitClassType(Type.ClassType type, Void p) {
        if (type.outerType != null) {
            System.err.println("Warning: Outer types not allowed for Array");
        }
        if (type.name.equals("teajay/lang/Array")) {
            Type arg = TypeUtils.object;
            if (type.typeArgs != null && type.typeArgs.size() == 1) {
                arg = type.typeArgs.get(0);
            } else if (type.typeArgs != null && !type.typeArgs.isEmpty()) {
                throw new TypeException("cannot convert to array: " + type);
            }
            return FieldDescriptor.makeArray(arg.accept(this, null), 1);
        } else {
            if (type.typeArgs != null && !type.typeArgs.isEmpty()) {
                System.err.println("Warning ignoring type arguments");
            }
            return FieldDescriptor.makeArray(utils.makeFieldDescriptor(type), 1);
        }
    }

    @Override
    public FieldDescriptor visitTypeParamType(Type.TypeParamType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }

    @Override
    public FieldDescriptor visitWildcardType(Type.WildcardType type, Void p) {
        throw new UnsupportedOperationException("Not supported.");
    }
}
