/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util;

import java.lang.reflect.Array;
import teajay.lang.List;
import teajay.lang.TJIterator;
import teajay.lang.TJObject;
import teajay.runtime.TJTypes;
import teajay.util.tjattribute.TeaJayAttribute;

/**
 * Utils for converting between java arrays and TeaJay lists.
 * TODO: Move to teajay.lang.java.
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(accessType = TeaJayAttribute.AccessType.Abstract)
public class ArrayUtils extends TJObject {
    private ArrayUtils() {
    }
    
    public static <T> T[] toArrayUnsafe(List<T> list) {
        T[] arr = (T[]) new Object[list.size().$int_val()];
        TJIterator<T> iter = list.iterator();
        for (int i = 0; i < arr.length; i++) {
            arr[i] = iter.next();
        }
        return arr;
    }

    public static <T> T[] toArray(List<T> list, Class<T> elemType) {
        T[] arr = (T[]) Array.newInstance(elemType, list.size().$int_val());
        TJIterator<T> iter = list.iterator();
        for (int i = 0; i < arr.length; i++) {
            arr[i] = iter.next();
        }
        return arr;
    }

    public static <T> List<T> toList(T[] arr) {
        //DO NOT USE List.<init>(T... values) here!
        List<T> result = TJTypes.instantiate(List.class);
        for (T elem : arr) {
            result.add(elem);
        }
        return result;
    }

}
