/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util.impl;

import java.math.BigInteger;
import teajay.util.tjattribute.TeaJayAttribute;
import teajay.lang.Number;
import teajay.util.RandomGenerator;
import teajay.util.TJType;
/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
@TJType(type = TeaJayAttribute.Type.Impl, accessType = TeaJayAttribute.AccessType.Final)
public final class RandomGeneratorImpl extends RandomGenerator {
    private static final BigInteger longModulus = BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.ONE);
    
    private final java.util.Random rand;
    private final RandomGenerator rthis;
    
    public RandomGeneratorImpl(RandomGenerator rthis) {
        this.rand = new java.util.Random();
        this.rthis = rthis;
    }
    
    public RandomGeneratorImpl(RandomGenerator rthis, Number seed) {
        if (!seed.isInteger()) {
            throw new IllegalArgumentException("Expecting integer seed");
        }
        this.rand = new java.util.Random(seed.$num().mod(longModulus).longValue());
        this.rthis = rthis;
    }
    
    @Override
    public Boolean nextBoolean() {
        return rand.nextBoolean();
    }
    
    @Override
    public Number nextNumber(Number magnitude) {
        return nextFloat().mult(magnitude).mult(nextBoolean() ? Number.ONE : Number.M_ONE);
    }
    
    //only double precision
    @Override
    public Number nextFloat() {
        return Number.$valueOf(rand.nextDouble());
    }
    
    @Override
    public Number nextInteger(Number maxVal) {
        if (!maxVal.isInteger() || maxVal.isNegative() || maxVal.equals(Number.ZERO)) {
            throw new IllegalArgumentException("maxVal must be a positive integer");
        }
        BigInteger num = maxVal.$num();
        int numBytes = num.bitLength() / 8 + (num.bitLength() % 8 == 0 ? 0 : 1);
        byte[] randomBytes = new byte[numBytes == 0 ? 1 : numBytes];
        rand.nextBytes(randomBytes);
        BigInteger result = new BigInteger(randomBytes).mod(num).abs();
        return Number.$valueOf(result, BigInteger.ONE);
    }
    
    @Override
    public void setSeed(Number seed) {
        if (!seed.isInteger()) {
            throw new IllegalArgumentException("Expecting integer seedd");
        }
        rand.setSeed(seed.$num().mod(longModulus).longValue());
    }
}
