/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package teajay.util.tjattribute;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import teajay.bytecode.U4;
import teajay.bytecode.attributes.AttributeInfo;
import teajay.bytecode.cpool.ConstantPool;
import teajay.bytecode.cpool.Utf8Info;
import teajay.bytecode.exceptions.ByteCodeException;
import static teajay.bytecode.util.Tools.ccpt;

/**
 *
 * @author Michael Kußmann <michi@outx.de>
 */
public class TJBridge extends AttributeInfo {
    public static final String NAME = "TJBridge";
    private final Utf8Info signature;
    
    public TJBridge(String signature) {
        this(new Utf8Info(NAME), new Utf8Info(signature));
    }
    
    private TJBridge(Utf8Info name, Utf8Info signature) {
        super(name);
        checkName(NAME, name);
        if (signature == null) {
            throw new NullPointerException("signature");
        }
        this.signature = signature;
    }
    
    public String getSignature() {
        return signature.getValue();
    }
    
    @Override
    public U4 getLength(ConstantPool cpool) throws ByteCodeException {
        return new U4(signature.getDefaultLengthAsOperand().size);
    }
    
    @Override
    public void write(DataOutputStream out, ConstantPool cpool) throws IOException, ByteCodeException {
        super.write(out, cpool);
        signature.writeIndex(out, cpool);
    }
    
    @Override
    public void add(ConstantPool cpool) throws ByteCodeException {
        super.add(cpool);
        signature.add(cpool);
    }
   
    public static TJBridge read(DataInputStream in, ConstantPool cpool, int major, int minor, int length, Utf8Info name) throws ByteCodeException, IOException {
        Utf8Info signature = ccpt(cpool.get(in.readUnsignedShort()), Utf8Info.class, cpool);
        return new TJBridge(name, signature);
    }
    
}
